#!/usr/bin/ruby -w
# Encoding: UTF-8
# frozen_string_literal: true
# =========================================================================== #
# === MultimediaParadise::AudioPlayer
#
# This is an "audio player".
#
# In other words - it plays audio files.
#
# For this to work, it distinguishes between different playmodes as will
# be explained here.
#
# - If the argument given is an existing audio file, we will play it, then
#   exit.
#
# - If we pass an array of files, we will store this array in our playlist
#   and play all of them before we will then exit.
#
# - If we omit any argument then we will default to a neverending 'loop'.
#   This also is the case when we pass 'loop' or nil as argument. Note
#   that the loop is not unlimited; it will honour a threshold determined
#   by an instance variable.
#
# - If we pass 'once', then we will play all audio files, and
#   then exit.
#
# - If we pass 'one' or 'rand' as argument, we will play one
#   random song, only ONCE, and then quit.
#
# - If we pass 'one_song', then we will play only this song.
#
# - If we pass in a number such as '3' then we will assume
#   that the user wants to play exactly 3 files.
#
# Usage example:
#
#   MultimediaParadise::AudioPlayer.new(ARGV)
#
# =========================================================================== #
# require 'multimedia_paradise/audio/audio_player.rb'
# =========================================================================== #
require 'multimedia_paradise/audio/base.rb'
require 'multimedia_paradise/audio/audio_player/constants.rb'
require 'multimedia_paradise/audio/audio_player/menu.rb'
require 'multimedia_paradise/audio/audio_player/misc.rb'
require 'multimedia_paradise/audio/audio_player/reset.rb'
require 'multimedia_paradise/audio/audio_player/tab.rb'

module MultimediaParadise

class AudioPlayer < AudioBase # === MultimediaParadise::AudioPlayer

  require 'multimedia_paradise/multimedia/play_from_this_list.rb'

end

  # ========================================================================= #
  # === MultimediaParadise.audio_player
  #
  # We also want to use a simpler module-method for the audio player.
  # ========================================================================= #
  def self.audio_player(options = nil)
    MultimediaParadise::AudioPlayer.new(options)
  end; self.instance_eval { alias audio_play audio_player } # === MultimediaParadise.audio_play
       self.instance_eval { alias play       audio_player } # === MultimediaParadise.play
       self.instance_eval { alias []         audio_player } # === MultimediaParadise[]

end

if __FILE__ == $PROGRAM_NAME
  _ = MultimediaParadise::AudioPlayer.new(ARGV, :dont_run_yet)
  if ARGV[1]
    _.try_to_find_genre_song(ARGV[1]) # To test this, try: rsong genre GOOD_BDANCE_SONGS
  end
  _.run
end # rsong power
    #
    # If you do this:
    #   rsong ELE
    # We load all ELE songs in our playlist
    #   rsong playlist
    # Will play our playlist.