#!/usr/bin/ruby -w
# Encoding: UTF-8
# frozen_string_literal: true
# =========================================================================== #
# require 'multimedia_paradise/audio/audio_player/constants.rb'
# =========================================================================== #
require 'multimedia_paradise/audio/base.rb'

module MultimediaParadise

class AudioPlayer < AudioBase # === MultimediaParadise::AudioPlayer

  require 'multimedia_paradise/project/project.rb'

  begin
    require 'x/audio/playlist_profile.rb' # PlaylistProfile is declared here.
  rescue LoadError; end # bl $RUBY_AUD/playlist_profile.rb

  # ========================================================================= #
  # === NAMESPACE
  # ========================================================================= #
  NAMESPACE = inspect

  # ========================================================================= #
  # === MY_SONGS
  # ========================================================================= #
  MY_SONGS = AUDIO_DIRECTORY
    AUDIO_DIR = MY_SONGS # === AUDIO_DIR

  # ========================================================================= #
  # === RENAME_KDE_KONSOLE_TAB
  # ========================================================================= #
  RENAME_KDE_KONSOLE_TAB = true

  # ========================================================================= #
  # === DEFAULT_DELAY_PLAYING_SONGS
  #
  # The following constant is used for determining the audio delay between
  # two songs.
  #
  # A value of 0.0 would mean no delay, aka "instant play", logically. 
  # ========================================================================= #
  DEFAULT_DELAY_PLAYING_SONGS = 1.8

  # ========================================================================= #
  # === MAIN_AUDIO_PLAYER_TO_USE
  #
  # The entry of this constant will become the instance variable
  # @audio_player_to_use.
  #
  # As of May 2015 we can dynamically set this.
  # ========================================================================= #
  _ = "#{::MultimediaParadise.project_yaml_dir?}video/video.yml"
  if File.exist? _
    _ = YAML.load_file(_)
    if _.has_key? 'MAIN_VIDEO_PLAYER'
      MAIN_AUDIO_PLAYER_TO_USE = _['MAIN_VIDEO_PLAYER']
    end
  else # We default to mpv since as of 2021.
    MAIN_AUDIO_PLAYER_TO_USE = 'mpv' # mplayer'
  end

  # ========================================================================= #
  # === LAST_SONG_PLAYED
  # ========================================================================= #
  LAST_SONG_PLAYED = 'LAST_SONG_PLAYED.md'

  # ========================================================================= #
  # === USE_OSS_FOR_AUDIO_PLAYER
  # ========================================================================= #
  USE_OSS_FOR_AUDIO_PLAYER = false

  # ========================================================================= #
  # === FULL_LOCATION_FOR_LAST_SONG_PLAYED
  # ========================================================================= #
  FULL_LOCATION_FOR_LAST_SONG_PLAYED = HOME+LAST_SONG_PLAYED # Specify the path to our file here.

  # ========================================================================= #
  # === MAXIMUM_ALLOWED_FILE_SIZE
  # ========================================================================= # 
  MAXIMUM_ALLOWED_FILE_SIZE = 1_350_000 # Shall not become larger than that, else we remove the file "last song played".

  # ========================================================================= #
  # === DEFAULT_PLAYMODE
  # ========================================================================= #
  DEFAULT_PLAYMODE = 'once'

  # ========================================================================= #
  # === NAME_OF_AUDIO_PLAYER
  # ========================================================================= #
  NAME_OF_AUDIO_PLAYER = 'AudioPlayer' # The name of the audio player.

  # ========================================================================= #
  # === PLAY_MAX_N_SONGS
  # ========================================================================= #
  PLAY_MAX_N_SONGS = 1250 # We won't play more than n songs.

  # ========================================================================= #
  # === USE_FAVOURITE_PLAYLIST
  # ========================================================================= #
  USE_FAVOURITE_PLAYLIST = false # If true we play only songs from this list

  # ========================================================================= #
  # === AVAILABLE_SONGS
  #
  # Obtain all available songs - and shuffle them a bit.
  # ========================================================================= #
  AVAILABLE_SONGS = Dir["#{MY_SONGS}*"].shuffle

  # ========================================================================= #
  # === SHALL_WE_DEBUG
  # ========================================================================= #
  SHALL_WE_DEBUG = false # If true we output debug info

  # ========================================================================= #
  # === PLAYLIST_CONSTANTS
  #
  # Whether we have the PlaylistProfile defined or not. If we have, we will
  # make use of it.
  # ========================================================================= #
  if Object.const_defined? :PlaylistProfile
    PLAYLIST_CONSTANTS = PlaylistProfile.constants
  end

  # ========================================================================= #
  # === THE_LOG_FILE_IS_STORED_HERE
  # ========================================================================= #
  THE_LOG_FILE_IS_STORED_HERE =
    "#{MultimediaParadise.log_dir?}"\
    "log_file_for_played_songs_via_the_audio_player.md"

end; end

if __FILE__ == $PROGRAM_NAME
  pp MultimediaParadise::AudioPlayer
end