#!/usr/bin/ruby -w
# Encoding: UTF-8
# frozen_string_literal: true
# =========================================================================== #
module MultimediaParadise

class AudioPlayer < AudioBase

  # ========================================================================= #
  # === menu                                                       (menu tag)
  # ========================================================================= #
  def menu(i)
    if i.is_a? Array
      if i.empty?
        i = 'loop'
      else
        i = i.join(' ').strip
      end
    end
    i = i.to_s.strip
    if i =~ /^\d+$/ # If input consists of only numbers.
      opnn; e 'We will next play '+simp(i.to_s)+' songs.'
      i.to_i.times { add_random_song }
    # ======================================================================= #
    # === 11:22:33
    #
    # Handle time-formats next.
    # ======================================================================= #
    elsif i =~ /^\d{1,2}:\d{1,2}:\d{1,2}$/ # If input consists of only numbers and ':'.
      # ===================================================================== #
      # User did input a time format such as "11:22:33". We will assume this
      # to mean to enter idle mode, and then play the songs via a loop.
      # ===================================================================== #
      begin
        require 'roebe/classes/at.rb'
        Roebe::At.new(i)
        do_play_in_a_loop # We play in a loop in this case.
      rescue LoadError
        e 'The file roebe/classes/at.rb is not available.'
      end
    else
      case i # case tag
      # ===================================================================== #
      # === help
      #
      # This entry point can be used to show the help section.
      # ===================================================================== #
      when *ARRAY_HELP_OPTIONS
        show_help :then_exit
      # ===================================================================== #
      # === rsong --last-songs?
      # ===================================================================== #
      when /^-?-?last(-|_)?songs\??$/i
        show_the_last_songs_that_were_played
        exit_program { :be_quiet }
      # ===================================================================== #
      # === --use-sox
      # ===================================================================== #
      when /^-?-?use(-|_)?sox/
        determine_which_audio_player_to_use(
          :sox, :ignore_environment_variable
        )
      # ===================================================================== #
      # === --use-mplayer
      # ===================================================================== #
      when /^-?-?use(-|_)?mplayer/
        determine_which_audio_player_to_use(
          :mplayer, :ignore_environment_variable
        )
      # ===================================================================== #
      # === rsong --log?
      # ===================================================================== #
      when /^-?-?log\??$/i
        show_the_log_file_if_it_exists
        exit_program(:force_exit_but_be_quiet)
      # ===================================================================== #
      # === OPEN
      # ===================================================================== #
      when /^-?-?OPEN$/i
        esystem "bluefish #{__FILE__}"
        exit_program(:force_exit)
      # ===================================================================== #
      # === rsong --player?
      # ===================================================================== #
      when /^-?-?player\??$/
        report_which_audio_player_to_use
        exit_program { :be_quiet }
      # ===================================================================== #
      # === random
      # ===================================================================== #
      when 'random',
           'return_random_song',
           'rand',
           'one' # Only add one song.
        add_random_song
      # ===================================================================== #
      # === chill
      # ===================================================================== #
      when 'list','chill','1'
        if Object.const_defined? :PlaylistProfile
          these_songs = PlaylistProfile::GOOD_CHILL_SONGS.map {|entry|
            audio_dir?+entry
          }
          add(these_songs.shuffle)
        end
      # ===================================================================== #
      # === playlist
      # ===================================================================== #
      when /playlist\??/i
        try_to_play_playlist
      # ===================================================================== #
      # === last?
      # ===================================================================== #
      when 'last?'
        show_last_file_played
      # ===================================================================== #
      # === once
      # ===================================================================== #
      when 'once','noloop','default','all', # This means we play every song exactly once, then quit.
           '','true','false',
           'all_songs',
           'play_all_songs'
        opnn; e 'We will play all available Audio files once.'
        add_all_songs
      # ===================================================================== #
      # === dance
      # ===================================================================== #
      when 'dance','bdance','breakdance' # Im Moment hardcoded auf bdance
        these_songs = PlaylistProfile::GOOD_BDANCE_SONGS.shuffle.map {|entry|
          audio_dir?+entry
        }
        add(these_songs.shuffle)
      # ===================================================================== #
      # === loop
      # ===================================================================== #
      when 'loop','l','nil'
        # Notify the user what we are doing next.
        opnn; e "#{main_colour?}Playing in a neverending #{sfancy('loop')}"\
                "#{main_colour?} next:"
        do_play_in_a_loop
      # ===================================================================== #
      # === genre
      # ===================================================================== #
      when 'genre','GENRES','genres','GENRE'
        unless ARGV[1]
          show_available_genres
          exit_program(:force_exit)
        end
      else # else tag
        # =================================================================== #
        # Grab all entries from the current directory that can match to the
        # input.
        # =================================================================== #
        entries = Dir['*'+i+'*']
        unless entries.empty?
          add_songs(entries)
        else
          try_to_find_genre_song(i)
        end
      end
    end
  end; alias check_against_menu menu # === check_against_menu

end; end