#!/usr/bin/ruby -w
# Encoding: UTF-8
# frozen_string_literal: true
# =========================================================================== #
# === MultimediaParadise::Compress
#
# This class will, right now, compress .mp3 files via lame.
#
# It will typically use a commandline variant such as 'lame -b 32'.
# =========================================================================== #
# require 'multimedia_paradise/audio/compress.rb'
# =========================================================================== #
require 'multimedia_paradise/audio/base.rb'

module MultimediaParadise

class Compress < ::MultimediaParadise::AudioBase

  # ========================================================================= #
  # === initialize
  #
  # The first argument should be our target file.
  # ========================================================================= #
  def initialize(
      i, run_already = true
    )
    reset
    set_input(i)
    run if run_already
  end

  # ========================================================================= #
  # === reset
  # ========================================================================= #
  def reset
    super()
  end

  # ========================================================================= #
  # === set_input
  # ========================================================================= #
  def set_input(i)
    i = i.join(' ').strip if i.is_a? Array
    @input = i
  end

  # ========================================================================= #
  # === input?
  # ========================================================================= #
  def input?
    @input
  end

  # ========================================================================= #
  # === determine_output_filename
  # ========================================================================= #
  def determine_output_filename
    "output_file_#{input?.sub(/\.mp3$/i,'')}.mp3"
  end

  # ========================================================================= #
  # === cmd?
  #
  # The main command to run. We will use the bitrate -b 32 here.
  #
  # We could also use ffmpeg here:
  #
  #   ffmpeg -i input.mp3 -ab 96 output.mp3
  #
  # ========================================================================= #
  def cmd?
    'lame -b 32 '+input?+' '+determine_output_filename
  end

  # ========================================================================= #
  # === run
  # ========================================================================= #
  def run
    _ = cmd?
    e sfancy(_) # Colourize it.
    system(_)
  end

end

# =========================================================================== #
# === MultimediaParadise.compress_this_audio_file
# =========================================================================== #
def self.compress_this_audio_file(this_audio_file)
  MultimediaParadise::Compress.new(this_audio_file)
end

end

if __FILE__ == $PROGRAM_NAME
  MultimediaParadise::Compress.new(ARGV)  
end # compress_audio