#!/usr/bin/ruby -w
# Encoding: UTF-8
# frozen_string_literal: true
# =========================================================================== #
# === MultimediaParadise::MergeAudioFiles
#
# This class can be used to merge together several audio files.
#
# It can also be used to scan through a directory, gather all audio
# files from there, and merge these together into a single (new)
# file. This will be the default modus operandi if no argument
# is provided otherwise.
#
# Usage example:
#
#   MultimediaParadise::MergeAudioFiles.new(ARGV)
#
# =========================================================================== #
# require 'multimedia_paradise/audio/merge_audio_files.rb'
# =========================================================================== #
require 'multimedia_paradise/audio/base.rb'
require 'multimedia_paradise/multimedia/merge_multimedia.rb'

module MultimediaParadise

class MergeAudioFiles < ::MultimediaParadise::AudioBase # === MultimediaParadise::MergeAudioFiles

  # ========================================================================= #
  # === NAMESPACE
  # ========================================================================= #
  NAMESPACE = inspect

  # ========================================================================= #
  # === initialize
  # ========================================================================= #
  def initialize(
      commandline_arguments = nil,
      run_already           = true
    )
    reset
    set_commandline_arguments(
      commandline_arguments
    )
    run if run_already
  end

  # ========================================================================= #
  # === reset                                                     (reset tag)
  # ========================================================================= #
  def reset
    super()
    # ======================================================================= #
    # === @namespace
    # ======================================================================= #
    @namespace = NAMESPACE
    # ======================================================================= #
    # === @array_merge_these_files
    # ======================================================================= #
    @array_merge_these_files = []
  end

  # ========================================================================= #
  # === set_input
  # ========================================================================= #
  def set_commandline_arguments(i = '')
    i = [i].flatten.compact
    @commandline_arguments = i
  end

  # ========================================================================= #
  # === commandline_arguments?
  # ========================================================================= #
  def commandline_arguments?
    @commandline_arguments
  end

  # ========================================================================= #
  # === add
  #
  # This method can be used to add files that are to be merged together.
  # ========================================================================= #
  def add(this_file)
    @array_merge_these_files << this_file
    @array_merge_these_files.flatten!
  end

  # ========================================================================= #
  # === determine_which_files_are_to_be_merged
  # ========================================================================= #
  def determine_which_files_are_to_be_merged
    _ = @commandline_arguments.select {|entry|
      File.exist?(entry) and is_an_audio_file?(entry)
    }
    # ======================================================================= #
    # Next, we have to find audio files if the above variable assigned to
    # _ is empty.
    # ======================================================================= #
    if _.empty?
      _ = Dir['**/**'].select {|entry| is_an_audio_file?(entry) }
      unless _.empty?
        opnn; e 'No specific argument was provided. We will thus '+steelblue('scan for')
        opnn; e steelblue('all audio files in this directory and all subdirectories')+
                ' to it'
        opnn; e 'and then merge these together into a new file.'
      end
    end
    add(_)
  end

  # ========================================================================= #
  # === run                                                         (run tag)
  # ========================================================================= #
  def run
    determine_which_files_are_to_be_merged
    result = MultimediaParadise::MergeMultimedia.new(
      @array_merge_these_files
    )
    output_file = result.output_file?
    if File.exist? output_file
      opnn; e "Merged into the file #{sfile(output_file)}."
    end
  end

  # ========================================================================= #
  # === MultimediaParadise::MergeAudioFiles[]
  # ========================================================================= #
  def self.[](i = '')
    new(i)
  end

end; end

if __FILE__ == $PROGRAM_NAME
  MultimediaParadise::MergeAudioFiles.new(ARGV)
end # merge_audio_files
    # merge_audio_files a.mp3 b.mp3 c.mp3