#!/usr/bin/ruby -w
# Encoding: UTF-8
# frozen_string_literal: true
# =========================================================================== #
# === MultimediaParadise::Audio::ModifyYearOfAudioFile
#
# This will modify the year of the .mp3 file in question.
#
# Usage example:
#
#   MultimediaParadise::Audio::ModifyYearOfAudioFile.new(ARGV)
#
# =========================================================================== #
# require 'multimedia_paradise/audio/modify_year_of_audio_file.rb'
# =========================================================================== #
require 'multimedia_paradise/audio/base.rb'

module MultimediaParadise

module Audio

class ModifyYearOfAudioFile < ::MultimediaParadise::AudioBase # === MultimediaParadise::Audio::ModifyYearOfAudioFile

  # ========================================================================= #
  # === NAMESPACE
  # ========================================================================= #
  NAMESPACE = inspect

  # ========================================================================= #
  # === initialize
  # ========================================================================= #
  def initialize(
      i           = nil,
      run_already = true
    )
    reset
    set_commandline_arguments(i)
    case run_already
    when :dont_run_yet,
         :do_not_run_yet
      run_already = false
    end
    run if run_already
  end

  # ========================================================================= #
  # === reset                                                     (reset tag)
  # ========================================================================= #
  def reset
    super()
    # ======================================================================= #
    # === @namespace
    # ======================================================================= #
    @namespace = NAMESPACE
  end

  # ========================================================================= #
  # === set_commandline_arguments
  # ========================================================================= #
  def set_commandline_arguments(i = [])
    @commandline_arguments = [i].flatten.compact
  end

  # ========================================================================= #
  # === check_commandline
  # ========================================================================= #
  def check_commandline
    if @commandline_arguments.empty?
      opnn; e 'Please provide to what year you want to set this.'
      e
      e 'Example:'
      e
      e '  --year 2005'
      e
    else
      determine_the_year
      modify_the_year
    end
  end

  # ========================================================================= #
  # === determine_the_year
  # ========================================================================= #
  def determine_the_year
    set_year(
      @commandline_arguments[1]
    )
  end

  # ========================================================================= #
  # === modify_the_year
  # ========================================================================= #
  def modify_the_year
    name_of_the_file = @commandline_arguments.first
    if File.exist? name_of_the_file
      _ = "mid3v2 --year=#{year?} #{name_of_the_file}"
      opnn; e "Now modifying the file `#{sfile(name_of_the_file)}`."
      esystem _
    else
      opnn; no_file_exists_at(name_of_the_file)
    end
  end

  # ========================================================================= #
  # === set_this_year
  # ========================================================================= #
  def set_this_year(i = nil)
    @this_year = i
  end; alias set_year set_this_year # === set_year

  # ========================================================================= #
  # === this_year?
  # ========================================================================= #
  def this_year?
    @this_year
  end; alias this_year this_year? # === this_year
       alias year?     this_year? # === year?

  # ========================================================================= #
  # === run                                                         (run tag)
  # ========================================================================= #
  def run
    check_commandline
  end

  # ========================================================================= #
  # === MultimediaParadise::Audio::ModifyYearOfAudioFile[]
  # ========================================================================= #
  def self.[](i = '')
    new(i)
  end

end; end

# =========================================================================== #
# === MultimediaParadise.modify_year_of_audio_file
#
# Invoke this like so:
#
#   MultimediaParadise.modify_year_of_audio_file /Depot/j/foobar.mp3 2012
#
# =========================================================================== #
def self.modify_year_of_audio_file(*i)
  MultimediaParadise::Audio::ModifyYearOfAudioFile.new(i)
end

end

if __FILE__ == $PROGRAM_NAME
  MultimediaParadise::Audio::ModifyYearOfAudioFile.new(ARGV)
end # modify_year_of_audio_file /Depot/j/foobar.mp3 2012