#!/usr/bin/ruby -w
# Encoding: UTF-8
# frozen_string_literal: true
# =========================================================================== #
# === MultimediaParadise::Audio::NAudioSongs
#
# This class will simply update how many songs I have stored on the local
# HDD.
#
# Usage example:
#
#   MultimediaParadise::Audio::NAudioSongs.new
#
# =========================================================================== #
# require 'multimedia_paradise/audio/n_audio_songs.rb'
# =========================================================================== #
require 'multimedia_paradise/audio/base.rb'

module MultimediaParadise

module Audio

class NAudioSongs < ::MultimediaParadise::AudioBase # === MultimediaParadise::Audio::NAudioSongs

  require 'date'

  # ========================================================================= #
  # === NAMESPACE
  # ========================================================================= #
  NAMESPACE = inspect

  # ========================================================================= #
  # === STORE_HERE
  # ========================================================================= #
  STORE_HERE = "#{AUDIO_DIRECTORY}README.md"

  # ========================================================================= #
  # === AUDIO_DIR
  # ========================================================================= #
  AUDIO_DIR = ENV['MY_SONGS'].to_s

  # ========================================================================= #
  # === ARRAY_AUDIO_FILES
  # ========================================================================= #
  ARRAY_AUDIO_FILES = %w(
    mp3 ogg
  )

  # ========================================================================= #
  # === ALSO_STORE_FULL_PATHNAME_TO_THE_SONGS
  #
  # If the following constant is true, then we will also append all songs
  # via comment.
  # ========================================================================= #
  ALSO_STORE_FULL_PATHNAME_TO_THE_SONGS = true

  # ========================================================================= #
  # === initialize
  # ========================================================================= #
  def initialize(
      i           = nil,
      run_already = true
    )
    reset
    set_input(i)
    run if run_already
  end

  # ========================================================================= #
  # === reset                                                     (reset tag)
  # ========================================================================= #
  def reset
    super()
    # ======================================================================= #
    # === @namespace
    # ======================================================================= #
    @namespace = NAMESPACE
    # ======================================================================= #
    # === @n_songs
    # ======================================================================= #
    @n_songs = 0
    # ======================================================================= #
    # === @string_to_store
    # ======================================================================= #
    @string_to_store = ''.dup
  end

  # ========================================================================= #
  # === set_input
  # ========================================================================= #
  def set_input(i = '')
    i = i.first if i.is_a? Array
    i = i.to_s.dup
    @input = i
  end

  # ========================================================================= #
  # === input?
  # ========================================================================= #
  def input?
    @input
  end

  # ========================================================================= #
  # === determine_how_many_songs_we_have
  # ========================================================================= #
  def determine_how_many_songs_we_have
    files = Dir["#{AUDIO_DIR}/*"]
    @n_songs = files.reject {|entry| ! is_audio_song?(entry) }
    return @n_songs
  end

  # ========================================================================= #
  # === n_songs_available
  # ========================================================================= #
  def n_songs_available
    @n_songs.size.to_s
  end

  # ========================================================================= #
  # === all_songs?
  #
  # This method will return all songs, as Array.
  # ========================================================================= #
  def all_songs?
    all_local_audio_songs = Dir["#{directory_to_the_local_songs?}*"].select {|entry| is_audio_song?(entry) }
    all_local_audio_songs = sort_array_alphabetically(
      all_local_audio_songs
    )
    return all_local_audio_songs
  end

  # ========================================================================= #
  # === sort_array_alphabetically
  # ========================================================================= #
  def sort_array_alphabetically(array)
    return array.sort_by {|entry|
      File.basename(entry)
    }
  end

  # ========================================================================= #
  # === do_save_all_the_necessary_parts
  # ========================================================================= #
  def do_save_all_the_necessary_parts
    if ALSO_STORE_FULL_PATHNAME_TO_THE_SONGS
      append ''
      append ''
      append '#'
      _ = ''.dup
      all_songs?.each {|this_song|
        _ << "# #{this_song}#{N}"
      }
      append _.chomp
      append '#'
    end
    save_file(@string_to_store, STORE_HERE)
  end

  # ========================================================================= #
  # === notify_the_user_what_will_be_done
  # ========================================================================= #
  def notify_the_user_what_will_be_done
    opnn; e "Now storing into #{sfile(STORE_HERE)} that there are"
    opnn; e sfancy(n_songs_available)+' songs available.'
  end

  # ========================================================================= #
  # === append                                                   (append tag)
  # ========================================================================= #
  def append(this)
    case this
    when :comment
      this = '# '+('=' * 76)+' #'
    end
    @string_to_store << "#{this}#{N}" # Has a mandatory newline.
  end

  # ========================================================================= #
  # === assemble_string_to_store
  # ========================================================================= #
  def assemble_string_to_store
    @string_to_store = "#{n_songs_available} Songs - #{get_date}".dup
  end

  # ========================================================================= #
  # === get_date
  # ========================================================================= #
  def get_date # This will return something like 'Jul 2013'
    Time.now.strftime('%d.%m.%Y')+
    ' ('+Time.now.strftime('%d')+' '+
    Date::MONTHNAMES[Date.today.month][0,3]+' '+Date.today.year.to_s+')'
  end

  # ========================================================================= #
  # === run                                                         (run tag)
  # ========================================================================= #
  def run
    determine_how_many_songs_we_have
    assemble_string_to_store
    notify_the_user_what_will_be_done
    do_save_all_the_necessary_parts
  end

  # ========================================================================= #
  # === MultimediaParadise::Audio::NAudioSongs[]
  # ========================================================================= #
  def self.[](i = '')
    new(i)
  end

end; end; end

if __FILE__ == $PROGRAM_NAME
  MultimediaParadise::Audio::NAudioSongs.new(ARGV)
end # naudiosongs