#!/usr/bin/ruby -w
# Encoding: UTF-8
# frozen_string_literal: true
# =========================================================================== #
# === MultimediaParadise::PlayAllAudioFiles
#
# Use this class to play all audio files from the target directory.
#
# The target directory, in this context, defaults to the current
# working directory by default.
# =========================================================================== #
require 'multimedia_paradise/audio/base.rb'

module MultimediaParadise

class PlayAllAudioFiles < ::MultimediaParadise::AudioBase

  # ========================================================================= #
  # === IGNORE_FILES_THAT_HAVE_THE_SUBSTRING_DONE
  # ========================================================================= #
  IGNORE_FILES_THAT_HAVE_THE_SUBSTRING_DONE = true

  # ========================================================================= #
  # === initialize
  #
  # The first argument is the directory from which we will obtain the
  # audio files from.s
  # ========================================================================= #
  def initialize(
      from_this_directory = Dir.pwd,
      run_already         = true
    )
    reset
    set_this_directory(
      from_this_directory
    )
    run if run_already
  end

  # ========================================================================= #
  # === reset
  # ========================================================================= #
  def reset
    super()
    # ======================================================================= #
    # === @audio_files
    # ======================================================================= #
    @audio_files = nil
    # ======================================================================= #
    # === @use_colours
    # ======================================================================= #
    @use_colours = true
  end

  # ========================================================================= #
  # === set_this_directory
  # ========================================================================= #
  def set_this_directory(i = ARGV)
    i = i.first if i.is_a? Array
    i = Dir.pwd if i.nil?
    i << '/' unless i.end_with? '/'
    @this_directory = i
  end

  # ========================================================================= #
  # === obtain_all_audio_files
  # ========================================================================= #
  def obtain_all_audio_files
    @audio_files = Dir['*'].select {|file|
      is_audio_file?(file)
    }
    if IGNORE_FILES_THAT_HAVE_THE_SUBSTRING_DONE # Ignore files with the "DONE_" substring.
      @audio_files.reject! {|file|
        file.include? 'DONE_' 
      }
    end
  end

  # ========================================================================= #
  # === shuffle_audio_files
  # ========================================================================= #
  def shuffle_audio_files
    @audio_files.shuffle!
  end

  # ========================================================================= #
  # === play_this_audio_file(file)
  # ========================================================================= #
  def play_this_audio_file(file)
    use_this_command = 'mplayer '+file
    cliner { e orange(use_this_command) }
    system use_this_command
  end

  # ========================================================================= #
  # === play_audio_files
  #
  # Use this method to play the audio files.
  # ========================================================================= #
  def play_audio_files
    @audio_files.each {|file|
      e
      e sfile(file)
      e
      play_this_audio_file(file)
    }
  end

  # ========================================================================= #
  # === run
  # ========================================================================= #
  def run
    obtain_all_audio_files
    shuffle_audio_files
    play_audio_files
  end

end; end

if __FILE__ == $PROGRAM_NAME
  MultimediaParadise::PlayAllAudioFiles.new(ARGV)  
end # play_all_audio_files