#!/usr/bin/ruby -w
# Encoding: UTF-8
# frozen_string_literal: true
# =========================================================================== #
# === MultimediaParadise::RemoveAudio
#
# This small module can be used to remove audio from a given file.
#
# Specific usage example:
#
#   MultimediaParadise::RemoveAudio[]
#
# =========================================================================== #
module MultimediaParadise

module RemoveAudio # == MultimediaParadise::RemoveAudio[]

  # ========================================================================= #
  # === remove_audio_from
  #
  # Use this method to remove audio from a given file.
  #
  # Usage:
  #
  #   remove_audio_from('/Depot/jjjj/264_PlanetOfTheApes.avi')
  #
  # ========================================================================= #
  def remove_audio_from(this_file)
    RemoveAudio.remove_audio_from(this_file)
  end

  # ========================================================================= #
  # === RemoveAudio.remove_audio_from
  #
  # Use this method to remove the audio from a multimedia file.
  #
  # We depend on ffmpeg for this task.
  # ========================================================================= #
  def self.remove_audio_from(this_file, &block)
    # ======================================================================= #
    # Handle arrays first.
    # ======================================================================= #
    this_file = this_file.first if this_file.is_a? Array
    # Next determine the output filename.
    output_file_name = (Dir.pwd+'/output_'+File.basename(this_file)).squeeze '/'
    _ = "ffmpeg -i #{this_file} -an -vcodec copy ".dup
    # ======================================================================= #
    # Next, any block-argument given can still modify the commandline
    # arguments used here.
    # ======================================================================= #
    if block_given?
      yielded = yield
      case yielded
      # ===================================================================== #
      # === :always_overwrite_existing_files
      #
      # This will simply overwrite any existing file.
      # ===================================================================== #
      when :always_overwrite_existing_files
        _ << "-y "
      end
    end
    _ << output_file_name
    puts _
    system _
    return output_file_name
  end

  # ========================================================================= #
  # === MultimediaParadise::RemoveAudio[]
  # ========================================================================= #
  def self.[](i)
    remove_audio_from(i)
  end

end

# =========================================================================== #
# === MultimediaParadise.remove_audio
#
# Also add a simpler API to remove audio. The input argument to this
# method should be a local file (typically a video-file) from which
# you wish to remove the audio.
#
# Usage example:
#
#   MultimediaParadise.remove_audio '/Depot/jj/StarTrek_Nemesis.mp4'
#
# =========================================================================== #
def self.remove_audio(i, &block)
  RemoveAudio.remove_audio_from(i, &block)
end

end

if __FILE__ == $PROGRAM_NAME
  MultimediaParadise::RemoveAudio[ARGV]
end # remove_audio