#!/usr/bin/ruby -w
# Encoding: UTF-8
# frozen_string_literal: true
# =========================================================================== #
# === MultimediaParadise::RemoveLastSecond
#
# This class can remove the last second of an audio file.
#
# Specific usage example:
#
#   MultimediaParadise::RemoveLastSecond.new(ARGV)
#
# =========================================================================== #
# require 'multimedia_paradise/audio/remove_last_second.rb'
# MultimediaParadise.remove_last_second(ARGV)
# =========================================================================== #
require 'multimedia_paradise/audio/base.rb'
require 'multimedia_paradise/audio/file_duration/file_duration.rb'

module MultimediaParadise

class RemoveLastSecond < ::MultimediaParadise::AudioBase

  # ========================================================================= #
  # === initialize
  # ========================================================================= #
  def initialize(
      i           = ARGV,
      run_already = true
    )
    reset
    set_commandline_arguments(i)
    run if run_already
  end

  # ========================================================================= #
  # === reset
  # ========================================================================= #
  def reset
    super()
  end

  # ========================================================================= #
  # === return_timestamp_minus_one_second
  # ========================================================================= #
  def return_timestamp_minus_one_second(i)
    begin
      require 'roebe/classes/twentyfour_hours_notation.rb'
    rescue LoadError; end
    n_seconds_new = ::MultimediaParadise.duration?(i) - 1
    return Roebe::TwentyfourHoursNotation[n_seconds_new]
  end

  # ========================================================================= #
  # === run
  # ========================================================================= #
  def run
    _ = first?
    output_filename = 'OUTPUT_'+_
    cmd = 'ffmpeg -ss 00:00:00'\
          ' -t '+return_timestamp_minus_one_second(_)+
          ' -i '+_+
          ' -acodec copy '+
          output_filename
    e
    esystem cmd
    e
  end

end

# =========================================================================== #
# === MultimediaParadise.remove_last_second
# =========================================================================== #
def self.remove_last_second(this_audio_file)
  MultimediaParadise::RemoveLastSecond.new(this_audio_file)
end

end

if __FILE__ == $PROGRAM_NAME
  MultimediaParadise::RemoveLastSecond.new(ARGV)
end # remove_last_second