#!/usr/bin/ruby -w
# Encoding: UTF-8
# frozen_string_literal: true
# =========================================================================== #
# === MultimediaParadise::Audio::ToMp3
#
# This class can convert the given input to a .mp3 file. The input is
# assumed to be an audio-file.
# =========================================================================== #
# require 'multimedia_paradise/audio/to_mp3.rb'
# =========================================================================== #
require 'multimedia_paradise/audio/base.rb'

module MultimediaParadise

module Audio

class ToMp3 < MultimediaParadise::AudioBase # === MultimediaParadise::Audio::ToMp3

  # ========================================================================= #
  # === BITRATE_TO_USE
  # ========================================================================= #
  BITRATE_TO_USE = '128k'

  # ========================================================================= #
  # === initialize
  # ========================================================================= #
  def initialize(*i)
    reset
    set_files(i)
    run
  end

  # ========================================================================= #
  # === reset
  # ========================================================================= #
  def reset
    super()
    # ======================================================================= #
    # === @prepend_this_string_to_the_filename
    # ======================================================================= #
    @prepend_this_string_to_the_filename = 'output_'
  end

  # ========================================================================= #
  # === filter_away_commandline_arguments_from_this_input
  #
  # We assume the input to this method will be an Array.
  # ========================================================================= #
  def filter_away_commandline_arguments_from_this_input(i)
    selection = i.select {|entry| entry.start_with? '--' }
    set_commandline_arguments(selection)
    return i.reject {|entry| entry.start_with? '--' }
  end

  # ========================================================================= #
  # === set_files
  # ========================================================================= #
  def set_files(i)
    i = [i].flatten.compact
    i = filter_away_commandline_arguments_from_this_input(i)
    if i and i.is_a?(Array) and i.flatten.empty?
      # ===================================================================== #
      # Try to use all .wav files in the current directory in this case.
      # ===================================================================== #
      i = Dir['*.wav']
    end
    @files = i
  end

  # ========================================================================= #
  # === menu                                                       (menu tag)
  # ========================================================================= #
  def menu(
      i = @commandline_arguments
    )
    if i.is_a? Array
      i.flatten.each {|entry| menu(entry) }
    else
      case i # case tag
      # ===================================================================== #
      # === tomp3 * --basic-output-name
      # ===================================================================== #
      when /^-?-?basic(-|_)?output(-|_)?name$/i
        @prepend_this_string_to_the_filename = ''
      end
    end
  end

  # ========================================================================= #
  # === process_the_files
  # ========================================================================= #
  def process_the_files(
      i = @files
    )
    i.each {|this_file|
      output_filename = @prepend_this_string_to_the_filename+
                        File.basename(this_file).
                             sub(/#{File.extname(this_file)}$/,'').delete('()')+
                             '.mp3'
      if this_file.include? '(' or this_file.include?(' ')
        this_file = '"'+this_file+'"'
      end
      _ = 'ffmpeg -i '+this_file+
          ' -vn -ar 44100 -ac 2 -ab '+BITRATE_TO_USE+' -f mp3 '+
          output_filename
      e "The output file will be called: #{sfile(output_filename)}"
      e
      esystem _, colour: :royalblue # <- Pass the command into this system()-variant.
      e
      output_filename # <- And return it here.
    }
  end

  # ========================================================================= #
  # === run
  # ========================================================================= #
  def run
    menu
    process_the_files
  end

end; end

# =========================================================================== #
# === MultimediaParadise.to_mp3
#
# Simpler top-level API to convert the given input towards a .mp3 file.
# =========================================================================== #
def self.to_mp3(*i)
  MultimediaParadise::Audio::ToMp3.new(i)
end

end

if __FILE__ == $PROGRAM_NAME
  MultimediaParadise::Audio::ToMp3.new(ARGV)
end # to_mp3