#!/usr/bin/ruby -w
# Encoding: UTF-8
# frozen_string_literal: true
# =========================================================================== #
# === MultimediaParadise::ToOgg
#
# Convert an audio file into .ogg format.
#
# Usage example:
#   MultimediaParadise::ToOgg.new
# =========================================================================== #
# require 'multimedia_paradise/audio/to_ogg.rb'
# =========================================================================== #
require 'multimedia_paradise/audio/base.rb'

module MultimediaParadise

class ToOgg < ::MultimediaParadise::AudioBase # === MultimediaParadise::ToOgg

  # ========================================================================= #
  # === initialize
  # ========================================================================= #
  def initialize(
      i           = nil,
      run_already = true
    )
    reset
    set_input(i)
    run if run_already
  end

  # ========================================================================= #
  # === reset                                                     (reset tag)
  # ========================================================================= #
  def reset
    super()
  end

  # ========================================================================= #
  # === set_input
  # ========================================================================= #
  def set_input(i = '')
    i = i.first if i.is_a? Array
    i = i.to_s.dup if i
    @input = i
  end

  # ========================================================================= #
  # === input?
  # ========================================================================= #
  def input?
    @input
  end

  # ========================================================================= #
  # === return_input_name
  # ========================================================================= #
  def return_input_name
    input?.gsub(/#{File.extname(input?)}$/, '')
  end

  # ========================================================================= #
  # === output_file?
  # ========================================================================= #
  def output_file?
    'output_'+return_input_name+'.ogg'
  end

  # ========================================================================= #
  # === cleanup_old_files
  # ========================================================================= #
  def cleanup_old_files
    if File.exist? output_file?
      File.delete(output_file?)
    end
  end

  # ========================================================================= #
  # === report_that_we_are_done_now
  # ========================================================================= #
  def report_that_we_are_done_now
    e "Finished creating the file #{sfile(output_file?)}"
  end

  # ========================================================================= #
  # === run                                                         (run tag)
  # ========================================================================= #
  def run
    cleanup_old_files
    _ = 'ffmpeg'.dup
    _ << ' -i '+input?
    _ << ' -strict -2' # So that ffmpeg does not complain about vorbis being experimental.
    _ << ' -acodec libvorbis'
    _ << ' -aq 100 '
    _ << output_file? 
    esystem _
    report_that_we_are_done_now
  end

end; end

if __FILE__ == $PROGRAM_NAME
  MultimediaParadise::ToOgg.new(ARGV)
end # to_ogg