#!/usr/bin/ruby -w
# Encoding: UTF-8
# frozen_string_literal: true
# =========================================================================== #
# === MultimediaParadise::Audio::WavToMp3
#
# Convert a .wav file into a .mp3 file.
#
# Usage example:
#
#   require 'multimedia_paradise/audio/wav_to_mp3.rb'
#   MultimediaParadise::Audio::WavToMp3.new
#
# =========================================================================== #
# require 'multimedia_paradise/audio/wav_to_mp3.rb'
# =========================================================================== #
module MultimediaParadise

module Audio

class WavToMp3 # === MultimediaParadise::Audio::WavToMp3

  require 'multimedia_paradise/conversions/conversions.rb'

  # ========================================================================= #
  # === initialize
  # ========================================================================= #
  def initialize(
      i = nil, run_already = true
    )
    reset
    set_input(i)
    run if run_already
  end

  # ========================================================================= #
  # === reset
  # ========================================================================= #
  def reset # (reset tag)
  end

  # ========================================================================= #
  # === set_input
  # ========================================================================= #
  def set_input(i = '')
    i = i[0] if i.is_a? Array
    i = i.to_s.dup if i
    @input = i
  end

  # ========================================================================= #
  # === input?
  # ========================================================================= #
  def input?
    @input
  end

  # ========================================================================= #
  # === input_file_without_extension?
  # ========================================================================= #
  def input_file_without_extension?
    input?.gsub(/#{File.extname(input?)}/, '')
  end

  # ========================================================================= #
  # === output?
  # ========================================================================= #
  def output?
    'output_'+input_file_without_extension?+'.mp3'
  end; alias output_file? output? # === output_file?

  # ========================================================================= #
  # === run
  # ========================================================================= #
  def run # (run tag)
    result = MultimediaParadise.wav_to_mp3(
      input?,
      output: output_file?
    )
    return result
  end

end; end; end

if __FILE__ == $PROGRAM_NAME
  MultimediaParadise::Audio::WavToMp3.new(ARGV)
end # wavtomp3