#!/usr/bin/ruby -w
# Encoding: UTF-8
# frozen_string_literal: true
# =========================================================================== #
# require 'multimedia_paradise/commandline/menu.rb'
# =========================================================================== #
module MultimediaParadise

  require 'multimedia_paradise/toplevel_methods/misc.rb'
  require 'multimedia_paradise/toplevel_methods/is_on_roebe.rb'
  require 'multimedia_paradise/toplevel_methods/is_video_file.rb'
  require 'multimedia_paradise/toplevel_methods/is_a_multimedia_file.rb'
  require 'multimedia_paradise/toplevel_methods/player_in_use.rb'
  require 'multimedia_paradise/toplevel_methods/radio.rb'
  require 'multimedia_paradise/toplevel_methods/return_data_from_video_collection_file_for_this_entry.rb'
  require 'multimedia_paradise/toplevel_methods/return_random_video_file_from_the_video_collection.rb'
  require 'multimedia_paradise/help/help.rb' # bl $MULTIMEDIA_PARADISE/help/help.rb
  require 'multimedia_paradise/video/find_video/find_video.rb'
  require 'multimedia_paradise/toplevel_methods/opn.rb'

  Signal.trap('SIGINT') { exit }

  # ========================================================================= #
  # === @array_commandline_arguments
  # ========================================================================= #
  @array_commandline_arguments = []

  # ========================================================================= #
  # === MultimediaParadise.array_commandline_arguments?
  # ========================================================================= #
  def self.array_commandline_arguments?
    @array_commandline_arguments
  end

  # ========================================================================= #
  # === MultimediaParadise.report_missing_videos
  # ========================================================================= #
  def self.report_missing_videos
    _ = REALVIDS_DIRECTORY
    if File.directory? _
      local_files = Dir[_+'*'].select {|entry| is_a_video_file?(entry) }.sort.map {|entry|
        File.basename(entry).
        sub(/#{File.extname(entry)}$/,'')
      }
      all_titles = ::MultimediaParadise.return_all_video_files.map {|key, value|
        title = value['title']
        key.to_s.rjust(3,'0')+
        '_'+title.tr(' ','_')
      }
      local_files.each {|entry|
        if all_titles.include? entry
        else
          e rev+'No match found for '+sfancy(entry)+'.'
        end
      }
    end
  end

  # ========================================================================= #
  # === MultimediaParadise.set_array_commandline_arguments.permanently_use_this_video_player
  # ========================================================================= #
  def self.permanently_use_this_video_player(
      i = :mpv
    )
    i = i.to_s
    require 'save_file'
    _ = FILE_USE_THIS_VIDEO_PLAYER
    if File.exist? _ # Store into the proper file.
      e 'Storing into '+sfile(_)+'.'
      SaveFile.write_what_into(i,_)
      if is_on_roebe?
        _ = '/home/x/programming/ruby/src/'\
            'multimedia_paradise/lib/multimedia_paradise/yaml/'+
            File.basename(_)
        e 'Storing into '+sfile(_)+'.'
        SaveFile.write_what_into(i,_)
      end
    else
      e 'No file exists at '+_+'.'
    end
  end

  # ========================================================================= #
  # === MultimediaParadise.set_array_commandline_arguments
  # ========================================================================= #
  def self.set_array_commandline_arguments(i)
    @array_commandline_arguments = i
  end

  # ========================================================================= #
  # === MultimediaParadise.start_the_libui_wrapper_over_cut_multimedia
  # ========================================================================= #
  def self.start_the_libui_wrapper_over_cut_multimedia
    require 'multimedia_paradise/gui/libui/cut_multimedia/cut_multimedia.rb'
    MultimediaParadise::GUI::LibUI::CutMultimedia.new
  end

  require 'multimedia_paradise/video/mplayer_wrapper.rb'
  # ========================================================================= #
  # === MultimediaParadise.menu                                    (menu tag)
  # ========================================================================= #
  def self.menu(i)
    if i.is_a? Array
      i.each {|entry| menu(entry) }
    else
      case i # case tag
      # ===================================================================== #
      # === mpa --rhythmbox
      # ===================================================================== #
      when /^-?-?rhythmbox$/,
           /^-?-?show(-|_| )?internet(-|_| )?radio$/,
           /^-?-?radio\??$/i,
           /^-?-?radiostations$/i
        show_internet_radio_URLs
      # ===================================================================== #
      # === mpa --file-radio-stations
      # ===================================================================== #
      when /^-?-?file-radio-stations$/
        e sfile(file_radio_stations_STATIONS)
      # ===================================================================== #
      # === mpa --radio-compact
      #
      # This entry point will show all registered radio stations in
      # a compact manner.
      # ===================================================================== #
      when /^-?-?radio(-|_| )?compact$/
        show_internet_radio_URLs { :in_a_compact_form }
      # ===================================================================== #
      # === mpa --use-mpv
      # ===================================================================== #
      when /^-?-?use(-|_| )?mpv$/i,
           /^-?-?permanently(-|_| )?use(-|_| )?mpv$/i
        e ::Colours.grey('Permanently setting to use mpv as the video '\
          'player for this project.')
        MultimediaParadise.permanently_use_this_video_player(:mpv)
        exit
      # ===================================================================== #
      # === mpa --use-mplayer
      # ===================================================================== #
      when /^-?-?use(-|_| )?mplayer$/i,
           /^-?-?permanently(-|_| )?use(-|_| )?mplayer$/i
        e ::Colours.grey('Permanently setting to use mplayer as the video '\
          'player for this project.')
        MultimediaParadise.permanently_use_this_video_player(:mplayer)
        exit
      # ===================================================================== #
      # === mpa --cut-audio
      #
      # This entry point will start the libui-wrapper over CutMultimedia.
      # ===================================================================== #
      when /^-?-?cut(-|_| )?audio$/
        start_the_libui_wrapper_over_cut_multimedia
      # ===================================================================== #
      # === mpa --create-default-playlist
      # === mpa --default-playlist
      # ===================================================================== #
      when /^-?-?create(-|_| )?default(-|_| )?playlist$/,
           /^-?-?create(-|_| )?playlist$/,
           /^-?-?playlist$/,
           /^-?-?default(-|_)?playlist$/i
        require 'multimedia_paradise/audio/create_m3u_playlist.rb'
        create_default_playlist
      # ===================================================================== #
      # === mpa --help
      # ===================================================================== #
      when /^-?-?help$/i,
           '-h'
        MultimediaParadise.show_help
        exit
      # ===================================================================== #
      # === mpa --lyrics
      # ===================================================================== #
      when /^-?-?lyrics$/i
        require 'multimedia_paradise/gui/libui/lyrics/lyrics.rb'
        MultimediaParadise::GUI::Libui::Lyrics.new
        exit
      # ===================================================================== #
      # === mpa --n-videos?
      # ===================================================================== #
      when /^-?-?n(-|_| )?videos\??$/i
        MultimediaParadise.report_how_many_real_videos_are_stored
        exit
      # ===================================================================== #
      # === mpa --missing-videos?
      # ===================================================================== #
      when /^-?-?missing(-|_| )?videos\??$/i
        MultimediaParadise.report_missing_videos
        exit
      # ===================================================================== #
      # === multimedia --video-player
      #
      # This entry point can be used to start the ruby-gtk3 interface
      # over the local video-files.
      # ===================================================================== #
      when /^-?-?video(-|_| )?player$/i
        require 'multimedia_paradise/gui/gtk3/play_video_from_my_collection/play_video_from_my_collection.rb'
        MultimediaParadise::GUI::Gtk::PlayVideoFromMyCollection.run
      # ===================================================================== #
      # === multimedia --simple-widget
      #
      # This entry point exists to start the libui wrapper for a
      # simple play-widget, for local audio files (such as .mp3
      # files).
      # ===================================================================== #
      when /^-?-?simple(-|_| )?widget(-|_| )?libui$/i,
           /^-?-?simple(-|_| )?play(-|_| )?widget$/i,
           /^-?-?simple(-|_| )?widget$/i
        require 'multimedia_paradise/gui/libui/simple_play_widget/simple_play_widget.rb'
        MultimediaParadise::GUI::Libui::SimplePlayWidget.new
        exit
      # ===================================================================== #
      # === multimedia --simple-widget-gtk3
      #
      # This entry point exists to start the ruby-gtk3 wrapper for a
      # simple play-widget, for local audio files (such as .mp3 files).
      # ===================================================================== #
      when /^-?-?simple(-|_| )?widget(-|_| )?gtk3?$/i
        require 'multimedia_paradise/gui/gtk3/simple_play_widget/simple_play_widget.rb'
        MultimediaParadise::GUI::Gtk::SimplePlayWidget.run
        exit
      # ===================================================================== #
      # === multimedia --gtk-id-renamer
      #
      # This entry point allows the user to open the GTK based
      # "id-renamer" from the commandline.
      # ===================================================================== #
      when /^-?-?gtk(-|_| )?id(-|_| )?renamer$/i,
           /^-?-?gtk(-|_| )?tag$/i,
           /^-?-?gtk(-|_| )?tag(-|_| )?mp3(-|_| )?files$/i,
           /^-?-?tagger$/i,
           /^-?-?id(-|_| )?renamer$/i # === mpa --idrenamer
        require 'multimedia_paradise/gui/gtk3/tag_mp3_files/tag_mp3_files.rb'
        MultimediaParadise::GUI::Gtk::TagMp3Files.run(ARGV)
        exit
      # ===================================================================== #
      # === multimedia --sinatra
      # ===================================================================== #
      when /^-?-?sinatra$/i
        e 'Trying to start the sinatra-web interface next.'
        require 'multimedia_paradise/sinatra/app.rb'
        MultimediaParadise.run_app
        exit
      # ===================================================================== #
      # === mpa --gtk3
      #
      # This is the entry point for the gtk3 bindings, defaulting to the
      # radio-station widget right now.
      # ===================================================================== #
      when /^-?-?gtk3$/i,
           /^-?-?gtk$/i,
           /^-?-?radio3$/i
        require 'multimedia_paradise/gui/gtk3/play_from_radio_station/play_from_radio_station.rb'
        ::MultimediaParadise::GUI::Gtk::PlayFromRadioStation.run
        exit
      # ===================================================================== #
      # === mpa --gtk2
      #
      # This entry point has to select a particular widget for now.
      #
      # In the future we should perhaps use Gtk::Notebook.
      # ===================================================================== #
      when /^-?-?gtk2$/i
        require 'multimedia_paradise/gui/gtk2/play_from_radio_station/play_from_radio_station.rb'
        ::MultimediaParadise::GUI::Gtk::PlayFromRadioStation.run
        exit
      # ===================================================================== #
      # === mpa --formats?
      # ===================================================================== #
      when 'FORMAT',
           /^-?-?format/i,
           '--formats',
           '--formats?',
           '-f?'
        MultimediaParadise.show_available_audio_formats
        exit
      # ===================================================================== #
      # === mpa --intro
      # ===================================================================== #
      when /^-?-?intro\??$/i
        MultimediaParadise.show_intro
        exit
      # ===================================================================== #
      # === mpa ivolume
      # ===================================================================== #
      when /^-?-?increase_volume/,
           /^-?-?ivolume/
        # =================================================================== #
        # We must determine the target-file.
        # =================================================================== #
        this_file = this_file.gsub(/--increase_volume/, '').strip
        MultimediaParadise.increase_volume(this_file)
        exit
      # ===================================================================== #
      # === mpa --extract-audio
      # ===================================================================== #
      when /-?-?extract-?audio$/
        this_file = MultimediaParadise.array_commandline_arguments?.last # A bit hackish, but ok.
        ::MultimediaParadise.extract_audio(this_file)
        exit
      # ===================================================================== #
      # === mpa --random
      #
      # This entry point will pick a random video file from the realvids
      # directory.
      # ===================================================================== #
      when /^-?-?random$/i,
           /^-?-?random(-|_| )?video$/i
        require 'multimedia_paradise/video/mplayer_wrapper.rb'
        available_video_files = filter_for_video_files_from(
          Dir[realvids_directory?+'*']
        )
        random_video = available_video_files.sample
        MultimediaParadise::MplayerWrapper.new(random_video)
        exit # And exit afterwards.
      # ===================================================================== #
      # === mpa --simpsons
      # ===================================================================== #
      when /^-?-?simpsons$/
        e 'Trying to play some simpsons-movies next:'
        require 'multimedia_paradise/video/mplayer_wrapper.rb'
        MultimediaParadise.play_simpsons
      # ===================================================================== #
      # === mpa --halloween
      # ===================================================================== #
      when /^-?-?halloween$/
        e 'These are the Simpsons Halloween movies:'
        x =  Dir[cartoon_directory?+'**/**']
        y = x.select {|line| line.include? 'Treehou' }.shuffle
        y.each {|entry|
          esystem "#{use_this_multimedia_player?} #{entry}"
        }
      # ===================================================================== #
      # === mpa --store
      # ===================================================================== #
      when /^-?-?store$/i
        require 'multimedia_paradise/video/store_available_video_files.rb'
        MultimediaParadise::StoreAvailableVideoFiles.new
        exit
      # ===================================================================== #
      # === mpa --bbc1
      # === mpa --ö3
      # ===================================================================== #
      when /^-?-?bbc1$/i,
           /^-?-?bbc2$/i,
           /^-?-?bbc3$/i,
           /^-?-?bbc4$/i,
           /^-?-?ö3$/i,
           /^-?-?oe3$/i,
           /^-?-?kronehit$/i,
           /^-?-?oe1$/i,
           /^-?-?ö1$/i,
           /^-?-?FM4$/i,
           /^-?-?germany$/i,
           /^-?-?trance$/i,
           /^-?-?energy$/i,
           /^-?-?radio(-|_| )?wien$/i,
           /^-?-?orf(-|_| )?radio(-|_| )?wien$/i
        rename_kde_konsole_tab(
          i.delete('-')
        ) if is_on_roebe?
        i = return_radio_station_url_from_this_input(i)
        play_radio_from_this_url(i)
      # ===================================================================== #
      # === mpa --player-in-use?
      # ===================================================================== #
      when /^-?-?player(-|_| )?in(-|_| )?use\??$/i,
           /^-?-?player\??$/i
        e MultimediaParadise.player_in_use?
      else # else tag
        this_file = i
        # =================================================================== #
        # We have three basic options here:
        #
        #   1) If it is a multimedia file that has been given to us,
        #      we can play it. (If it does not exist, we will try to
        #      also look at /Depot/Video, before failing.)
        #
        #   2) Or we will play a video file that was registered in the
        #      video collection yaml-file.
        #
        #   3) Or we convert into .mp3, which was the default
        #      action to do for the MultimediaParadise project
        #      until December 2019.
        #
        # =================================================================== #
        # =================================================================== #
        # === 1)
        # =================================================================== #
        if is_a_multimedia_file?(this_file)
          unless File.exist? this_file
            possible_new_file = Dir["#{realvids_directory?}*#{this_file}*"].first
            if File.exist?(possible_new_file.to_s)
              this_file = possible_new_file
            end
          end
          MultimediaParadise::MplayerWrapper.new(this_file)
        # =================================================================== #
        # === 2)
        #
        # Here we will delegate towards class FindVideo.
        # =================================================================== #
        elsif has_this_video_file?(this_file)
          result = find_this_video(this_file)
          # ================================================================= #
          # result may now look like this: [["Dr. No", "115"]]
          # ================================================================= #
          # ================================================================= #
          # Apply a Regex match first. 
          # ================================================================= #
          array = result.select {|inner_array|
            if this_file =~ /^\d{1,5}$/ # Only numbers.
              inner_array.flatten.first
            else
              inner_array.first =~ /#{this_file}/
            end
          }
          if array == result
            # Something may not be correct here.
            array = result[0].reverse.join('_')
          end
          if array.is_a?(String) and array.include?(' ')
            array.delete!(' ')
          end
          if result
            result = array
            if result.is_a? Array
              result = result.flatten.first
            end
            # =============================================================== #
            # Next we replace all ' ' with '*', as this should work better
            # for when we apply a glob.
            # =============================================================== #
            result.tr!(' ','*') if result and result.include?(' ')
            if result and result.include?("'")
              # We forbid "'" tokens.
              result.delete!("'")
            end
          else
            opnn; e 'No result was found.'
          end
          target = "#{realvids_directory?}*#{result}*"
          result = Dir[target].first.to_s
          # ================================================================= #
          # Usage Examples:
          #
          #   mpa gladi
          #   mpa blade runner
          #   mpa 242
          #
          # ================================================================= #
          if result and !result.empty?
            opnn; e 'Now playing '+sfile(result)+'.'
            MultimediaParadise::MplayerWrapper.new(result)
          else
            opnn; e 'No file exists at '+sfile(result)+'.'
          end
        # =================================================================== #
        # === 3) Check if the input has only numbers
        # =================================================================== #
        elsif this_file.to_s =~ /^\d+$/ # If input has only numbers.
          dataset = YAML.load_file(file_video_collection?)
          title = dataset[this_file.to_i]['title']
          result = find_this_video(title).first.first
          result.tr!(' ','*') if result.include? ' ' # This should work better.
          target = "#{realvids_directory?}/*#{result}*"
          result = Dir[target].first
          # ================================================================= #
          # Usage Examples:
          #
          #   mpa gladi
          #   mpa blade runner
          #   mpa 242
          #
          # ================================================================= #
          MultimediaParadise::MplayerWrapper.new(result)
        # ==================================================================== #
        # === Handle playing of a random video
        #
        # This is possibility 4.
        # ==================================================================== #
        elsif this_file.include? 'rand' # Invocation example: mpa random
          _ = return_random_video_file_from_the_video_collection
          _.each {|entry|
            e
            e "We will next play #{sfile(entry.to_s)}."
            e
            MultimediaParadise::MplayerWrapper.new(entry)
          }
        else
          # convert(this_file, target_format)
          # ^^^ this used to be the default until 05.12.2019
          e 'Unknown action: '+sfancy(i)
        end
      end
    end
  end

end