#!/usr/bin/ruby -w
# Encoding: UTF-8
# frozen_string_literal: true
# =========================================================================== #
# require 'multimedia_paradise/constants/conversions.rb'
# =========================================================================== #
module MultimediaParadise

  # ========================================================================= #
  # === AUDIO_CHANNELS
  #
  # Most of the following constants come from the LAME documentation
  # ========================================================================= #
  AUDIO_CHANNELS = {
    :auto     => 'a',
    :joint    => 'j',
    :mid_side => 'f',
    :mono     => 'm',
    :stereo   => 's',
  }

  # ========================================================================= #
  # === MP3_BITRATES
  # ========================================================================= #
  MP3_BITRATES = [
    32, 40, 48, 56, 64, 80, 96, 112, 128, 160, 192, 224, 256, 320
  ]

  # ========================================================================= #
  # === SAMPLE_RATES
  # ========================================================================= #
  SAMPLE_RATES = [
    8, 11.025, 12, 16, 22.05, 24, 32, 44.1, 48
  ]

  # ========================================================================= #
  # === ENCODING_QUALITY
  # ========================================================================= #
  ENCODING_QUALITY = (0..9) # An Array from 0 to 9.

  # ========================================================================= #
  # === VBR_QUALITY
  # ========================================================================= #
  VBR_QUALITY      = (0..6)

  # ========================================================================= #
  # === DEFAULT_VIDEO_BITRATE
  # ========================================================================= #
  DEFAULT_VIDEO_BITRATE = 100_000

  # ========================================================================= #
  # === REPLAY_GAIN
  # ========================================================================= #
  REPLAY_GAIN  = { 
    :accurate    => '--replaygain-accurate',
    :clip_detect => '--clipdetect',
    :default     => nil,
    :fast        => '--replaygain-fast',                  
    :none        => '--noreplaygain'
  }

end