#!/usr/bin/ruby -w
# Encoding: UTF-8
# frozen_string_literal: true
# =========================================================================== #
# require 'multimedia_paradise/constants/directory_constants.rb'
# =========================================================================== #
module MultimediaParadise

  require 'multimedia_paradise/project/project.rb'

  # ========================================================================= #
  # === HOME_DIRECTORY_OF_USER_X
  # ========================================================================= #
  HOME_DIRECTORY_OF_USER_X = '/home/x/'

  # ========================================================================= #
  # === DEPOT_VIDEO
  #
  # This constant specifies where the local videos ought to be kept.
  #
  # An environment flag, called MULTIMEDIA_PARADISE_DEPOT_VIDEO, can
  # overrule this setting, if it has been set.
  # ========================================================================= #
  if ENV['MULTIMEDIA_PARADISE_DEPOT_VIDEO']
    DEPOT_VIDEO = ENV['MULTIMEDIA_PARADISE_DEPOT_VIDEO'].dup
  else
    DEPOT_VIDEO = "#{HOME_DIRECTORY_OF_USER_X}video/" # === DEPOT_VIDEO
  end

  # ========================================================================= #
  # === REALVIDS_DIRECTORY
  # ========================================================================= #
  REALVIDS_DIRECTORY = "#{DEPOT_VIDEO}Realvids/"
            REALVIDS = REALVIDS_DIRECTORY # === REALVIDS

  # ========================================================================= #
  # === MultimediaParadise.realvids_directory?
  #
  # This method is only really useful on my home setup. It will yield
  # a String such as:
  #
  #   "/home/x/video/Realvids/"
  #
  # ========================================================================= #
  def self.realvids_directory?
    REALVIDS_DIRECTORY
  end; self.instance_eval { alias realvid_directory?     realvids_directory? } # === MultimediaParadise.realvid_directory?
       self.instance_eval { alias directory_to_realvids  realvids_directory? } # === MultimediaParadise.directory_to_realvids
       self.instance_eval { alias directory_to_realvids? realvids_directory? } # === MultimediaParadise.directory_to_realvids?

  # ========================================================================= #
  # === CARTOONS_DIRECTORY
  #
  # This constant is mildly deprecated. Prefer to use the method
  # MultimediaParadise.cartoons_directory? instead.
  # ========================================================================= #
  CARTOONS_DIRECTORY = "#{DEPOT_VIDEO}Cartoons/"

  # ========================================================================= #
  # === MultimediaParadise.cartoons_directory?
  # ========================================================================= #
  def self.cartoons_directory?
    CARTOONS_DIRECTORY
  end; self.instance_eval { alias cartoon_directory? cartoons_directory? } # === MultimediaParadise.cartoon_directory?

  # ========================================================================= #
  # === MY_DATA_DIR
  # ========================================================================= #
  MY_DATA_DIR = "#{ENV['MY_DATA']}/" # Needs to have a trailing /, since it is assumed to be a directory.

  # ========================================================================= #
  # === MULTIMEDIA_PARADISE_DOC_DIR
  # ========================================================================= #
  MULTIMEDIA_PARADISE_DOC_DIR =
    "#{MultimediaParadise.project_base_dir?}doc/"

  # ========================================================================= #
  # === MY_AUDIO_DIR
  #
  # This constant is really only useful on my home system.
  #
  # It would point to a target such as "/home/x/DATA/AUDIO/".
  # ========================================================================= #
  MY_AUDIO_DIR = "#{MY_DATA_DIR}AUDIO/"

  # ========================================================================= #
  # === DIRECTORY_TO_THE_LOCAL_SONGS
  #
  # This constant will refer to the local directory that contains
  # the songs on my home setup.
  # ========================================================================= #
  if ENV['MULTIMEDIA_PARADISE_DEPOT_AUDIO']
    DIRECTORY_TO_THE_LOCAL_SONGS = rds("#{ENV['MULTIMEDIA_PARADISE_DEPOT_AUDIO']}/")
  else # else simply hardcode it.
    DIRECTORY_TO_THE_LOCAL_SONGS = "#{HOME_DIRECTORY_OF_USER_X}songs/"
  end; SONG_DIR        = DIRECTORY_TO_THE_LOCAL_SONGS # === SONG_DIR
       AUDIO_DIRECTORY = DIRECTORY_TO_THE_LOCAL_SONGS # === AUDIO_DIRECTORY

  # ========================================================================= #
  # === MultimediaParadise.local_audio_directory?
  #
  # On my home system this will yield a string such as "/home/x/songs/".
  # ========================================================================= #
  def self.directory_to_the_local_songs?
    DIRECTORY_TO_THE_LOCAL_SONGS
  end; self.instance_eval { alias local_audio_directory? directory_to_the_local_songs? } # === MultimediaParadise.local_audio_directory? 

  # ========================================================================= #
  # === HOME
  #
  # Determine the HOME directory of the user at hand.
  #
  # USER_HOME_DIR is an alias to it.
  # ========================================================================= #
  if ENV['HOME'].nil? # In this case we will set a default.
    if ENV['IS_ON_GOBOLINUX'].to_s == '1'
      HOME = '/Users/root/'
    else
      HOME = '/root/'
    end
  else # Else this is the default entry point.
    HOME = "#{ENV['HOME']}/"
  end
    USER_HOME_DIR  = HOME # === USER_HOME_DIR And add some "aliases" to the constant above ^^^.
    HOME_DIRECTORY = HOME # === HOME_DIRECTORY See above ^^^.
    HOME_DIR       = HOME # === HOME_DIR See above ^^^.

  # =========================================================================== #
  # === CONFIGURATION_DIRECTORY
  # =========================================================================== #
  CONFIGURATION_DIRECTORY = "#{PROJECT_BASE_DIRECTORY}configuration/"

end