#!/usr/bin/ruby -w
# Encoding: UTF-8
# frozen_string_literal: true
# =========================================================================== #
# require 'multimedia_paradise/constants/file_constants.rb'
# =========================================================================== #
module MultimediaParadise

  require 'multimedia_paradise/project/project.rb'
  require 'multimedia_paradise/constants/directory_constants.rb'
  require 'multimedia_paradise/constants/my_video_directory.rb'

  # ========================================================================= #
  # === FILE_MUSIC_GENRES
  # ========================================================================= #
  FILE_MUSIC_GENRES =
    "#{MultimediaParadise.project_yaml_dir?}music_genres.yml"

  # ========================================================================= #
  # === FILE_LYRICS
  # ========================================================================= #
  FILE_LYRICS =
    "#{MultimediaParadise.project_yaml_dir?}lyrics.yml"

  # ========================================================================= #
  # === MultimediaParadise.file_lyrics
  # ========================================================================= #
  def self.file_lyrics
    FILE_LYRICS
  end
  # ========================================================================= #
  # === FILE_AUDIO_FORMATS
  #
  # This constant has the file path towards the file called audio_formats.yml
  # ========================================================================= #
  FILE_AUDIO_FORMATS =
    "#{MultimediaParadise.project_yaml_dir?}audio_formats.yml"

  # ========================================================================= #
  # === FILE_RADIO_IMAGE
  #
  # Denote where the image for the radio can be found.
  # ========================================================================= #
  FILE_RADIO_IMAGE =
    "#{::MultimediaParadise.project_base_dir?}images/radio_image.png"

  # ========================================================================= #
  # === FILE_USE_THIS_VIDEO_PLAYER
  #
  # bl $MULTIMEDIA_PARADISE/yaml/use_this_video_player.yml
  # ========================================================================= #
  FILE_USE_THIS_VIDEO_PLAYER =
    "#{MultimediaParadise.project_yaml_dir?}use_this_video_player.yml"

  # ========================================================================= #
  # === FILE_IMAGE_FORMATS
  #
  # This constant has the file path towards the file called audio_formats.yml
  # ========================================================================= #
  FILE_IMAGE_FORMATS =
    "#{MultimediaParadise.project_yaml_dir?}image_formats.yml"

  # ========================================================================= #
  # === FILE_USAGE_EXAMPLES
  # ========================================================================= #
  FILE_USAGE_EXAMPLES =
    "#{MULTIMEDIA_PARADISE_DOC_DIR}USAGE_EXAMPLES.md"

  # ========================================================================= #
  # === FILE_SONG_TAGS
  # ========================================================================= #
  FILE_SONG_TAGS =
    "#{MY_AUDIO_DIR}yaml/song_tags.yml"

  # ========================================================================= #
  # === LYRICS_YAML_FILE
  # ========================================================================= #
  LYRICS_YAML_FILE =
    "#{MY_AUDIO_DIR}yaml/lyrics.yml"

  # ========================================================================= #
  # === FILE_RADIO_STATIONS
  #
  # If an ENVIRONMENT variable called MULTIMEDIA_PARADISE_RADIO_STATIONS
  # has been set then this will be favoured.
  #
  # Commandline invocation:
  #
  #   mparadise --file-radio-stations
  #
  # ========================================================================= #
  if ENV.has_key? 'MULTIMEDIA_PARADISE_RADIO_STATIONS'
    FILE_RADIO_STATIONS = ENV['MULTIMEDIA_PARADISE_RADIO_STATIONS']
  else
    # ======================================================================= #
    # This clause is the default on my home system - and for most
    # users too, I assume.
    # ======================================================================= #
    FILE_RADIO_STATIONS =
      "#{MultimediaParadise.project_yaml_dir?}radio/radio_stations.yml"
  end

  # ========================================================================= #
  # === MultimediaParadise.file_radio_stations
  # ========================================================================= #
  def self.file_radio_stations
    FILE_RADIO_STATIONS
  end

  # ========================================================================= #
  # === FILE_VIDEO_COLLECTION
  #
  # Use this constant to define your video collection.
  #
  # On my home system this will be:
  #
  #   /home/x/DATA/VIDEO/yaml/video_collection.yml
  #
  # ========================================================================= #
  FILE_VIDEO_COLLECTION = "#{PROJECT_YAML_DIRECTORY}video_collection/video_collection.yml"
    VIDEO_COLLECTION = FILE_VIDEO_COLLECTION # === VIDEO_COLLECTION

  # ========================================================================= #
  # === FILE_PLAYLIST
  # ========================================================================= #
  FILE_PLAYLIST =
    "#{PROJECT_YAML_DIRECTORY}playlist.yml"

  # ========================================================================= #
  # === MultimediaParadise.file_video_collection?
  #
  # Easier toplevel-method to find out where the file storing all local
  # videos is kept at.
  #
  # The file path returned by this method would typically be at:
  #
  #   /home/x/DATA/VIDEO/yaml/video_collection.yml
  #
  # ========================================================================= #
  def self.file_video_collection?
    FILE_VIDEO_COLLECTION
  end; self.instance_eval { alias file_video_collection                            file_video_collection? } # === MultimediaParadise.file_video_collection
       self.instance_eval { alias yaml_file?                                       file_video_collection? } # === MultimediaParadise.yaml_file?
       self.instance_eval { alias video_collection?                                file_video_collection? } # === MultimediaParadise.video_collection?
       self.instance_eval { alias location_of_yaml_file_with_registered_videofiles file_video_collection? } # === MultimediaParadise.location_of_yaml_file_with_registered_videofiles

  # ========================================================================= #
  # === MultimediaParadise.file_tv_channels
  #
  # This method will return a String such as:
  #
  #   "/usr/lib/ruby/site_ruby/3.0.0/multimedia_paradise/yaml/tv_channels/tv_channels.yml"
  #
  # ========================================================================= #
  def self.file_tv_channels
    "#{PROJECT_YAML_DIRECTORY}tv_channels/tv_channels.yml"
  end

  # ========================================================================= #
  # === MultimediaParadise.file_tales_from_the_crypt
  # ========================================================================= #
  def self.file_tales_from_the_crypt
    "#{MultimediaParadise.project_yaml_dir?}tales_from_the_crypt/tales_from_the_crypt.yml"
  end

end

if __FILE__ == $PROGRAM_NAME
  puts MultimediaParadise::FILE_AUDIO_FORMATS
  # puts MultimediaParadise::FILE_RADIO_STATIONS  
end