#!/usr/bin/ruby -w
# Encoding: UTF-8
# frozen_string_literal: true
# =========================================================================== #
# require 'multimedia_paradise/constants/misc.rb'
# =========================================================================== #
require 'multimedia_paradise/constants/file_constants.rb'
require 'multimedia_paradise/constants/video_filetypes.rb'

module MultimediaParadise

  # ========================================================================= #
  # === MultimediaParadise::TRY_TO_USE_UNICODE_SYMBOLS
  #
  # If the following constant is set to true then we will try to make use
  # of unicode symbols, when applicable. This may aid in the process of
  # displaying some multimedia icons on the commandline, for example.
  # ========================================================================= #
  TRY_TO_USE_UNICODE_SYMBOLS = true

  # ========================================================================= #
  # === MultimediaParadise::YOUR_EDITOR
  # ========================================================================= #
  YOUR_EDITOR = ENV['EDITOR'].to_s

  # ========================================================================= #
  # === MultimediaParadise::TRY_TO_RENAME_THE_KDE_KONSOLE_TAB
  #
  # If true then we assume that we use the KDE konsole and that we can
  # also rename a konsole-tab title.
  # ========================================================================= #
  TRY_TO_RENAME_THE_KDE_KONSOLE_TAB = true

  # ========================================================================= #
  # === ARRAY_HELP_OPTIONS                                         (help tag)
  #
  # We will commonly respond to these help options here.
  # ========================================================================= #
  ARRAY_HELP_OPTIONS = %w(
    HELP HEL help hel he h h? ? --help
  )

  # ========================================================================= #
  # === MultimediaParadise::IMAGE_FORMATS
  #
  # The file can usually be found here:
  #   bl $MULTIMEDIA_PARADISE/yaml/image_formats.yml
  # ========================================================================= #
  IMAGE_FORMATS = File.readlines(FILE_IMAGE_FORMATS).select {|entry|
    entry.start_with? '- .'
  }.map {|entry|
    entry.gsub!(/^- /,'') # Get rid of leading '-' characters.
    entry = entry[0, entry.index('#')].strip if entry.include? '#'
    entry.delete('.').strip
  } if File.exist? FILE_IMAGE_FORMATS

  # ========================================================================= #
  # === MultimediaParadise::AUDIO_FORMATS
  #
  # The file can usually be found here:
  #   bl $MULTIMEDIA_PARADISE/yaml/audio_formats.yml
  # ========================================================================= #
  AUDIO_FORMATS = File.readlines(FILE_AUDIO_FORMATS).select {|entry|
    entry.start_with? '- .'
  }.map {|entry|
    entry.gsub!(/^- /,'') # Get rid of leading '-' characters.
    entry = entry[0, entry.index('#')].strip if entry.include? '#'
    entry.delete('.').strip
  } if File.exist? FILE_AUDIO_FORMATS

  # ========================================================================= #
  # === MultimediaParadise::ARRAY_MULTIMEDIA_FILETYPES
  #
  # Build an array that holds all multimedia filetypes. This is a merger
  # between the video-filetypes and the audio-formats.
  # ========================================================================= #
  ARRAY_MULTIMEDIA_FILETYPES = ARRAY_VIDEO_FILETYPES+AUDIO_FORMATS # Assemble these two together.
    ARRAY_MULTIMEDIA_FILES = ARRAY_MULTIMEDIA_FILETYPES # === ARRAY_MULTIMEDIA_FILES

end