#!/usr/bin/ruby -w
# Encoding: UTF-8
# frozen_string_literal: true
# =========================================================================== #
require 'games_paradise/modifications/gosu/gosu.rb'

module MultimediaParadise

module GUI

require 'gosu'

class VideoPlayer < ::Gosu::Window

  # ========================================================================= #
  # === NAMESPACE
  # ========================================================================= #
  NAMESPACE = inspect

  # ========================================================================= #
  # === WIDTH
  # ========================================================================= #
  WIDTH = 1200

  # ========================================================================= #
  # === HEIGHT
  # ========================================================================= #
  HEIGHT = 480

  # ========================================================================= #
  # === TITLE
  # ========================================================================= #
  TITLE = 'Video Play for Gosu'

  # ========================================================================= #
  # === initialize
  # ========================================================================= #
  def initialize(
      run_already = true
    )
    super(WIDTH, HEIGHT)
    set_title(TITLE)
    reset
    run if run_already
  end

  # ========================================================================= #
  # === reset                                                     (reset tag)
  # ========================================================================= #
  def reset
    # ======================================================================= #
    # === @song
    # ======================================================================= #
    @song = Gosu::Song.new(
      '/home/x/songs/Youtube2008_Soundmix.mp3'
    )
    @song.play
    # ======================================================================= #
    # === @font
    # ======================================================================= #
    @font = set_font(:hack_30)
    # ======================================================================= #
    # === @available_songs
    # ======================================================================= #
    @available_songs = Dir['/home/x/songs/**']
  end

  # ========================================================================= #
  # === button_down
  # ========================================================================= #
  def button_down(id)
    case id
    when Gosu::KbEscape,
         Gosu::KbQ
      close # Exit on 'Q'.
    end
  end

  # ========================================================================= #
  # === draw
  # ========================================================================= #
  def draw
    y_position = 10 
    @font.red('A simple videoplayer in Gosu', 15, y_position)
    @available_songs.each {|this_song|
      y_position += 24
      @font.green(this_song, 15, y_position)
    }
  end

  # ========================================================================= #
  # === update
  #
  # We can not call draw methods from within the main update method.
  # ========================================================================= #
  def update
  end

  # ========================================================================= #
  # === run
  # ========================================================================= #
  def run
  end

end; end; end

if __FILE__ == $PROGRAM_NAME
  MultimediaParadise::GUI::VideoPlayer.new.show
end