#!/usr/bin/ruby -w
# Encoding: UTF-8
# frozen_string_literal: true
# =========================================================================== #
# === MultimediaParadise::GUI::Gtk::Notebook
#
# Usage example:
#
#   MultimediaParadise::GUI::Gtk::Notebook.new
#
# =========================================================================== #
# require 'multimedia_paradise/gui/gtk/notebook.rb'
# =========================================================================== #
require 'gtk_paradise/requires/require_the_base_module.rb'

module MultimediaParadise

module GUI

module Gtk

class Notebook < Gtk::Frame # === MultimediaParadise::Notebook

  require 'multimedia_paradise/gui/gtk2/play_video_from_my_collection.rb'
  require 'multimedia_paradise/gui/gtk2/video_collection.rb'
  require 'multimedia_paradise/gui/gtk2/youtube_downloader.rb'
  require 'multimedia_paradise/gui/gtk2/play_from_radio_station/play_from_radio_station.rb'
  require 'multimedia_paradise/gui/gtk2/lyrics/lyrics.rb'
  require 'multimedia_paradise/gui/gtk2/id_renamer/id_renamer.rb'

  require 'opn'

  begin
    require 'colours'
    include Colours
  rescue LoadError; end

  include ::Gtk::BaseModule

  # ========================================================================= #
  # === TITLE
  # ========================================================================= #
  TITLE = 'Multimedia Notebook'

  # ========================================================================= #
  # === WIDTH
  # ========================================================================= #
  WIDTH = 1280

  # ========================================================================= #
  # === initialize
  # ========================================================================= #
  def initialize(
      i           = nil,
      run_already = true
    )
    super(TITLE)
    set_label_widget(
      ::Gtk::Label.new.set_markup(
        %Q(<span size="larger" foreground="darkblue" weight="bold">#{TITLE}</span>)
      )
    )
    reset
    set_input(i)
    run if run_already
  end

  # ========================================================================= #
  # === reset                                                     (reset tag)
  # ========================================================================= #
  def reset
  end

  # ========================================================================= #
  # === set_input
  # ========================================================================= #
  def set_input(i = '')
    i = i.first if i.is_a? Array
    i = i.to_s.dup if i
    @input = i
  end

  # ========================================================================= #
  # === input?
  # ========================================================================= #
  def input?
    @input
  end

  # ========================================================================= #
  # === run                                                         (run tag)
  # ========================================================================= #
  def run
    # ======================================================================= #
    # Add the notebook next, via Gtk.add_notebook().
    # ======================================================================= #
    play_video_from_my_collection = MultimediaParadise::GUI::Gtk::PlayVideoFromMyCollection.new
    video_collection              = MultimediaParadise::GUI::Gtk::VideoCollection.new
    youtube_downloader            = MultimediaParadise::GUI::Gtk::YoutubeDownloader.new
    play_from_radio_station       = MultimediaParadise::GUI::Gtk::PlayFromRadioStation.new
    lyrics_widget                 = MultimediaParadise::GUI::Gtk::Lyrics.new
    #id_renamer                    = MultimediaParadise::GUI::Gtk::IdRenamer.new

    notebook = ::Gtk.add_notebook(
      play_video_from_my_collection,
      video_collection,
      youtube_downloader,
      play_from_radio_station,
      lyrics_widget,
      id_renamer
    ) { :enable_drag_and_support }
    add(notebook)
    show_all
  end

  # ========================================================================= #
  # === width?
  # ========================================================================= #
  def width?
    WIDTH
  end

  # ========================================================================= #
  # === height?
  # ========================================================================= #
  def height?
    900
  end

end; end; end; end

if __FILE__ == $PROGRAM_NAME
  require 'gtk_paradise/run.rb'
  _ = MultimediaParadise::GUI::Gtk::Notebook.new
  r = Gtk.run(
    _.width?,
    _.height?,
    :tabble,
    File.basename(__FILE__)
  )
  r << _
  r.modify_background(:normal, :lightgreen)
  r.top_left_then_run
end # gtk_videos