#!/usr/bin/ruby -w
# Encoding: UTF-8
# frozen_string_literal: true
# =========================================================================== #
# === MultimediaParadise::GUI::Gtk::ChangeMetadataWidget
#
# This widget can be used to change the metadata of video files, such
# as .mp4 files.
#
# For .mp4 files ffmpeg uses the following metadata tags:
#
#   - title
#   - author
#   - album_artist
#   - album
#   - comment
#   - composer
#   - copyright
#   - description
#   - episode_id
#   - genre
#   - grouping
#   - lyrics
#   - network
#   - show
#   - synopsis
#   - track
#   - year
#
# =========================================================================== #
# require 'multimedia_paradise/gui/gtk3/change_metadata_widget/change_metadata_widget.rb'
# MultimediaParadise::GUI::Gtk::ChangeMetadataWidget.run
# =========================================================================== #
require 'gtk_paradise/require_gtk3'

module MultimediaParadise

module GUI

module Gtk

class ChangeMetadataWidget < ::Gtk::Box # === MultimediaParadise::GUI::Gtk::ChangeMetadataWidget 

  require 'gtk_paradise/requires/require_the_base_module.rb'
  include ::Gtk::BaseModule

  require 'multimedia_paradise/gui/shared_code/change_metadata_widget/change_metadata_widget_module.rb'
  include MultimediaParadise::GUI::ChangeMetadataWidgetModule

  # ========================================================================= #
  # === NAMESPACE
  # ========================================================================= #
  NAMESPACE = inspect

  # ========================================================================= #
  # === initialize
  # ========================================================================= #
  def initialize(
      commandline_arguments = ARGV,
      run_already           = true
    )
    super(:vertical)
    reset
    set_commandline_arguments(
      commandline_arguments
    )
    run if run_already
  end

  # ========================================================================= #
  # === reset                                                     (reset tag)
  # ========================================================================= #
  def reset
    reset_the_internal_variables
    # ======================================================================= #
    # === @configuration
    # ======================================================================= #
    @configuration = [true, __dir__, NAMESPACE]
    # ======================================================================= #
    # === Set the title, width, height and the font in use.
    # ======================================================================= #
    title_width_height_font(TITLE, WIDTH, HEIGHT, USE_THIS_FONT)
    use_gtk_paradise_project_css_file
    infer_the_size_automatically
  end

  # ========================================================================= #
  # === connect_skeleton                                        (connect tag)
  # ========================================================================= #
  def connect_skeleton
    abort_on_exception
    minimal(@header)
    @grid.left_right(return_text1, @entry_input_file)
    @grid.new_line
    small_hbox = gtk_hbox
    small_hbox.maximal(@entry_output_file)
    event_box = gtk_event_box(gtk_icon('view-refresh'))
    event_box.hint =
      "Click on this image to automatically rename the video file to "\
      "the original name seen above.\n\n"\
      "This currently does not notify the user that the rename-action "\
      "has been applied, but it should work just fine nonetheless."
    event_box.on_clicked {
      do_rename_the_newly_created_file_to_the_original_file
    }
    small_hbox.minimal(event_box)
    @grid.left_right(return_text2, small_hbox) # Output file to use.
    @grid.new_line
    @grid.left_right(return_text3, entry_title?)
    @grid.new_line
    @grid.left_right(return_text4, entry_comment?)
    @grid.new_line
    @grid.left_right(return_text5, entry_year?)
    hbox_small = gtk_hbox
    hbox_small.minimal(@button_run_the_sys_command, 4)
    hbox_small.minimal(@spinner,                    4)
    hbox_small.center_into_the_middle
    @grid.full_row(hbox_small)
    minimal(@grid, 2)
  end

  # ========================================================================= #
  # === run                                                         (run tag)
  # ========================================================================= #
  def run
    super()
    menu
    respond_to_scroll_events_of_the_first_entry
  end

  # ========================================================================= #
  # === MultimediaParadise::GUI::Gtk::ChangeMetadataWidget.run
  # ========================================================================= #
  def self.run(
      i = ARGV
    )
    require 'gtk_paradise/app/app.rb'
    _ = ::MultimediaParadise::GUI::Gtk::ChangeMetadataWidget.new(i)
    r = ::Gtk.run
    r << _
    r.automatic_size_then_automatic_title
    r.enable_quick_exit
    r.top_left_then_run
  end

end; end; end; end

if __FILE__ == $PROGRAM_NAME
  MultimediaParadise::GUI::Gtk::ChangeMetadataWidget.run
end