#!/usr/bin/ruby -w
# Encoding: UTF-8
# frozen_string_literal: true
# =========================================================================== #
# require 'multimedia_paradise/gui/gtk3/tag_mp3_files/create.rb'
# =========================================================================== #
module MultimediaParadise

module GUI

module Gtk

class TagMp3Files < ::Gtk::Box # === MultimediaParadise::GUI::Gtk::TagMp3Files

  # ========================================================================= #
  # === create_the_context_menu
  # ========================================================================= #
  def create_the_context_menu
    # ======================================================================= #
    # === @context_menu
    # ======================================================================= #
    @context_menu = create_context_menu(self) {{
      # n_entries: 12,
      actions: {
        '• Play the selected audio file via mpv':              :play_the_selected_audio_file,
        '• Display some debug-information on the commandline': :show_debug_information
      }
    }}
    @context_menu.use_this_font(main_font?)
    @context_menu.css_class('bblack3_and_padding')
  end

  # ========================================================================= #
  # === create_the_status_bar
  # ========================================================================= #
  def create_the_status_bar
    # ======================================================================= #
    # === @status_bar
    # ======================================================================= #
    @status_bar = gtk_status_bar
    @status_bar.bblack1
    @status_bar_context_id = @status_bar.get_context_id('tag_mp3_files')
  end

  # ========================================================================= #
  # === create_the_labels                             (labels tag, label tag)
  # ========================================================================= #
  def create_the_labels
    # ======================================================================= #
    # === @label_description
    #
    # This is the very top-level description.
    # ======================================================================= #
    @label_description = bold_label(
      'This widget can be used to modify the ID3 tags of .mp3 files'
    )
    @label_description.css_class('indigo')
    @label_description.font_to_use = FONT_TO_USE_FOR_THE_MAIN_DESCRIPTION
  end

  # ========================================================================= #
  # === create_the_settings
  # ========================================================================= #
  def create_the_settings
    # ======================================================================= #
    # === @settings_vbox
    # ======================================================================= #
    @settings_vbox = gtk_vbox
    # ======================================================================= #
    # Style this vbox a bit:
    # ======================================================================= #
    @settings_vbox.pad5px
    @settings_vbox.set_border_width(20)
    @settings_vbox.use_this_font = main_font?
    _ = text("Settings\n")
    _.use_this_font = main_font?
    @settings_vbox.add(_)
    @settings_vbox.add_hspacer
    hbox = gtk_hbox
    hbox.minimal(left_aligned_text('Specify the directory to the local songs here: '), 2)
    hbox.minimal(@entry_use_this_as_the_directory_to_the_local_songs)
    @settings_vbox << hbox
    @settings_vbox.add_hspacer
    @settings_vbox.use_this_font = main_font?
    @settings_vbox.width_height(800, 300)
    @settings_vbox.show_all
  end

  # ========================================================================= #
  # === create_the_entries                           (entries tag, entry tag)
  #
  # This method will bundle - and create - all relevant gtk-entries
  # in this application.
  # ========================================================================= #
  def create_the_entries
    # ======================================================================= #
    # === @entry_use_this_as_the_directory_to_the_local_songs
    #
    # This entry can be used to designate another directory for the
    # local .mp3 files.
    # ======================================================================= #
    @entry_use_this_as_the_directory_to_the_local_songs = gtk_entry
    @entry_use_this_as_the_directory_to_the_local_songs.set_text(
      PATH_TO_THE_LOCAL_AUDIO_DIRECTORY
    )
    # ======================================================================= #
    # === @entry_for_the_comment
    # ======================================================================= #
    @entry_for_the_comment = gtk_entry
    @entry_for_the_comment.set_max_length(120)
    @entry_for_the_comment.set_size_request(680, 40)
    # ======================================================================= #
    # === @entry_for_the_title_of_the_song
    #
    # The next entry contains the title of the song.
    # ======================================================================= #
    @entry_for_the_title_of_the_song = gtk_entry
    @entry_for_the_title_of_the_song.set_max_length(150)
    # ======================================================================= #
    # === @entry_file_path
    #
    # The next entry, stored in @entry_file_path, will keep track
    # of our audio-file. This is the most important entry.
    # ======================================================================= #
    @entry_file_path = gtk_entry
    @entry_file_path.clear_background
    @entry_file_path.pad5px
    @entry_file_path.css_class('mar4px')
    @entry_file_path.yellow_background
    @entry_file_path.bblack2
    @entry_file_path.make_bold
    @entry_file_path.width_height(620, 42)
    @entry_file_path.ghost_text('Provide the path to the .mp3 file here ...')
    array = Dir['/home/x/songs/*'].map {|entry| File.basename(entry) }
    @entry_file_path.simple_completion(array)
    @entry_file_path.on_enter {
      do_analyse_the_mp3_file
    }
    @entry_file_path.make_scrollable
    @entry_file_path.signal_connect(:event) {|widget, event|
      if scroll_up_event?(event)
        do_assign_a_new_random_audio_file
      elsif scroll_down_event?(event)
        do_assign_a_new_random_audio_file
      end
    }
    # ======================================================================= #
    # If the user double-clicks on the file path then a new local file
    # will be added.
    # ======================================================================= #
    @entry_file_path.on_mouse_double_click_event {|widget, event|
      do_open_a_local_file_and_then_set_the_proper_entry
    }
  end

  # ========================================================================= #
  # === create_the_combo_box_entry_for_genre_select
  # ========================================================================= #
  def create_the_combo_box_entry_for_genre_select
    if File.exist? FILE_MUSIC_GENRES
      array_available_genres = YAML.load_file(FILE_MUSIC_GENRES)
    else
      array_available_genres = [] # Safeguard.
    end
    # ======================================================================= #
    # === @combo_box_entry_for_genre_select
    # ======================================================================= #
    @combo_box_entry_for_genre_select = return_combo_box_entry(
      with_these_elements: array_available_genres
    )
    @combo_box_entry_for_genre_select.clear_background
    # @combo_box_entry_for_genre_select.bblack1
    @combo_box_entry_for_genre_select.pad1px
    @combo_box_entry_for_genre_select.css_class('fancy_combobox')
    @combo_box_entry_for_genre_select.six_per_row
    # ======================================================================= #
    # Next we will unify the background for these entries here. Disabled
    # in August 2021.
    # ======================================================================= #
    # set_a_uniform_background_for_the_entries_on_the_right_side_widget
    # ======================================================================= #
    # Next add the combo-box that contains all genres. We will not set
    # an active entry, so that we can determine that this depends on
    # the user having set it (or not, accordingly).
    #
    #   @combo_box_entry_for_genre_select.active = 0
    #
    # ======================================================================= #
    @combo_box_entry_for_genre_select.on_changed {
      if @combo_box_entry_for_genre_select.active_iter
        text = @combo_box_entry_for_genre_select.active_text
        sync_this_text_onto_the_genre_entry(text)
      end
    }
  end

  # ========================================================================= #
  # === create_status_icon
  # ========================================================================= #
  def create_status_icon(
      tooltip_text = 'This text will show up upon a hover-event.'
    )
    @status_icon       = gtk_status_icon
    @status_icon.stock = ::Gtk::Stock::DIALOG_INFO
    @status_icon.set_icon_name('default_status_icon_name') # We can set a name.
    @status_icon.set_tooltip_text(tooltip_text)
    @status_icon.signal_connect(:activate) { |icon|
      if icon.respond_to? :blinking
        icon.blinking = !(icon.blinking?)
      end
    }
    # =========================================================================== #
    # Add a menu and a quit-button too. These will be triggered upon a
    # right-mouse-button presse vent.
    # =========================================================================== #
    menu = gtk_menu

    play_button = ::Gtk::ImageMenuItem.new(stock: ::Gtk::Stock::MEDIA_PLAY)
    play_button.signal_connect(:activate) {
      do_play_the_selected_song
    }
    menu.append(play_button)

    quit_button = ::Gtk::ImageMenuItem.new(stock: ::Gtk::Stock::QUIT)
    quit_button.signal_connect(:activate) { ::Gtk.main_quit }
    menu.append(quit_button)

    menu.show_all
    @status_icon.signal_connect(:popup_menu) { |icon, button, time|
      menu.popup(nil, nil, button, time)
    }
  end

  # ========================================================================= #
  # === create_the_buttons                          (buttons tag, button tag)
  # ========================================================================= #
  def create_the_buttons
    # ======================================================================= #
    # === @button_analyse_the_mp3_file
    # ======================================================================= #
    @button_analyse_the_mp3_file = button('_Analyse the .mp3 file')
    @button_analyse_the_mp3_file.on_clicked {
      do_analyse_the_mp3_file
    }
    # ======================================================================= #
    # === @box_for_the_button_analyse_the_mp3_file
    # ======================================================================= #
    @box_for_the_button_analyse_the_mp3_file = gtk_hbox
    @box_for_the_button_analyse_the_mp3_file.maximal(@button_analyse_the_mp3_file, 2)
    @spinner = gtk_spinner
    @box_for_the_button_analyse_the_mp3_file.minimal(@spinner, 4)
    # ======================================================================= #
    # === @button_assume
    # ======================================================================= #
    @button_assume = button('Assume')
    @button_assume.hint = 'This button can be used to assume and infer '\
                          'from the filename of the .mp3 file at hand '\
                          'the artist and the name of the song. This will '\
                          'only work for some songs, naturally, and you need '\
                          'to have a file assigned prior to being able to '\
                          'use this button.'
    @button_assume.on_clicked {
      do_assume_the_name_of_the_song_and_the_artist_from_the_filename_directly
    }
    # ======================================================================= #
    # === @button_play
    # ======================================================================= #
    @button_play = return_button_play
    # ======================================================================= #
    # === @event_box_open_file
    #
    # This is a small "icon" that also makes use of an open-file functionality.
    # ======================================================================= #
    @event_box_open_file = return_event_box_for_opening_a_file
    # ======================================================================= #
    # === @button_debug
    #
    # The debug-button is created next.
    # ======================================================================= #
    @button_debug = gtk_button('_Debug')
    @button_debug.on_hover(:lightgreen)
    @button_debug.on_clicked {
      load_the_id3_dataset_from_this_file
      do_debug
    }
    @button_debug.hint = 'This button is just for quick debugging. '\
                         'Most users will not need this button.'
    # ======================================================================= #
    # === @button_update
    #
    # The update button. It will update the id3 tags at hand, once pressed.
    # ======================================================================= #
    @button_update = gtk_button('_Update') # update tag
    use_this_icon_name = 'software-update-available-symbolic.symbolic'
    icon = load_icon(use_this_icon_name, 32, 0) 
    @button_update.set_image(icon)
    @button_update.set_always_show_image(true)
    @button_update.set_border_width(1)
    @button_update.set_size_request(125, 68)
    @button_update.lightgreen
    @button_update.hint = 'Press this button to '\
      '<b>update the id3-tags</b> of the .mp3 file at hand.'
    @button_update.on_clicked {
      do_save_the_tags # Save the dataset upon a click-event.
    }
    # ======================================================================= #
    # === @button_delete_the_tags
    #
    # The delete-the-tags-button.
    # ======================================================================= #
    @button_delete_the_tags = gtk_button('Delete the tags')
    use_this_icon_name = 'edit-delete-symbolic.symbolic'
    icon = load_icon(use_this_icon_name, 32, 0)
    @button_delete_the_tags.set_image(icon)
    @button_delete_the_tags.set_always_show_image(true)
    @button_delete_the_tags.set_border_width(1)
    @button_delete_the_tags.set_size_request(125, 68)
    @button_delete_the_tags.hint =
      "Click on this button to <b>remove all tags</b> "\
      "from the given .mp3 file at hand. (The "\
      "<b>save-button</b> works in precisely the same "\
      "manner, by the way.)\n\nBe certain to use this only when you really"\
      "want to remove all tags. Otherwise it may be better to ignore this "\
      "button and instead modify the content, as-is, on the right-hand "\
      "side."
    @button_delete_the_tags.set_border_width(1)
    @button_delete_the_tags.on_clicked {
      do_empty_the_tags
    }
    @button_delete_the_tags.set_size_request(125, 68)
    # ======================================================================= #
    # === @button_save
    #
    # The save-button. It is located on the left-hand side of the main
    # UI interface. This could be made inactive via a not_sensitive call.
    # ======================================================================= #
    @button_save = return_the_save_button(:do_not_make_use_of_a_special_CSS_class)
    # ======================================================================= #
    # === @button_close
    # ======================================================================= #
    @button_close = gtk_button_with_label(' _Close') # close tag
    use_this_icon_name = 'application-exit'
    icon = load_icon(use_this_icon_name, 32, 0) # Use the video-x-generic button.
    @button_close.set_image(icon)
    @button_close.set_always_show_image(true)
    @button_close.set_border_width(1)
    @button_close.set_size_request(125, 68)
    @button_close.on_clicked { ::Gtk.main_quit }
    @button_close.do_focus
    @button_close.hint = 'Press this button to '\
                         '<b>exit the application</b>.'
    # ======================================================================= #
    # === @button_open_a_random_audio_file
    # ======================================================================= #
    @button_open_a_random_audio_file = return_button_open_a_random_audio_file
    # ======================================================================= #
    # === @button_perform_a_google_search
    # ======================================================================= #
    @button_perform_a_google_search = gtk_button_with_label(' _Google search')
    use_this_icon_name = 'help-browser'
    icon = load_icon(use_this_icon_name, 32, 0) # Use the video-x-generic button.
    @button_perform_a_google_search.set_image(icon)
    @button_perform_a_google_search.set_always_show_image(true)
    @button_perform_a_google_search.set_border_width(1)
    @button_perform_a_google_search.set_size_request(125, 68)
    @button_perform_a_google_search.on_clicked {
      do_perform_a_google_search
    }
    @button_perform_a_google_search.do_focus
    @button_perform_a_google_search.hint = "Search via google "\
      "for wikipedia-related entries about the assumed "\
      "song and group.\n\nNote that this requires some entries to "\
      "exist already, such as the name of the song, and ideally "\
      "the name of the artist as well.\n\nThis functionality depends "\
      "on the gem called '<b>open</b>', so install that gem before "\
      "trying to use this button.\n"
  end

  # ========================================================================= #
  # === create_the_header_bar                                    (header tag)
  # ========================================================================= #
  def create_the_header_bar
    # ======================================================================= #
    # === @header_bar
    # ======================================================================= #
    @header_bar = widget_top
  end

  # ========================================================================= #
  # === create_skeleton                            (create tag, skeleton tag)
  #
  # The order for the various create-method calls is important.
  # ========================================================================= #
  def create_skeleton
    create_the_labels       # (1) Must come before create_the_header_bar()
    create_the_entries      # (4) Must come before we create the header, the main grid and the buttons.
    create_the_header_bar   # (2)
    create_the_context_menu # (3) Ok to call it here.
    create_the_buttons      # (5) Must come after we create_the_entries().
    create_the_status_bar   # (6)
    create_status_icon      # (7)
    create_the_settings     # (8)
  end

end; end; end; end