#!/usr/bin/ruby -w
# Encoding: UTF-8
# frozen_string_literal: true
# =========================================================================== #
# === MultimediaParadise::GUI::Gtk::YoutubeChannels
# =========================================================================== #
# require 'multimedia_paradise/gui/gtk3/gtk3_standalone/gtk3_standalone.rb'
# MultimediaParadise::GUI::Gtk::YoutubeChannels.run
# =========================================================================== #
require 'gtk_paradise/require_gtk3'

module MultimediaParadise

module GUI

module Gtk

class YoutubeChannels < ::Gtk::Box

  require 'gtk_paradise/requires/require_the_base_module.rb'
  include ::Gtk::BaseModule

  # ========================================================================= #
  # === NAMESPACE
  # ========================================================================= #
  NAMESPACE = inspect

  # ========================================================================= #
  # === TITLE
  # ========================================================================= #
  TITLE = 'Youtube Channels'

  # ========================================================================= #
  # === WIDTH
  # ========================================================================= #
  WIDTH = 600

  # ========================================================================= #
  # === HEIGHT
  # ========================================================================= #
  HEIGHT = 500

  # ========================================================================= #
  # === USE_THIS_FONT
  # ========================================================================= #
  USE_THIS_FONT = :dejavu_condensed_20

  require 'beautiful_url'
  require 'open'

  # ========================================================================= #
  # === ARRAY_ADD_THESE_CHANNELS
  # ========================================================================= #
  ARRAY_ADD_THESE_CHANNELS = %i(
    daily_dose_of_internet
    ozzy_man
    wt3
    chris_sumo
    devon_larrat
    monster_michael
    tods_workshop
  )

  # ========================================================================= #
  # === initialize
  # ========================================================================= #
  def initialize(
      commandline_arguments = ARGV,
      run_already           = true
    )
    super(:vertical)
    reset
    set_commandline_arguments(
      commandline_arguments
    )
    run if run_already
  end

  # ========================================================================= #
  # === reset                                                     (reset tag)
  # ========================================================================= #
  def reset
    reset_the_internal_variables
    # ======================================================================= #
    # === @configuration
    # ======================================================================= #
    @configuration = [true, __dir__, NAMESPACE]
    # ======================================================================= #
    # === @title
    # ======================================================================= #
    @title  = TITLE
    # ======================================================================= #
    # === @width
    # ======================================================================= #
    set_width(WIDTH)
    # ======================================================================= #
    # === @height
    # ======================================================================= #
    set_height(HEIGHT)
    set_use_this_font(USE_THIS_FONT)
    use_gtk_paradise_project_css_file 
    infer_the_size_automatically
    apply_custom_CSS_rules
  end

  # ========================================================================= #
  # === apply_custom_CSS_rules
  # ========================================================================= #
  def apply_custom_CSS_rules
    more_CSS_then_apply_these_rules '.fancy_border {
border: 2px dotted royalblue;
}'
  end

  # ========================================================================= #
  # === padding?
  # ========================================================================= #
  def padding?
    2
  end

  # ========================================================================= #
  # === border_size?
  # ========================================================================= #
  def border_size?
    2
  end

  # ========================================================================= #
  # === create_skeleton                                          (create tag)
  # ========================================================================= #
  def create_skeleton
    create_grid
  end

  # ========================================================================= #
  # === create_grid
  # ========================================================================= #
  def create_grid
    # ======================================================================= #
    # === @grid
    # ======================================================================= #
    @grid = return_default_grid
    @grid.css_class('pad8px')
    @grid.css_class('mar5px')
    @grid.css_class('fancy_border')
  end

  # ========================================================================= #
  # === connect_skeleton                                        (connect tag)
  # ========================================================================= #
  def connect_skeleton
    abort_on_exception
    bold_header = gtk_left_aligned_label(' Show video channels ')
    bold_header.make_bold
    minimal(bold_header, 5)
    ARRAY_ADD_THESE_CHANNELS.each_with_index {|this_channel, index| index += 1
      button = gtk_button(
        this_channel.to_s.tr('_',' ')
      )
      button.use_this_font = :dejavu_15
      button.on_clicked {
        Thread.new { do_open_this_channel_associated_URL(this_channel) }
      }
      button.make_bold
      button.bblack2
      @grid.left(text(" #{index} "))
      @grid.right(button)
      @grid.new_row
    }
    minimal(@grid, 28)
  end

  # ========================================================================= #
  # === do_open_this_channel_associated_URL
  # ========================================================================= #
  def do_open_this_channel_associated_URL(
      i
    )
    remote_url = BeautifulUrl[i]
    e 'Now opening '+sfancy(remote_url)+'.'
    Open.in_browser(remote_url)
  end

  # ========================================================================= #
  # === run                                                         (run tag)
  # ========================================================================= #
  def run
    create_skeleton_then_connect_skeleton
  end

  # ========================================================================= #
  # === MultimediaParadise::GUI::Gtk::YoutubeChannels.run
  # ========================================================================= #
  def self.run(
      i = ARGV
    )
    require 'gtk_paradise/run'
    _ = ::MultimediaParadise::GUI::Gtk::YoutubeChannels.new(i)
    r = ::Gtk.run
    r << _
    r.automatic
    r.top_left_then_run
  end

end; end; end; end

if __FILE__ == $PROGRAM_NAME
  MultimediaParadise::GUI::Gtk::YoutubeChannels.run
end