#!/usr/bin/ruby -w
# Encoding: UTF-8
# frozen_string_literal: true
# =========================================================================== #
# require 'multimedia_paradise/gui/gui_base.rb'
# include MultimediaParadise::GUI
# =========================================================================== #
require 'multimedia_paradise/toplevel_methods/misc.rb'

module MultimediaParadise

module GUI

  # =========================================================================== #
  # === N
  # =========================================================================== #
  N = "\n"

  # =========================================================================== #
  # === ISO
  # =========================================================================== #
  ISO = 'iso-8859-1'

  # =========================================================================== #
  # === UTF
  # =========================================================================== #
  UTF = 'utf-8'

  # ========================================================================= #
  # === VIDEO_COLLECTION
  # ========================================================================= #
  VIDEO_COLLECTION = ::MultimediaParadise.video_collection?

  # ========================================================================= #
  # === MAIN_FONT_TO_USE
  #
  # This is the main font to be used in the GUI.
  # ========================================================================= #
  MAIN_FONT_TO_USE = 'Hack 22' # 'DejaVu Sans Mono 22'

  # ========================================================================= #
  # === set_gtk_font_to_use
  #
  # This variant is specific for GTK.
  # ========================================================================= #
  def set_gtk_font_to_use(
      i = MAIN_FONT_TO_USE
    )
    ::Gtk::Settings.default.gtk_font_name = i
  end

  require 'multimedia_paradise/toplevel_methods/is_audio_file.rb'
  # ========================================================================= #
  # === populate_the_dataset_with_this_array
  #
  # This method will populate the @dataset variable - by default with
  # all local songs, which defaults to my home system. You can use
  # another path, though, as the third argument to this method.
  #
  # Note that by default we will filter.
  # ========================================================================= #
  def populate_the_dataset_with_this_array(
      array                       = :default,
      filter_for_audio_files_only = :yup,
      use_this_path               = '/home/x/songs/' # Here you can specify another path to use.
    )
    case array
    when :default
      array = Dir[use_this_path+'**']
    end
    case filter_for_audio_files_only
    when :yup,
         :default
      filter_for_audio_files_only = true
    end
    _ = array
    array.sort!
    _.select! {|entry|
      ::MultimediaParadise.is_audio_file?(entry)
    } if filter_for_audio_files_only
    @dataset = _
  end

  # ========================================================================= #
  # === esystem
  # ========================================================================= #
  def esystem(i)
    e i
    system i
  end

end; end