#!/usr/bin/ruby -w
# Encoding: UTF-8
# frozen_string_literal: true
# =========================================================================== #
# === MultimediaParadise::GUI::Libui::SimplePlayWidget
# =========================================================================== #
# require 'multimedia_paradise/gui/libui/simple_play_widget/simple_play_widget.rb'
# MultimediaParadise::GUI::Libui::SimplePlayWidget.new
# =========================================================================== #
module MultimediaParadise

module GUI

module Libui

class SimplePlayWidget # === MultimediaParadise::GUI::Libui::SimplePlayWidget

  require 'libui_paradise/autoinclude'
  require 'multimedia_paradise/constants/file_constants.rb'

  require 'multimedia_paradise/gui/gui_base.rb'
  include MultimediaParadise::GUI

  begin
    require 'colours'
  rescue LoadError; end

  # ========================================================================= #
  # === TITLE
  # ========================================================================= #
  TITLE = 'Simple Play Widget'

  # ========================================================================= #
  # === WIDTH
  # ========================================================================= #
  WIDTH = 620

  # ========================================================================= #
  # === EMOJI_PLAY
  # ========================================================================= #
  EMOJI_PLAY = '🔊️'

  # ========================================================================= #
  # === initialize
  # ========================================================================= #
  def initialize
    reset
    create_skeleton
    @main_window = ui_main_window(
      TITLE, WIDTH, 250, 0
    )
    outer_vbox = ui_vbox
    outer_vbox.is_padded

    middle_vbox = ui_vbox
    middle_vbox.is_padded
    # ======================================================================= #
    # Add the various buttons next:
    # ======================================================================= #
    populate_the_dataset_with_this_array.each_with_index {|entry, index| index += 1
      hbox = ui_hbox
      hbox.add(
        ui_text(index.to_s), 0
      )
      button = button(
        EMOJI_PLAY+' '+
        File.basename(entry)
      )
      button.on_clicked {
        do_play_this_audio_file(entry)
      }
      hbox.add(button)
      middle_vbox.add(hbox, 0)
     }
    # scrolled_window = ui_scrolled_window(middle_vbox)
    # outer_vbox.add(scrolled_window)
    outer_vbox.add(middle_vbox)
    @main_window.child = outer_vbox
    @main_window.intelligent_exit
  end

  # ========================================================================= #
  # === reset                                                     (reset tag)
  # ========================================================================= #
  def reset
    # ======================================================================= #
    # === @array_threads
    # ======================================================================= #
    @array_threads = []
    # ======================================================================= #
    # === @array_pids
    # ======================================================================= #
    @array_pids = []
  end

  # ========================================================================= #
  # === play_this_song_via_mpv
  # ========================================================================= #
  def play_this_song_via_mpv(this_song)
    e
    e "Next playing #{::Colours.steelblue(this_song)} via mpv."
    e
    @array_threads << Thread.new {
      pid = Process.spawn("mpv #{this_song}")
      @array_pids << pid
    }
  end; alias do_play_this_audio_file play_this_song_via_mpv # === do_play_this_audio_file

  # ========================================================================= #
  # === create_skeleton
  # ========================================================================= #
  def create_skeleton
  end

end; end; end; end

if __FILE__ == $PROGRAM_NAME
  MultimediaParadise::GUI::Libui::SimplePlayWidget.new
end