#!/usr/bin/ruby -w
# Encoding: UTF-8
# frozen_string_literal: true
# =========================================================================== #
# === MultimediaParadise::GUI::Libui::VideoPlayer
# =========================================================================== #
module MultimediaParadise

module GUI

module Libui

class VideoPlayer # === MultimediaParadise::GUI::Libui::VideoPlayer

  require 'libui_paradise/autoinclude'
  require 'multimedia_paradise/constants/file_constants.rb'
  require 'multimedia_paradise/toplevel_methods/return_random_video_file_from_the_video_collection.rb'

  begin
    require 'colours'
  rescue LoadError; end

  # ========================================================================= #
  # === initialize
  # ========================================================================= #
  def initialize
    create_skeleton
    @main_window = ui_main_window(
      'Video Player', 600, 250, 0
    )
    outer_vbox = ui_vbox
    outer_vbox.is_padded

    # ======================================================================= #
    # Add the buttons next:
    # ======================================================================= #
    button_play_the_video = button('Play a random video 📺')
    button_play_the_video.on_clicked {
      do_play_a_random_video
    }
    button = ui_quit_button # Use a quit button.
    vbox = padded_vbox
    vbox.add(button_play_the_video, 0)
    vbox.add(button,                0)
    outer_vbox << vbox

    @main_window.child = outer_vbox
    @main_window.intelligent_exit
  end

  # ========================================================================= #
  # === do_play_a_random_video
  # ========================================================================= #
  def do_play_a_random_video
    # ======================================================================= #
    # The -wid option tells mpv to connect to the window represented by the
    # ID $WINDOW_XID.
    #
    # Note that a simpler alternative to obtaining the xid value would be:
    #
    #   drawing_area.window.xid
    #
    # For now I retain the approach I chosen specified at the end of this
    # file, though.
    # ======================================================================= #
    _ = 'mpv '+::MultimediaParadise.return_random_video_that_exists.to_s
    esystem _
  end

  # ========================================================================= #
  # === create_skeleton
  # ========================================================================= #
  def create_skeleton
    @entry_URL_to_use_for_the_youtube_video = ui_entry
  end

end; end; end; end

if __FILE__ == $PROGRAM_NAME
  MultimediaParadise::GUI::Libui::VideoPlayer.new
end