#!/usr/bin/ruby -w
# Encoding: UTF-8
# frozen_string_literal: true
# =========================================================================== #
# === MultimediaParadise::GUI::Libui::WidgetIncreaseOrDecreaseAudio
# =========================================================================== #
module MultimediaParadise

module GUI

module Libui

class WidgetIncreaseOrDecreaseAudio # === MultimediaParadise::GUI::Libui::WidgetIncreaseOrDecreaseAudio

  require 'libui_paradise/autoinclude'

  begin
    require 'colours'
  rescue LoadError; end

  # ========================================================================= #
  # === TITLE
  # ========================================================================= #
  TITLE = 'Increase or Decrease audio'

  # ========================================================================= #
  # === WIDTH
  # ========================================================================= #
  WIDTH = 350

  # ========================================================================= #
  # === HEIGHT
  # ========================================================================= #
  HEIGHT = 200

  # ========================================================================= #
  # === initialize
  # ========================================================================= #
  def initialize(
      run_already = true
    )
    reset
    run if run_already
  end

  # ========================================================================= #
  # === reset
  # ========================================================================= #
  def reset
    @n_percent_changes = 3
  end

  # ========================================================================= #
  # === increase_audio_by_n_percent
  # ========================================================================= #
  def increase_audio_by_n_percent
    system return_cmd_to_run_for_the_audio_increase
  end

  # ========================================================================= #
  # === return_cmd_to_run_for_the_audio_increase
  # ========================================================================= #
  def return_cmd_to_run_for_the_audio_increase
    'amixer set Master '+n_percent?.to_s+'%+'
  end

  # ========================================================================= #
  # === decrease_audio_by_n_percent
  # ========================================================================= #
  def decrease_audio_by_n_percent
    system return_cmd_to_run_for_the_audio_decrease
  end

  # ========================================================================= #
  # === return_cmd_to_run_for_the_audio_decrease
  # ========================================================================= #
  def return_cmd_to_run_for_the_audio_decrease
    "amixer set Master #{n_percent?}%-"
  end

  # ========================================================================= #
  # === n_percent_changes?
  # ========================================================================= #
  def n_percent_changes?
    @n_percent_changes
  end; alias n_percent? n_percent_changes? # === n_percent?

  # ========================================================================= #
  # === create_skeleton
  # ========================================================================= #
  def create_skeleton
  end

  # ========================================================================= #
  # === run
  # ========================================================================= #
  def run
    create_skeleton
    @main_window = padded_main_window(
      TITLE, WIDTH, HEIGHT, 0
    )
    outer_vbox = ui_padded_vbox

    hbox_containing_the_two_buttons = padded_hbox
    button_increase_the_audio = button('Increase audio by +3% ⬆️')
    button_increase_the_audio.on_clicked {
      increase_audio_by_n_percent
    }
    hbox_containing_the_two_buttons << button_increase_the_audio
    button_decrease_the_audio = button('Decrease audio by -3% ⬇️')
    button_decrease_the_audio.on_clicked {
      decrease_audio_by_n_percent
    }
    hbox_containing_the_two_buttons << button_decrease_the_audio

    outer_vbox.minimal(hbox_containing_the_two_buttons)

    @main_window.child = outer_vbox
    @main_window.intelligent_exit
  end

end; end; end; end

if __FILE__ == $PROGRAM_NAME
  MultimediaParadise::GUI::Libui::WidgetIncreaseOrDecreaseAudio.new
end