#!/usr/bin/ruby -w
# Encoding: UTF-8
# frozen_string_literal: true
# =========================================================================== #
# === MultimediaParadise::GUI::Libui::YoutubeChannels
# =========================================================================== #
module MultimediaParadise

module GUI

module Libui

class YoutubeChannels # === MultimediaParadise::GUI::Libui::YoutubeChannels

  require 'libui_paradise/autoinclude'
  require 'multimedia_paradise/constants/file_constants.rb'

  begin
    require 'colours'
  rescue LoadError; end

  require 'beautiful_url'
  require 'open'

  # ========================================================================= #
  # === ARRAY_ADD_THESE_CHANNELS
  # ========================================================================= #
  ARRAY_ADD_THESE_CHANNELS = %i(
    daily_dose_of_internet
    ozzy_man
    wt3
    chris_sumo
    devon_larrat
    monster_michael
    tods_workshop
  )

  # ========================================================================= #
  # === initialize
  # ========================================================================= #
  def initialize
    reset
    create_skeleton
    @main_window = ui_main_window(
      'YoutubeChannels', 300, 300, 0
    )
    outer_vbox = ui_vbox
    outer_vbox.is_padded
    outer_vbox.minimal(
      ui_text('Show video channels')
    )

    ARRAY_ADD_THESE_CHANNELS.each_with_index {|this_channel, index|
      use_this_button = ui_button(
        this_channel.
        to_s.tr('_',' ').
        center(65)
      )
      use_this_button.on_clicked {
        Thread.new {
          do_open_this_channel_associated_URL(this_channel)
        }
      }
      @grid.ui_grid_append(
        ui_text(" #{index+1} "),
        0, 0+index, 1, 1, 1, 1, 1, 1
      )
      @grid.ui_grid_append(
        use_this_button,
        1, 0+index, 2, 1, 1, 1, 1, 1
      )
    }
    outer_vbox.maximal(@grid)
    @main_window.child = outer_vbox
    @main_window.intelligent_exit
  end

  # ========================================================================= #
  # === do_open_this_channel_associated_URL
  # ========================================================================= #
  def do_open_this_channel_associated_URL(
      i
    )
    remote_url = BeautifulUrl[i]
    e 'Now opening '+::Colours.sfancy(remote_url)+'.'
    Open.in_browser(remote_url)
  end

  # ========================================================================= #
  # === reset                                                     (reset tag)
  # ========================================================================= #
  def reset
  end

  # ========================================================================= #
  # === create_skeleton
  # ========================================================================= #
  def create_skeleton
    create_the_grid
  end

  # ========================================================================= #
  # === create_the_grid
  # ========================================================================= #
  def create_the_grid
    # ======================================================================= #
    # === @grid
    # ======================================================================= #
    @grid = ui_grid
  end

end; end; end; end

if __FILE__ == $PROGRAM_NAME
  MultimediaParadise::GUI::Libui::YoutubeChannels.new
end