#!/usr/bin/ruby -w
# Encoding: UTF-8
# frozen_string_literal: true
# =========================================================================== #
# === MultimediaParadise::GUI::CutMultimediaModule
# =========================================================================== #
# require 'multimedia_paradise/gui/shared_code/cut_multimedia/cut_multimedia_module.rb'
# include MultimediaParadise::GUI::CutMultimediaModule
# =========================================================================== #
module MultimediaParadise

module GUI

module CutMultimediaModule # === MultimediaParadise::GUI::CutMultimediaModule

  require 'multimedia_paradise/multimedia/cut_multimedia/cut_multimedia.rb'
  require 'multimedia_paradise/requires/require_file_duration.rb'

  # ========================================================================= #
  # === TITLE
  # ========================================================================= #
  TITLE = 'Cut multimedia files'

  # ========================================================================= #
  # === WIDTH
  # ========================================================================= #
  WIDTH = '82% or minimum 800px'

  # ========================================================================= #
  # === HEIGHT
  # ========================================================================= #
  HEIGHT = '55% or minimum 600px'

  # ========================================================================= #
  # === USE_THIS_FONT
  # ========================================================================= #
  USE_THIS_FONT = :dejavu_condensed_20

  # ========================================================================= #
  # === padding?
  # ========================================================================= #
  def padding?
    2
  end

  # ========================================================================= #
  # === border_size?
  # ========================================================================= #
  def border_size?
    2
  end

  # ========================================================================= #
  # === duration?
  # ========================================================================= #
  def duration?(i)
    ::MultimediaParadise.duration?(i) { :do_not_exit }.to_s
  end

  # ========================================================================= #
  # === duration_of_the_file?
  # ========================================================================= #
  def duration_of_the_file?
    duration?(main_file?).to_f # This will always return a Float.
  end

  # ========================================================================= #
  # === do_play_the_audio_file
  # ========================================================================= #
  def do_play_the_audio_file(
      i = main_file?
    )
    if i
      @cut_multimedia_object.set_main_file(i)
      Thread.new {
        @cut_multimedia_object.interactive_menu(
          'play_the_audio_file'
        )
      }
    end
  end

  # ========================================================================= #
  # === create_the_trim_button
  # ========================================================================= #
  def create_the_trim_button
    @button_trim = button('_Trim')
    @button_trim.hint = 'Trim n seconds from the beginning '\
      'of the multimedia file.'
    @button_trim.on_clicked {
      do_trim
    }
  end

  # ========================================================================= #
  # === button_trim?
  # ========================================================================= #
  def button_trim?
    @button_trim
  end; alias trim_button? button_trim? # === trim_button?

  # ========================================================================= #
  # === create_the_play_the_audio_file_button
  # ========================================================================= #
  def create_the_play_the_audio_file_button
    @button_play_the_audio_file = button('_Play the audio file')
    @button_play_the_audio_file.hint = 'Click this button to '\
      'play the audio file. This requires that you have assigned '\
      'a file in the above user-input entry.'
    @button_play_the_audio_file.on_clicked {
      do_play_the_audio_file
    }
  end

  # ========================================================================= #
  # === create_the_commit_button                                 (commit tag)
  # ========================================================================= #
  def create_the_commit_button
    @button_commit = button('_Commit')
    @button_commit.hint = 'Click on this button to cut at the specified start and end positions.'
    @button_commit.bblack2
    @button_commit.on_clicked {
      new_file = do_cut_the_audio_file
      if !uses_gtk3?
        main_entry?.set_text(new_file.to_s)
      end
    }
  end

  # ========================================================================= #
  # === commit_button?
  # ========================================================================= #
  def commit_button?
    @button_commit
  end; alias button_commit? commit_button? # === button_commit?

  # ========================================================================= #
  # === return_hbox_containing_the_start_and_end_position_entries
  # ========================================================================= #
  def return_hbox_containing_the_start_and_end_position_entries
    hbox = create_hbox
    hbox.minimal(text('Start position:'), 5)
    # ======================================================================= #
    # === @start_position_entry
    # ======================================================================= #
    @start_position_entry = create_entry('0')
    @start_position_entry.light_blue_background
    @start_position_entry.bblack1
    @start_position_entry.center
    @start_position_entry.hint = 'You can provide n seconds, such as '\
                                 '20, to start at the 20 seconds mark.'
    hbox.minimal(@start_position_entry, 3)
    hbox.minimal(text('End position:'), 5)
    # ======================================================================= #
    # === @end_position_entry
    # ======================================================================= #
    @end_position_entry = create_entry
    @end_position_entry.light_blue_background
    @end_position_entry.bblack1
    @end_position_entry.center
    @end_position_entry.hint = 'You can provide n seconds, such as '\
                               '20, to end at the 20 seconds mark.'
    hbox.minimal(@end_position_entry, 3)
    hbox.minimal(button_commit?, 3)
    button_merge_the_created_files = button('_Merge the created files')
    button_merge_the_created_files.clear_background
    button_merge_the_created_files.bblack2
    button_merge_the_created_files.css_class('BG_lightgreen')
    button_merge_the_created_files.on_hover_colour(:springgreen)
    button_merge_the_created_files.on_clicked {
      do_merge_the_created_files
    }
    hbox.minimal(button_merge_the_created_files, 3)
    return hbox
  end

  # ========================================================================= #
  # === create_the_delete_the_audio_file_button
  # ========================================================================= #
  def create_the_delete_the_audio_file_button
    @button_delete_the_audio_file = button('_Delete the audio file')
    @button_delete_the_audio_file.hint = 'Click this button in '\
      'order to delete the audio-file.'
    @button_delete_the_audio_file.on_clicked {
      do_delete_the_audio_file
    }
  end

  # ========================================================================= #
  # === do_cut_the_audio_file
  # ========================================================================= #
  def do_cut_the_audio_file
    start_position = @start_position_entry.text?
    end_position   = @end_position_entry.text?
    audio_file_to_use = File.absolute_path(main_file?)
    # ======================================================================= #
    # Navigate to a "safe" directory next:
    # ======================================================================= #
    cd('/tmp/') if File.directory? '/tmp/'
    cd('/home/x/Temp/') if File.directory? '/home/x/Temp/' # This is for my home directory.
    e 'The file `'+sfile(audio_file_to_use.to_s)+'` will be used.'
    _ = MultimediaParadise::CutMultimedia.new(audio_file_to_use, :do_not_run_yet)
    _.assign_this_file(audio_file_to_use)
    _.set_start_position(start_position)
    _.set_end_position(end_position)
    _.do_cut
    result = _.absolute_path_of_the_output_file?
    popup_over_this_widget(
      commit_button?,
      "The new file can now be found at:\n\n  <b>#{result}</b>\n\n"
    ) { :make_selectable } if uses_gtk3?
    return result # Return the file position of the newly created file as well, just in case.
  end

  # ========================================================================= #
  # === main_entry?
  # ========================================================================= #
  def main_entry?
    @entry_work_on_this_file
  end

  # ========================================================================= #
  # === main_file?
  # ========================================================================= #
  def main_file?
    main_entry?.text?.to_s
  end

  # ========================================================================= #
  # === set_use_this_file
  # ========================================================================= #
  def set_use_this_file(i)
    main_entry?.set_text(i.to_s)
  end; alias set_main_entry set_use_this_file # === set_main_entry

  # ========================================================================= #
  # === do_trim
  # ========================================================================= #
  def do_trim
    _ = "trim #{@entry_n_seconds_for_trimming.text?}"
    @cut_multimedia_object.interactive_menu(_)
    this_file = @cut_multimedia_object.output_file?
    set_main_entry(this_file)
    update_the_duration
    @cut_multimedia_object.set_work_on_this_file(this_file)
  end

  # ========================================================================= #
  # === do_assume_the_duration_as_end_position_if_a_file_has_been_assigned
  #
  # This method may modify the content of the entry stored under
  # @end_position_entry. This will only work if
  # ========================================================================= # 
  def do_assume_the_duration_as_end_position_if_a_file_has_been_assigned
    _ = duration_of_the_file?
    @end_position_entry.set_text(_.to_s)
  end

  # ========================================================================= #
  # === a_new_file_has_been_assigned
  # ========================================================================= #
  def a_new_file_has_been_assigned(i)
    @cut_multimedia_object.set_work_on_this_file(i)
    set_use_this_file(i)
    update_the_duration
  end

end; end; end