#!/usr/bin/ruby -w
# Encoding: UTF-8
# frozen_string_literal: true
# =========================================================================== #
# === MultimediaParadise::GUI::InformationAboutAMp3FileModule
# =========================================================================== #
# require 'multimedia_paradise/gui/shared_code/information_about_a_mp3_file/information_about_a_mp3_file_module.rb'
# include MultimediaParadise::GUI::InformationAboutAMp3FileModule
# =========================================================================== #
module MultimediaParadise

module GUI

module InformationAboutAMp3FileModule 

  require 'multimedia_paradise/constants/directory_constants.rb'
  require 'yaml'

  # ========================================================================= #
  # === ID_TAGGER_TO_USE_FOR_THE_MP3_FILES
  #
  # This constant presently can be either :taglib or :id3lib.
  #
  # :taglib is the default since as of 2021. I transitioned away from
  # :id3lib for several reasons.
  #
  # Depending on the value of this constant, a different library will
  # be used for tagging the .mp3 files at hand.
  # ======================================================================= #
  ID_TAGGER_TO_USE_FOR_THE_MP3_FILES = :taglib # or :idlib.

  # ========================================================================= #
  # === PATH_TO_THE_LOCAL_AUDIO_DIRECTORY
  # ========================================================================= #
  PATH_TO_THE_LOCAL_AUDIO_DIRECTORY =
    MultimediaParadise.local_audio_directory?

  # ========================================================================= #
  # === N_TIMES_UNICODE_HORIZONTAL_SPACER
  # ========================================================================= #
  N_TIMES_UNICODE_HORIZONTAL_SPACER = 11

  # ========================================================================= #
  # === FONT_SIZE
  # ========================================================================= #
  FONT_SIZE = 18

  # ========================================================================= #
  # === USE_THIS_FONT
  # ========================================================================= #
  USE_THIS_FONT = "Liberation serif #{FONT_SIZE}"

  # ========================================================================= #
  # === LARGER_FONT
  # ========================================================================= #
  LARGER_FONT = "Liberation serif #{FONT_SIZE+6}"

  # ========================================================================= #
  # === SLIGHTLY_SMALLER_FONT
  # ========================================================================= #
  SLIGHTLY_SMALLER_FONT = "Liberation serif #{FONT_SIZE-2}"

  # ========================================================================= #
  # === SMALLEST_FONT
  # ========================================================================= #
  SMALLEST_FONT = "Liberation serif #{FONT_SIZE-4}"

  # ========================================================================= #
  # === FONT_TO_USE_FOR_THE_MAIN_DESCRIPTION
  #
  # This is the font for use in the top header-bar. The user will typically
  # see this font first.
  #
  # Miriam Libre is better than Noto Serif, in my opinion.
  # ========================================================================= #
  FONT_TO_USE_FOR_THE_MAIN_DESCRIPTION =
    'Miriam Libre 20' # 'Noto Serif 22'

  # ========================================================================= #
  # === n_times_unicode_horizontal_spacer?
  # ========================================================================= #
  def n_times_unicode_horizontal_spacer?
    N_TIMES_UNICODE_HORIZONTAL_SPACER
  end

  # ========================================================================= #
  # === large_font_in_use?
  #
  # This is like the main font, but a bit larger.
  # ========================================================================= #
  def large_font_in_use?
    LARGER_FONT
  end

  # ========================================================================= #
  # === main_font?
  # ========================================================================= #
  def main_font?
    USE_THIS_FONT
  end; alias normal_font? main_font? # === normal_font?

  # ========================================================================= #
  # === slightly_smaller_font?
  # ========================================================================= #
  def slightly_smaller_font?
    SLIGHTLY_SMALLER_FONT
  end

  # ========================================================================= #
  # === return_path_to_the_local_songs_directory_or_return_pwd
  # ========================================================================= #
  def return_path_to_the_local_songs_directory_or_return_pwd
    _ = "#{Dir.pwd}/".squeeze('/')
    if File.directory? PATH_TO_THE_LOCAL_AUDIO_DIRECTORY
      _ = PATH_TO_THE_LOCAL_AUDIO_DIRECTORY # Use this directory instead.
    end
    return _
  end

end; end; end