#!/usr/bin/ruby -w
# Encoding: UTF-8
# frozen_string_literal: true
# =========================================================================== #
# === MultimediaParadise::GUI::Gtk::LyricsModule
# =========================================================================== #
# require 'multimedia_paradise/gui/shared_code/lyrics/lyrics_module.rb'
# include ::MultimediaParadise::GUI::Gtk::LyricsModule
# =========================================================================== #
module MultimediaParadise

module GUI

module Gtk

module LyricsModule # === MultimediaParadise::GUI::Gtk::LyricsModule

  require 'multimedia_paradise/constants/file_constants.rb'

  require 'gtk_paradise/widgets/gtk3/file_viewer/file_viewer.rb'

  require 'gtk_paradise/requires/require_the_base_module.rb'
  include ::Gtk::BaseModule

  # ========================================================================= #
  # === NAMESPACE
  # ========================================================================= #
  NAMESPACE = inspect

  # ========================================================================= #
  # === TITLE
  # ========================================================================= #
  TITLE = 'Lyrics'

  # ========================================================================= #
  # === WIDTH
  # ========================================================================= #
  WIDTH = '90%'

  # ========================================================================= #
  # === HEIGHT
  # ========================================================================= #
  HEIGHT = 920

  # ========================================================================= #
  # === initialize
  # ========================================================================= #
  def initialize(
      run_already = true
    )
    super()
    reset
    run if run_already
  end

  # ========================================================================= #
  # === reset
  # ========================================================================= #
  def reset
    reset_the_internal_variables
    # ======================================================================= #
    # === @configuration
    # ====================================================================== #
    @configuration = [true, __dir__, NAMESPACE]
    title_width_height_font(TITLE, width?, height?, main_font?)
    append_project_css_file
  end

  # ========================================================================= #
  # === width?
  # ========================================================================= #
  def width?
    WIDTH
  end

  # ========================================================================= #
  # === height?
  # ========================================================================= #
  def height?
    HEIGHT
  end

  # ========================================================================= #
  # === try_to_read_in_the_main_dataset
  # ========================================================================= #
  def try_to_read_in_the_main_dataset(
      this_file = FILE_LYRICS
    )
    if File.exist? this_file
      _ = File.read(FILE_LYRICS, encoding: 'utf-8')
      @dataset = _
    else
      @dataset = nil
    end
  end

  # ========================================================================= #
  # === main_font?
  # ========================================================================= #
  def main_font?
    :hack_18
  end

  # ========================================================================= #
  # === create_skeleton
  # ========================================================================= #
  def create_skeleton
    try_to_read_in_the_main_dataset
    # ======================================================================= #
    # === @file_viewer
    # ======================================================================= #
    @file_viewer = ::Gtk::FileViewer.new(FILE_LYRICS)
    @file_viewer.use_this_font(main_font?)
    add(@file_viewer)
  end

  # ========================================================================= #
  # === run                                                         (run tag)
  # ========================================================================= #
  def run
    create_skeleton
  end

  # =========================================================================== #
  # === MultimediaParadise::GUI::Gtk::LyricsModule.run
  # =========================================================================== #
  def self.run
    require 'gtk_paradise/run'
    lyrics_widget = MultimediaParadise::GUI::Gtk::Lyrics.new
    r = ::Gtk.run
    r << lyrics_widget
    r.modify_background(:normal, :azure)
    r.set_size_request(r.width?, r.height?)
    r.top_left_then_run
  end

end; end; end; end

if __FILE__ == $PROGRAM_NAME
  MultimediaParadise::GUI::Gtk::LyricsModule.run
end # gtklyrics