#!/usr/bin/ruby -w
# Encoding: UTF-8
# frozen_string_literal: true
# =========================================================================== #
# === MultimediaParadise::GUI::Gtk::MultimediaNotebookModule
# =========================================================================== #
# require 'multimedia_paradise/gui/shared_code/multimedia_notebook/multimedia_notebook_module.rb'
# include ::MultimediaParadise::GUI::Gtk::MultimediaNotebookModule
# =========================================================================== #
module MultimediaParadise

module GUI

module Gtk

module MultimediaNotebookModule

  require 'gtk_paradise/requires/require_the_base_module.rb'
  include ::Gtk::BaseModule

  # ========================================================================= #
  # === NAMESPACE
  # ========================================================================= #
  NAMESPACE = inspect

  # ========================================================================= #
  # === TITLE
  # ========================================================================= #
  TITLE = 'Notebook for Multimedia Applications'

  # ========================================================================= #
  # === WIDTH
  # ========================================================================= #
  WIDTH = '90%'

  # ========================================================================= #
  # === HEIGHT
  # ========================================================================= #
  HEIGHT = 500

  # ========================================================================= #
  # === initialize
  # ========================================================================= #
  def initialize(
      optional_file = nil,
      run_already   = true
    )
    super(:vertical)
    reset
    run if run_already
  end

  # ========================================================================= #
  # === reset                                                     (reset tag)
  # ========================================================================= #
  def reset
    reset_the_internal_variables
    # ======================================================================= #
    # === @configuration
    # ======================================================================= #
    @configuration = [true, __dir__, NAMESPACE]
    # ======================================================================= #
    # === @title
    # ======================================================================= #
    @title  = TITLE
    # ======================================================================= #
    # === @width
    # ======================================================================= #
    set_width(WIDTH)
    # ======================================================================= #
    # === @height
    # ======================================================================= #
    set_height(HEIGHT)
    set_use_this_font(:dejavu_condensed_22)
    append_project_css_file
    infer_the_size_automatically
  end

  # ========================================================================= #
  # === padding?
  # ========================================================================= #
  def padding?
    2
  end

  # ========================================================================= #
  # === border_size?
  # ========================================================================= #
  def border_size?
    2
  end

  # ========================================================================= #
  # === create_skeleton
  # ========================================================================= #
  def create_skeleton
    create_main_notebook
  end

  # ========================================================================= #
  # === create_main_notebook
  # ========================================================================= #
  def create_main_notebook
    @notebook = gtk_notebook(:useful_default)
  end

  # ========================================================================= #
  # === connect_skeleton                                        (connect tag)
  # ========================================================================= #
  def connect_skeleton
    Thread.abort_on_exception = true
    require 'multimedia_paradise/gui/gtk3/id_renamer/id_renamer.rb'
    require 'multimedia_paradise/gui/gtk3/lyrics/lyrics.rb'
    require 'multimedia_paradise/gui/gtk3/multimedia_converter/multimedia_converter.rb'
    require 'multimedia_paradise/gui/gtk3/play_from_radio_station/play_from_radio_station.rb'
    require 'multimedia_paradise/gui/gtk3/playlist/playlist.rb'
    require 'multimedia_paradise/gui/gtk3/play_video_from_my_collection/play_video_from_my_collection.rb'
    require 'multimedia_paradise/gui/gtk3/simple_play_widget/simple_play_widget.rb'
    # The next one has not yet been ported.
    # require 'multimedia_paradise/gui/gtk3/video_collection/video_collection.rb'
    require 'multimedia_paradise/gui/gtk3/widget_increase_or_decrease_audio/widget_increase_or_decrease_audio.rb'
    require 'multimedia_paradise/gui/gtk3/youtube_downloader/youtube_downloader.rb'
    # ======================================================================= #
    # The Hash is currently hardcoded. While it could easily be made a 
    # dynamic query, and the Hash removed altogether, for the time being I 
    # think this approach is simpler and faster, even if not as 
    # sophisticated.
    # ======================================================================= #
    @hash = {
      'MultimediaParadise::GUI::Gtk::IdRenamer' => 'ID Renamer',
      'MultimediaParadise::GUI::Gtk::Lyrics' => 'Lyrics',
      'MultimediaParadise::GUI::Gtk::MultimediaConverter' => 'Multimedia Converter',
      'MultimediaParadise::GUI::Gtk::PlayFromRadioStation' => 'Play from Radio Station',
      'MultimediaParadise::GUI::Gtk::Playlist' => 'Playlist',
      'MultimediaParadise::GUI::Gtk::PlayVideoFromMyCollection' => 'Play Video from my Collection',
      'MultimediaParadise::GUI::Gtk::SimplePlayWidget' => 'Simple Play Widget',
      #'MultimediaParadise::GUI::Gtk::VideoCollection' => 'Video Collection',
      'MultimediaParadise::GUI::Gtk::WidgetIncreaseOrDecreaseAudio' => 'Widget Increase or Decrease Audio',
      'MultimediaParadise::GUI::Gtk::YoutubeDownloader' => 'Youtube Downloader'
    }
    @hash.each_pair {|name_of_the_widget, title_that_is_to_be_used_for_the_tab|
      @notebook.append_this_widget(
        eval(name_of_the_widget).new,
        title_that_is_to_be_used_for_the_tab
      )
    }
    add(@notebook)
  end

  # ========================================================================= #
  # === run                                                         (run tag)
  # ========================================================================= #
  def run
    create_skeleton_then_connect_skeleton
  end

  # ========================================================================= #
  # === GtkParadise::GUI::Gtk::FoobarModule.run
  # ========================================================================= #
  def self.run(
      i = ARGV
    )
    require 'gtk_paradise/run'
    _ = ::MultimediaParadise::GUI::Gtk::MultimediaNotebook.new(i)
    r = ::Gtk.run
    r << _
    r.set_size_request(_.width?, _.height?)
    r.top_left_then_run
  end

end; end; end; end