#!/usr/bin/ruby -w
# Encoding: UTF-8
# frozen_string_literal: true
# =========================================================================== #
# === MultimediaParadise::GUI::Gtk::PlayVideoFromMyCollectionModule
# =========================================================================== #
# require 'multimedia_paradise/gui/shared_code/play_video_from_my_collection/play_video_from_my_collection_module.rb'
# include ::MultimediaParadise::GUI::Gtk::PlayVideoFromMyCollectionModule
# =========================================================================== #
module MultimediaParadise

module GUI

module Gtk

module PlayVideoFromMyCollectionModule

  require 'gtk_paradise/requires/require_the_base_module.rb'
  include ::Gtk::BaseModule

  require 'multimedia_paradise/gui/gui_base.rb'
  require 'multimedia_paradise/video/find_video/find_video.rb'
  require 'multimedia_paradise/toplevel_methods/is_video_file.rb'
  require 'multimedia_paradise/toplevel_methods/esystem.rb'
  require 'multimedia_paradise/toplevel_methods/return_data_from_video_collection_file_for_this_entry.rb'

  begin
    require 'cyberweb/requires/require_web_images.rb'
  rescue LoadError; end

  begin
    require 'mountpoints'
  rescue LoadError; end

  include ::Colours
  include MultimediaParadise::GUI

  extend ::Colours

  # ========================================================================= #
  # === NAMESPACE
  # ========================================================================= #
  NAMESPACE  = inspect

  # ========================================================================= #
  # === WIDTH
  # ========================================================================= #
  WIDTH = '90%'

  # ========================================================================= #
  # === HEIGHT
  # ========================================================================= #
  HEIGHT = 820

  # ========================================================================= #
  # === MPLAYER
  # ========================================================================= #
  MPLAYER = 'mplayer -vo x11 -zoom'

  # ========================================================================= #
  # === NPV
  # ========================================================================= #
  MPV = 'mpv'

  # ========================================================================= #
  # === VIDEOCAMERA
  #
  # We will use this image to denote that we are playing something.
  # ========================================================================= #
  if Object.const_defined?(:Cyberweb) and File.directory?('/home/x/data/')
    FILE_VIDEOCAMERA =
      "/home/x/data/images/#{Cyberweb.web_images(:videocamera)}"
  end

  # ========================================================================= #
  # === FROM_WHERE
  # ========================================================================= #
  FROM_WHERE = '/home/x/video/Realvids/*'

  # ========================================================================= #
  # === TITLE
  # ========================================================================= #
  TITLE = 'Play Videos'

  # ========================================================================= #
  # === FONT_FAMILY_TO_USE
  # ========================================================================= #
  FONT_FAMILY_TO_USE = 'DejaVu Sans Mono'

  # ========================================================================= #
  # === FONT_SIZE
  # ========================================================================= #
  FONT_SIZE = '20'

  # ========================================================================= #
  # === initialize
  # ========================================================================= #
  def initialize(
      i           = nil,
      run_already = true
    )
    super('')
    label_widget.set_markup(
      '<span weight="bold" size="x-large" foreground="darkblue"> '+TITLE+' </span>'
    )
    reset
    set_input(i)
    # ======================================================================= #
    # Delegate all key-press events next.
    # ======================================================================= #
    signal_connect(:key_press_event) {|widget, event|
      on_key_event(widget, event)
    }
    run if run_already
  end

  # ========================================================================= #
  # === reset                                                     (reset tag)
  # ========================================================================= #
  def reset
    reset_the_internal_variables
    # ======================================================================= #
    # === @configuration
    # ======================================================================= #
    @configuration = [true, __dir__, NAMESPACE]
    # ======================================================================= #
    # === @title
    # ======================================================================= #
    @title  = TITLE
    # ======================================================================= #
    # === @width
    # ======================================================================= #
    set_width(WIDTH)
    # ======================================================================= #
    # === @height
    # ======================================================================= #
    set_height(HEIGHT)
    @child_widget = gtk_vbox # This widget will hold the other variants.
    set_font(:dejagnu_20)
    append_project_css_file
  end

  # ========================================================================= #
  # === is_video_file?
  # ========================================================================= #
  def is_video_file?(i)
    ::MultimediaParadise.is_video_file?(i) # Delegate to the module-method here.
  end

  # ========================================================================= #
  # === set_input
  # ========================================================================= #
  def set_input(i = '')
    i = i.first if i.is_a? Array
    i = i.to_s.dup if i
    @input = i
  end

  # ========================================================================= #
  # === input?
  # ========================================================================= #
  def input?
    @input
  end

  # ========================================================================= #
  # === PlayVideoFromMyCollection
  # ========================================================================= #
  def self.padding?
    12
  end

  # ========================================================================= #
  # === obtain_all_video_files
  #
  # Simply return all video files
  # ========================================================================= #
  def obtain_all_video_files
    from_where = '/home/x/video/Realvids/*' # Hardcoded by default.
    from_where = FROM_WHERE
    results = Dir[from_where].select {|entry|
      is_video_file?(entry)
    }.sort
    return results
  end

  # ========================================================================= #
  # === add_buttons_from_video_files
  # ========================================================================= #
  def add_buttons_from_video_files
    # ======================================================================= #
    # Next iterate over all the available video files.
    # ======================================================================= #
    @array_all_video_files = obtain_all_video_files
    @array_all_video_files.each_with_index {|videofile, index| index += 1
      # ===================================================================== #
      # We want to show a little tooltip, so we have to return the
      # information from the yaml-file that keeps track of things.
      # ===================================================================== #
      this_entry = File.basename(videofile).split('_')
      this_entry = this_entry.first.to_i
      use_this_as_tooltip = ::MultimediaParadise.return_data_from_video_collection_file_for_this_entry(
        this_entry
      )
      name_of_the_videofile = File.basename(videofile).tr('_',' ')
      # ===================================================================== #
      # We add a button for each videofile.
      # ===================================================================== #
      button = gtk_bold_button(name_of_the_videofile, false) # False for "Do not use a shortcut".
      button.css_class('bblack1')
      button.on_clicked {|widget, event|
        play_this_videofile(videofile) # This contains the full path.
      }
      button.set_border_width(3)
      button.set_image(return_new_videocamera_image) # use the videocamera image.
      button.align_to_the_left_side
      # ===================================================================== #
      # The index button is the small button on the very left side.
      # ===================================================================== #
      index_button = gtk_bold_button("#{index.to_s.rjust(3)}")
      index_button.css_class('bblack1')
      index_button.set_size_request(90, 32)
      index_button.on_clicked {
        play_this_videofile(videofile)
      }
      if use_this_as_tooltip
        # =================================================================== #
        # === Extensive tooltip
        #
        # Add a tooltip there if it was given, but convert it into a String 
        # if it is a Hash. Furthermore, add a bit of pseudo-css as well.
        # =================================================================== #
        if use_this_as_tooltip.is_a? Hash
          if use_this_as_tooltip.has_key? 'imdb'
            imdb_URL = use_this_as_tooltip['imdb']
            button.on_right_mouse_button_clicked {
              try_to_open_the_imdb_link_in_the_browser(imdb_URL)
            }
          end
          use_this_as_tooltip = use_this_as_tooltip.map {|key, value|
            "#{key.tr('_',' ')}: #{value}"
          }.join("\n").
            sub(/title:/, '<b>title:</b>').
            sub(/release date:/, '<b>release date:</b>').
            sub(/genre:/, '<b>genre:</b>').
            sub(/addition date:/, '<b>addition date:</b>').
            sub(/imdb:/, '<b>imdb:</b>').
            sub(/language:/, '<b>language:</b>').
            sub(/my rating:/, '<b>my rating:</b>').dup
          # ================================================================= #
          # Add the file-extension as well, for additional info.
          # ================================================================= #
          use_this_as_tooltip << "\n<b>File type</b>: #{File.extname(videofile)}"
        end
        button.fancy_tooltip = use_this_as_tooltip
        index_button.fancy_tooltip = use_this_as_tooltip
      end
      button.show_all
      hbox_with_two_buttons = gtk_hbox
      hbox_with_two_buttons.minimal(index_button, 0)
      hbox_with_two_buttons.maximal(button, 0)
      @vbox_with_buttons.add(hbox_with_two_buttons)
    }
  end

  # ========================================================================= #
  # === add_alignment
  #
  # The alignment will hold the @scrolled_window widgt.
  # ========================================================================= #
  def add_alignment
    padding = 0.98
    @alignment = ::Gtk::Alignment.new(padding, padding, padding, padding)
    @alignment.set_padding(1,1,1,1) # oben,unten,links,rechts
    vbox = gtk_vbox
    vbox.minimal(@search_bar,       2)
    vbox.maximal(@scrolled_window, 1)
    @child_widget.maximal(vbox)
    @child_widget.set_size_request(500, 600)
    @alignment.add @child_widget
    add(@alignment)
  end

  # ========================================================================= #
  # === create_skeleton
  # ========================================================================= #
  def create_skeleton
    create_the_fancy_search_bar
    @vbox_with_buttons = gtk_vbox
    add_buttons_from_video_files
  end

  # ========================================================================= #
  # === MultimediaParadise::GUI::Gtk::PlayVideoFromMyCollectionModule[]
  # ========================================================================= #
  def self.[](i = '')
    new(i)
  end

  # ========================================================================= #
  # === return_new_videocamera_image
  # ========================================================================= #
  def return_new_videocamera_image
    pixbuf = ::GdkPixbuf::Pixbuf.new(file: FILE_VIDEOCAMERA)
    pixbuf.scale(5, 5, :hyper)
    image = gtk_image(pixbuf)
    return image
  end

  # ========================================================================= #
  # === add_the_scrolled_window
  # ========================================================================= #
  def add_the_scrolled_window
    @scrolled_window = gtk_scrolled_window(@vbox_with_buttons)
  end

  # ========================================================================= #
  # === padding?
  # ========================================================================= #
  def padding?
    8
  end

  # ========================================================================= #
  # === favicon?
  # ========================================================================= #
  def favicon?
    :tabble
  end

  # ========================================================================= #
  # === try_to_open_the_imdb_link_in_the_browser
  # ========================================================================= #
  def try_to_open_the_imdb_link_in_the_browser(i)
    require 'open'
    Open.in_browser(i)
  end

  # ========================================================================= #
  # === run                                                         (run tag)
  # ========================================================================= #
  def run
    set_the_font_in_use_for_this_application
    create_skeleton
    input_field  = gtk_input_field {{ max_length: 50 }}
    result_field = gtk_input_field
    input_field.on_key_press_event { |widget, event|
      case Gdk::Keyval.to_name(event.keyval)
      when 'Return','KP_Enter'
        _ = input_field.text
        _ = FindVideo[_]
        unless _.empty?
          _.map! {|entry| entry.last }
        end
        if _.is_a? Array
          _ = _.join(', ')
        end
        result_field.set_text(_)
      end
    }
    search_box = gtk_hbox(input_field, result_field)
    @vbox_with_buttons.minimal(search_box, 0)
    add_the_scrolled_window
    add_alignment # We will put all the
  end

  # ========================================================================= #
  # === MultimediaParadise::GUI::Gtk::PlayFromRadioStation.run
  # ========================================================================= #
  def self.run
    require 'gtk_paradise/run'
    video_collection = MultimediaParadise::GUI::Gtk::PlayVideoFromMyCollection.new(ARGV)
    r = ::Gtk.run
    r << video_collection
    r.set_size_request(video_collection.width?, video_collection.height?)
    r.signal_connect(:delete_event) { |_widget| Gtk.main_quit }
    r.modify_background(:normal, :lightskyblue)
    r.top_left_then_run
  end

  # ========================================================================= #
  # === on_key_event
  #
  # Cause the searchbar to show up.
  # ========================================================================= #
  def on_key_event(
      widget, event
    )
    shortcut = ::Gtk.accelerator_get_label(event.keyval, event.state)
    # ======================================================================= #
    # Get support for Ctrl+F next:
    # ======================================================================= #
    if ['Ctrl+F', 'Ctrl+Mod2+F'].include? shortcut
      if @search_bar.search_mode?
        # =================================================================== #
        # This causes the search bar to disappear.
        # =================================================================== #
        @search_bar.disappear
      else
        # =================================================================== #
        # And appear here in this event.
        # =================================================================== #
        @search_bar.appear
      end
    end
  end

  # ========================================================================= #
  # === play_this_videofile
  #
  # This is the method that will play a given videofile.
  # ========================================================================= #
  def play_this_videofile(videofile)
    efancy videofile
    @thread = Thread.new {
      _ = MPV+' '+videofile
      cliner
      e sfancy(_)
      cliner
      system _
    }
    @thread.join
    @thread = nil # Unsure whether we will need this here.
  end; alias play_this_video_file play_this_videofile # === play_this_video_file

  # ========================================================================= #
  # === try_to_find_a_match
  #
  # This method will try to find a corresponding match to the given
  # search term.
  # ========================================================================= #
  def try_to_find_a_match
    _ = @search_entry.text?
    possible_matches = @array_all_video_files.select {|entry|
      basename = File.basename(entry)
      basename.downcase.include? _
    }
    if possible_matches.empty?
      # ===================================================================== #
      # Show a popover in this case
      # ===================================================================== #
      popover_on(
        this_widget:      @search_entry,
        use_this_message: 'No match was found for the search '\
                          'term <b>'+_+'</b>.'
      )
    else
      first = possible_matches.first
      play_this_video_file(first)
    end
  end

  # ========================================================================= #
  # === create_the_fancy_search_bar
  # ========================================================================= #
  def create_the_fancy_search_bar
    @search_bar, @search_entry = drop_down_find_box
    @search_entry.width_height(400, 50)
    @search_entry.on_enter {
      try_to_find_a_match
    }
  end

end; end; end; end