#!/usr/bin/ruby -w
# Encoding: UTF-8
# frozen_string_literal: true
# =========================================================================== #
# === MultimediaParadise::GUI::Gtk::PlaylistModule
#
# This just provides a GUI wrapper over the commandline-based playlist
# functionality. I haven't really improved on this much at all,
# though. It was added in January of 2021.
# =========================================================================== #
# require 'multimedia_paradise/gui/shared_code/playlist/playlist_module.rb'
# include ::MultimediaParadise::GUI::Gtk::PlaylistModule
# =========================================================================== #
module MultimediaParadise

module GUI

module Gtk

module PlaylistModule

  require 'gtk_paradise/requires/require_the_base_module.rb'
  include ::Gtk::BaseModule

  require 'multimedia_paradise/audio/playlist/playlist.rb'

  # ========================================================================= #
  # === TITLE
  # ========================================================================= #
  TITLE = 'Playlist'

  # ========================================================================= #
  # === WIDTH
  # ========================================================================= #
  WIDTH = '90%'

  # ========================================================================= #
  # === HEIGHT
  # ========================================================================= #
  HEIGHT = 600

  # ========================================================================= #
  # === initialize
  # ========================================================================= #
  def initialize(
      run_already = true
    )
    super(:vertical)
    reset
    run if run_already
  end

  # ========================================================================= #
  # === reset
  # ========================================================================= #
  def reset
    reset_the_internal_variables
    infer_the_namespace
    # ======================================================================= #
    # === @configuration
    # ======================================================================= #
    @configuration = [true, __dir__, namespace?]
    # ======================================================================= #
    # === @title
    # ======================================================================= #
    set_title(TITLE)
    # ======================================================================= #
    # === @width
    # ======================================================================= #
    set_width(WIDTH)
    # ======================================================================= #
    # === @height
    # ======================================================================= #
    set_height(HEIGHT)
    @dataset = ::MultimediaParadise::Playlist.return_array
    set_use_this_font(:dejavu_condensed_22)
    append_project_css_file
    infer_the_size_automatically
  end

  # ========================================================================= #
  # === padding?
  # ========================================================================= #
  def padding?
    2
  end

  # ========================================================================= #
  # === border_size?
  # ========================================================================= #
  def border_size?
    2
  end

  # ========================================================================= #
  # === create_skeleton
  # ========================================================================= #
  def create_skeleton
  end

  # ========================================================================= #
  # === play_this_song
  # ========================================================================= #
  def play_this_song(i)
    esystem "mpv #{i}"
  end

  # ========================================================================= #
  # === connect_skeleton                                        (connect tag)
  # ========================================================================= #
  def connect_skeleton
    abort_on_exception
    vbox = gtk_vbox
    @dataset.each {|this_song|
      button = gtk_button(this_song)
      button.fancy_tooltips = this_song
      button.on_clicked {
        play_this_song(this_song)
      }
      button.text_is_left
      vbox.maximal(button)
    }
    scroll_bar = gtk_scrolled_window(vbox)
    scroll_bar.set_size_request(600, 500)
    maximal(scroll_bar)
  end

  # ========================================================================= #
  # === run
  # ========================================================================= #
  def run
    create_skeleton
    connect_skeleton
  end

  # ========================================================================= #
  # === MultimediaParadise::GUI::Gtk::PlaylistModule
  # ========================================================================= #
  def self.run
    require 'gtk_paradise/run'
    _ = ::MultimediaParadise::GUI::Gtk::Playlist.new
    r = ::Gtk.run
    r << _
    r.set_size_request(
      _.width?,
      _.height?
    )
    r.top_left_then_run
  end

end; end; end; end # gtk_playlist