#!/usr/bin/ruby -w
# Encoding: UTF-8
# frozen_string_literal: true
# =========================================================================== #
# === MultimediaParadise::GUI::Gtk::SoundEffectWidgetModule
# =========================================================================== #
# require 'multimedia_paradise/gui/shared_code/sound_effect_widget/sound_effect_widget_module.rb'
# include ::MultimediaParadise::GUI::Gtk::SoundEffectWidgetModule
# =========================================================================== #
module MultimediaParadise

module GUI

module Gtk

module SoundEffectWidgetModule

  require 'multimedia_paradise/project/project.rb'

  require 'gtk_paradise/requires/require_the_base_module.rb'
  include ::Gtk::BaseModule

  # ========================================================================= #
  # === NAMESPACE
  # ========================================================================= #
  NAMESPACE = inspect

  # ========================================================================= #
  # === TITLE
  # ========================================================================= #
  TITLE = 'Sound Effect Widget'

  # ========================================================================= #
  # === WIDTH
  # ========================================================================= #
  WIDTH = '80%'

  # ========================================================================= #
  # === HEIGHT
  # ========================================================================= #
  HEIGHT = 500

  # ========================================================================= #
  # Some sound effects are grouped next.
  # ========================================================================= #
  FILE_AUDIO1 = MultimediaParadise::PROJECT_BASE_DIRECTORY+'sound_effects/cat_meow.mp3'
  FILE_AUDIO2 = MultimediaParadise::PROJECT_BASE_DIRECTORY+'sound_effects/channel_opened.mp3'
  FILE_AUDIO3 = MultimediaParadise::PROJECT_BASE_DIRECTORY+'sound_effects/phone_ring.mp3'

  # ========================================================================= #
  # === initialize
  # ========================================================================= #
  def initialize(
      run_already = true
    )
    super(:vertical)
    reset
    run if run_already
  end

  # ========================================================================= #
  # === reset                                                     (reset tag)
  # ========================================================================= #
  def reset
    reset_the_internal_variables
    # ======================================================================= #
    # === @configuration
    # ======================================================================= #
    @configuration = [true, __dir__, NAMESPACE]
    # ======================================================================= #
    # === @title
    # ======================================================================= #
    @title  = TITLE
    # ======================================================================= #
    # === @width
    # ======================================================================= #
    set_width(WIDTH)
    # ======================================================================= #
    # === @height
    # ======================================================================= #
    set_height(HEIGHT)
    set_use_this_font(:dejavu_condensed_22)
    append_project_css_file # or use use_gtk_paradise_project_css_file
    infer_the_size_automatically
  end

  # ========================================================================= #
  # === padding?
  # ========================================================================= #
  def padding?
    2
  end

  # ========================================================================= #
  # === border_size?
  # ========================================================================= #
  def border_size?
    2
  end

  # ========================================================================= #
  # === create_skeleton                                          (create tag)
  # ========================================================================= #
  def create_skeleton
  end

  # ========================================================================= #
  # === connect_skeleton                                        (connect tag)
  # ========================================================================= #
  def connect_skeleton
    abort_on_exception

    button1 = gtk_bold_button(no_ext(File.basename(FILE_AUDIO1)))
    button1.on_clicked {
      Thread.new { esystem 'mpv '+FILE_AUDIO1 }
    }
    minimal(button1, 1)

    button2 = gtk_bold_button(no_ext(File.basename(FILE_AUDIO2)))
    button2.on_clicked {
      Thread.new { esystem 'mpv '+FILE_AUDIO2 }
    }
    minimal(button2, 1)

    button3 = gtk_bold_button(no_ext(File.basename(FILE_AUDIO3)))
    button3.on_clicked {
      Thread.new { esystem 'mpv '+FILE_AUDIO3 }
    }
    minimal(button3, 1)
    style_the_buttons_uniformly
  end

  # ========================================================================= #
  # === no_ext
  # ========================================================================= #
  def no_ext(i)
    i.sub(/#{File.extname(i)}$/,'')
  end

  # ========================================================================= #
  # === style_the_buttons_uniformly
  # ========================================================================= #
  def style_the_buttons_uniformly
    all_buttons?.each {|this_button|
      this_button.bblack1
      this_button.disallow_resizing
      this_button.set_size_request(30, 30)
    }
  end

  # ========================================================================= #
  # === run                                                         (run tag)
  # ========================================================================= #
  def run
    create_skeleton_then_connect_skeleton
  end

  # ========================================================================= #
  # === MultimediaParadise::GUI::Gtk::SoundEffectWidgetModule.run
  # ========================================================================= #
  def self.run(
      i = ARGV
    )
    require 'gtk_paradise/run'
    _ = ::MultimediaParadise::GUI::Gtk::SoundEffectWidget.new(i)
    r = ::Gtk.run
    r << _
    r.set_size_request(_.width?, _.height?)
    r.automatic_title
    r.top_left_then_run
  end

end; end; end; end