#!/usr/bin/ruby -w
# Encoding: UTF-8
# frozen_string_literal: true
# =========================================================================== #
# require 'multimedia_paradise/help/help.rb'
# =========================================================================== #
module MultimediaParadise

  require 'multimedia_paradise/colours/colours.rb'

  # ========================================================================= #
  # === MultimediaParadise.show_help
  #
  # This method shows some of the options available through the
  # commandline interface at bin/multimedia_paradise (mpa).
  #
  # Commandline way to invoke this method is:
  #
  #   mpa --help
  #
  # ========================================================================= #
  def self.show_help
    e "#{rev}You can use the MultimediaParadise project in this manner:"
    e
    e '  '+simp($PROGRAM_NAME.to_s)+' foo.mp3 foo.wav'
    e
    e 'This would convert '+orange('foo.mp3')+' into '+orange('foo.wav')
    e
    e '  --formats              # Will list all available format-conversions.'
    e
    e '  --increase_volume file # Use this to increase the volume of an '\
      'audio file'
    e
    e '  --intro                # Display some intro-information'
    e
    e '  --extract-audio        # Use ffmpeg to extract the audio '\
      'from a video file'
    e
    e '  --store                # Store all available video files into '\
      'a local yaml file'
    e
    e 'There are more options available, though:'
    e
    e '  --radio?               # Show URLs to radio-stations for '\
      'use in e. g. '+orange('rhythmbox')
    e '  --cut-audio            # Start the libui-wrapper for '\
      'cutting multimedia files (such as a .mp3 file)'
    e
  end

  # ========================================================================= #
  # === MultimediaParadise.show_intro
  #
  # This will show some intro in how to use this project from the
  # commandline.
  # ========================================================================= #  
  def self.show_intro
intro = <<-EOF

  This is an introduction to the MultimediaParadise project.

  This project integrates audio and video related components.

  This here is a short introduction into some of the possibilities
  of the project.

EOF
    _ = ::MultimediaParadise::FILE_USAGE_EXAMPLES
    if File.exist? _ # Append the file only if it exists.
      intro << File.read(_) # Append the file content here.
    end
    e intro # Output the intro.
  end

end