#!/usr/bin/ruby -w
# Encoding: UTF-8
# frozen_string_literal: true
# =========================================================================== #
# === MultimediaParadise::LongFormatToMillisecondsConverter
#
# Usage example:
#
#   MultimediaParadise::LongFormatToMillisecondsConverter.new
#
# =========================================================================== #
# require 'multimedia_paradise/misc/long_format_to_milliseconds_converter.rb'
# MultimediaParadise.long_format_to_milliseconds(ARGV)
# =========================================================================== #
module MultimediaParadise

class LongFormatToMillisecondsConverter # === MultimediaParadise::LongFormatToMillisecondsConverter

  alias e puts

  # ========================================================================= #
  # === DEFAULT_INPUT
  # ========================================================================= #
  DEFAULT_INPUT = '00:00:33.55'

  # ========================================================================= #
  # === UNITS_IN_MS
  # ========================================================================= #
  UNITS_IN_MS = {
    0 => 3600000, # hour
    1 =>   60000, # minute
    2 =>    1000  # second
  }

  # ========================================================================= #
  # === initialize
  # ========================================================================= #
  def initialize(
      optional_input = ARGV,
      run_already    = true
    )
    reset
    set_input(
      optional_input
    )
    run if run_already
  end

  # ========================================================================= #
  # === reset
  # ========================================================================= #
  def reset
    # ======================================================================= #
    # === @ms
    # ======================================================================= #
    @ms = 0.0
    # ======================================================================= #
    # === @input
    # ======================================================================= #
    @input = nil
  end

  # ========================================================================= #
  # === set_input
  # ========================================================================= #
  def set_input(i = DEFAULT_INPUT)
    i = i.join(' ') if i.is_a? Array
    i = DEFAULT_INPUT if i.nil?
    i = i.to_s.dup.strip
    if i.include? 'std'
      i = (i.gsub(/std/,'').to_f * 60) * 1000
    elsif i.include? 'minuten'
      i = (i.gsub(/minuten/,'').to_f * 60) * 1000 # We first need to get the result in milliseconds.
    end
    @input = i
  end

  # ========================================================================= #
  # === determine_ms
  # ========================================================================= #
  def determine_ms
    if input?.to_s.include? ':'
      @ms = input?.split(':').each_with_index.map {|d, i|
        if i == 2 # Convert all , into .
          d = d.gsub(/,/,'.') if d.include? ','
        end
        d.to_f * UNITS_IN_MS[i]
      }.reduce(:+).to_i.to_s # Store this in either integer or string.
    else
      @ms = input?
    end
  end

  # ========================================================================= #
  # === input?
  # ========================================================================= #
  def input?
    @input
  end

  # ========================================================================= #
  # === ms?
  # ========================================================================= #
  def ms?
    @ms
  end; alias ms ms? # === ms

  # ========================================================================= #
  # === set_ms
  # ========================================================================= #
  def set_ms(i = 0.0)
    i = i.to_f if i.is_a? String
    @ms = i
  end

  # ========================================================================= #
  # === add
  # ========================================================================= #
  def add(i) # Simply add some milliseconds
    set_ms(@ms.to_f + i.to_f)
  end

  # ========================================================================= #
  # === output_result_in_seconds
  # ========================================================================= #
  def output_result_in_seconds
    e @ms.to_f / 1000
  end

  # ========================================================================= #
  # === output_result
  # ========================================================================= #
  def output_result
    e @ms
  end

  # ========================================================================= #
  # === result
  # ========================================================================= #
  def result?
    @ms
  end; alias result result?

  # ========================================================================= #
  # === run                                                         (run tag)
  # ========================================================================= #
  def run
    determine_ms
    # output_result
  end

  # ========================================================================= #
  # === MultimediaParadise::LongFormatToMillisecondsConverter[]
  #
  # Usage example:
  #
  #   MultimediaParadise::LongFormatToMillisecondsConverter['00:33:32.70']
  #
  # ========================================================================= #
  def self.[](i)
    new(i).result? # This returns the result in milliseconds.
  end

end

# =========================================================================== #
# === MultimediaParadise.long_format_to_milliseconds
# =========================================================================== #
def self.long_format_to_milliseconds(i = ARGV)
  MultimediaParadise::LongFormatToMillisecondsConverter.new(i).result?
end

end
#  "01020:09:40,50 --> 01020:16:36,50",
if __FILE__ == $PROGRAM_NAME
  _ = MultimediaParadise::LongFormatToMillisecondsConverter.new(ARGV)
  _.output_result
  puts '  ^^^ Note: The above result is in milliseconds.'
  _.output_result_in_seconds
  puts '  ^^^ And now it is output in seconds.'
end # lfm 01:15:15.55
    # lfm 00:00:17,440