#!/usr/bin/ruby -w
# Encoding: UTF-8
# frozen_string_literal: true
# =========================================================================== #
# The input to this class must be in n milliseconds.
#
# Usage example:
#
#   MultimediaParadise::MillisecondsToLongFormatConverter.new(ARGV)
#
# =========================================================================== #
# require 'multimedia_paradise/misc/milliseconds_to_long_format_converter.rb'
# MultimediaParadise.milliseconds_to_long_format
# =========================================================================== #
module MultimediaParadise

class MillisecondsToLongFormatConverter # MillisecondsToLongFormatConverter

  alias e puts

  # ========================================================================= #
  # === DEFAULT_INPUT
  # ========================================================================= #
  DEFAULT_INPUT = 55.050

  # ========================================================================= #
  # === FORMAT_STRING
  #
  # This also "rounds" the result.
  # ========================================================================= #
  FORMAT_STRING = '%02d:%02d:%02d,%02d'

  # ========================================================================= #
  # === N_SEKUNDEN_IN_EINER_STUNDE
  # ========================================================================= #
  N_SEKUNDEN_IN_EINER_STUNDE = 3600

  # ========================================================================= #
  # === initialize
  # ========================================================================= #
  def initialize(
      optional_input = ARGV,
      report_result  = false,
      run_already    = true
    )
    reset
    @report_result = report_result
    set_n_milliseconds(
      optional_input
    )
    run if run_already
  end

  # ========================================================================= #
  # === reset
  # ========================================================================= #
  def reset
  end

  # ========================================================================= #
  # === set_n_milliseconds
  # ========================================================================= #
  def set_n_milliseconds(i = nil)
    i = DEFAULT_INPUT if i.nil?
    if i.is_a? Array
      if i.empty?
        i = DEFAULT_INPUT
      else
        i = i.first
      end
    end
    # ======================================================================= #
    # From this point on, we must have a String.
    # ======================================================================= #
    i = i.to_s.delete('_') if i.to_s.include? '_'
    i = i.to_i
    @n_milliseconds = i
  end

  # ========================================================================= #
  # === set_result
  # ========================================================================= #
  def set_result(i)
    @result = i
  end

  # ========================================================================= #
  # === do_conversion
  # ========================================================================= #
  def do_conversion
    input = (@n_milliseconds / 1000.0).to_s # Convert into seconds here.
    if input.include? '.'
      # We ensure that at least 3 characters are after the .
      splitted = input.split('.')
      splitted[1] << '0' if splitted[1].size < 3
      splitted[1] << '0' if splitted[1].size < 3
      input = splitted.join('.')
    end
    # Must have three numbers
    ms    = input[-3,3].to_s.delete('.')
    input = input[0, input.length-3]
    total_sekunden = input.to_i
    stunden = (total_sekunden.to_i / N_SEKUNDEN_IN_EINER_STUNDE).to_s
    total_sekunden = total_sekunden - ( stunden.to_i * N_SEKUNDEN_IN_EINER_STUNDE )
    minuten = total_sekunden.to_i / 60  
    total_sekunden =  total_sekunden.to_i - (minuten.to_i * 60)
    minuten = '0'+minuten.to_s  if minuten < 10
    total_sekunden = '0'+total_sekunden.to_s   if total_sekunden < 10
    _ = stunden+':'+minuten.to_s+':'+total_sekunden.to_s+','+ms
    _[0,0] = '0' if stunden.to_i < 10
    set_result _ 
    report_result if @report_result
  end

  # ========================================================================= #
  # === report_result
  # ========================================================================= #
  def report_result
    e @result
  end; alias report report_result # === report

  # ========================================================================= #
  # === verbose_report
  # ========================================================================= #
  def verbose_report
    e @n_milliseconds.to_s+' milliseconds correspond '+
      'to '+@result.to_s+' in long format.'
  end

  # ========================================================================= #
  # === result
  # ========================================================================= #
  def result?
    @result
  end; alias result result? # === result
  
  # ========================================================================= #
  # === run                                                         (run tag)
  # ========================================================================= #
  def run
    do_conversion
  end

  # ========================================================================= #
  # === MultimediaParadise::MillisecondsToLongFormatConverter.convert
  # ========================================================================= #
  def self.convert(i)
    new(i, false).result
  end

end

# =========================================================================== #
# === MultimediaParadise.milliseconds_to_long_format
#
# Usage example:
#
#   MultimediaParadise.milliseconds_to_long_format('15_05') # => "00:00:01,505"
#
# =========================================================================== #
def self.milliseconds_to_long_format(i = ARGV)
  MultimediaParadise::MillisecondsToLongFormatConverter.new(i).result?
end

end

if __FILE__ == $PROGRAM_NAME
  if ARGV.empty?
    test_this_data = %w(
      15_321
      215_321
      3_615_321
      83_615_321
      395_615_321
      31_057
    )
    test_this_data.each {|entry|
      _ = MultimediaParadise::MillisecondsToLongFormatConverter.new(entry, false)
      _.verbose_report
    }
  else
    _ = MultimediaParadise::MillisecondsToLongFormatConverter.new(ARGV, false)
    _.verbose_report
  end
end # rb milliseconds_to_long_format_converter.rb
    # mtl 15_321