#!/usr/bin/ruby -w
# Encoding: UTF-8
# frozen_string_literal: true
# =========================================================================== #
# === MultimediaParadise::MultimediaBase
#
# You can subclass from this class if you have a subclass that somehow has
# to deal with multimedia related aspects - that is audio and video. For
# example, class AudioBase will subclass from this class here specifically.
#
# This class will also contain audio-specific code, such as the method
# is_audio_song?().
#
# Usage example:
#
#   MultimediaParadise::MultimediaBase.new
#
# =========================================================================== #
# require 'multimedia_paradise/multimedia/base.rb'
# class Foobar < MultimediaParadise::MultimediaBase
# =========================================================================== #
require 'multimedia_paradise/base/base.rb'

module MultimediaParadise

class MultimediaBase < ::MultimediaParadise::Base # === MultimediaParadise::MultimediaBase

  # ========================================================================= #
  # === initialize
  # ========================================================================= #
  def initialize
    reset
  end

  # ========================================================================= #
  # === reset
  # ========================================================================= #
  def reset
    super()
  end

  # ========================================================================= #
  # === is_audio_file?
  # ========================================================================= #
  def is_audio_file?(i)
    ::MultimediaParadise.is_audio_file?(i)
  end; alias is_audio_song?    is_audio_file? # === is_audio_song?
       alias is_an_audio_file? is_audio_file? # === is_audio_file?

  # ========================================================================= #
  # === filter_for_audio_files
  #
  # We will filter for audio files - so we will keep only files that
  # are audio files.
  # ========================================================================= #
  def filter_for_audio_files(i)
    i.select {|file| is_audio_file?(file) }
  end

end; end