#!/usr/bin/ruby -w
# Encoding: UTF-8
# frozen_string_literal: true
# =========================================================================== #
# require 'multimedia_paradise/multimedia/cut_multimedia/evaluate_from_this_file.rb'
# =========================================================================== #
require 'multimedia_paradise/multimedia/base.rb'
require 'multimedia_paradise/multimedia/cut_multimedia/cut_multimedia.rb'

module MultimediaParadise

class CutMultimedia < ::MultimediaParadise::MultimediaBase # === MultimediaParadise::CutMultimedia

  require 'multimedia_paradise/toplevel_methods/e.rb'
  require 'multimedia_paradise/toplevel_methods/esystem.rb'
  require 'multimedia_paradise/toplevel_methods/merge_multimedia_file.rb'
  require 'multimedia_paradise/audio/wav_to_mp3.rb'

  # ========================================================================= #
  # === MultimediaParadise::CutMultimedia.evaluate_from_this_file
  #
  # This method can be used to read from a local file and use this file
  # as batch-instruction to cut at the specified position.
  # ========================================================================= #
  def self.evaluate_from_this_file(i)
    if i.is_a? Array
      i = i.first
    end
    i = i.to_s
    if File.exist? i
      file_content = File.readlines(i).map {|inner_line|
        inner_line.strip.delete(' ')
      }.reject {|line|
        line.start_with?('#') or line.strip.empty? # Ignore comments and empty lines.
      }
      all_multimedia_files = Dir['*'] - [i]
      sample = all_multimedia_files.sample
      _ = MultimediaParadise::CutMultimedia.new(nil, :dont_run_yet)
      _.set_work_on_this_file(sample)
      _.interactive_menu(file_content)
      _.do_cut
      # ===================================================================== #
      # Next, merge these files:
      # ===================================================================== #
      files_to_be_merged = Dir['cutted_*']
      result = MultimediaParadise.merge(files_to_be_merged)
      if File.exist? result
        MultimediaParadise.e 'Converting to .wav, then converting back to .mp3.'
        MultimediaParadise.esystem 'lame --decode -v '+result
        FileUtils.mv(
          result.delete_suffix(File.extname(result))+'.wav',
          'new_file.wav'
        )
        result = MultimediaParadise::Audio::WavToMp3.new('new_file.wav')
        if File.exist? 'new_file.mp3'
          MultimediaParadise.e 'The (new) file can be found at: '+
                                File.absolute_path('new_file.mp3')
          # ================================================================= #
          # And do some clean-up actions in this case:
          # ================================================================= #
          File.delete('new_file.wav') if File.exist? 'new_file.wav'
          File.delete('output.mp3')   if File.exist? 'output.mp3'
          Dir['cutted_*'].each {|this_file|
            File.delete(this_file)
          }
        else
          MultimediaParadise.e 'No file exists at new_file.mp3.'
        end
      end
    else
      MultimediaParadise.e 'No file exists at '+sfile(i)+'.'
    end
  end

end; end

if __FILE__ == $PROGRAM_NAME
  MultimediaParadise::CutMultimedia.evaluate_from_this_file(ARGV)
end # evaluatefromthisfile