#!/usr/bin/ruby -w
# Encoding: UTF-8
# frozen_string_literal: true
# =========================================================================== #
# require 'multimedia_paradise/multimedia/cut_multimedia/interactive_menu.rb'
# =========================================================================== #
require 'multimedia_paradise/multimedia/base.rb'

module MultimediaParadise

class CutMultimedia < ::MultimediaParadise::MultimediaBase # === MultimediaParadise::CutMultimedia

  # ========================================================================= #
  # === interactive_menu
  # ========================================================================= #
  def interactive_menu(
      i = @user_input
    )
    if i.is_a? Array
      i.each {|entry| interactive_menu(entry) }
    else
      work_on_this_file = @work_on_this_file.to_s
      first_argument = nil
      # ===================================================================== #
      # The last check is to prevent "1.5 - 6.86" from being chopped up.
      # ===================================================================== #
      if i and i.include?(' ') and !i.include?(' - ')
        splitted = i.split(' ')
        first_argument = splitted[1]
        all_arguments  = splitted[1..-1]
        cmd = splitted[0]
      else
        cmd = i
      end
      f = first_argument
      case cmd.to_s # case tag
      # ===================================================================== #
      # === run                                                     (cut tag)
      #
      # This is the main entry point for this class: it will commit towards
      # cutting the multimedia file at hand.
      # ===================================================================== #
      when 'run',
           'cut',
           'start',
           '1',
           /commit/,
           /cu/,
           /^do(_|-)?cut$/i
        do_cut_the_multimedia_file
      # ===================================================================== #
      # === set_start
      # ===================================================================== #
      when /^s ?\d+/,
           /^s(\d+:\d+:\d+.\d)/,
           /^start ?\d+/,
           'set_start' # "s1:32:48.2"
        define_start_point(i)
      # ===================================================================== #
      # === set_end
      # ===================================================================== #
      when /^e(\d{1,2}:?\d{1,2}:\d{1,2})\.?\d?/,
           /^e ?\d+/,
           /^end ?\d+/,
           'set_end' # "e1:32:48.2", see: http://rubular.com/r/NX5xENtGzB
        define_end_point(i)
      # ===================================================================== #
      # === 5.5
      #
      # This will match input such as: "5.5   -678.3".
      #
      # It will set both start and end positions.
      #
      # See the following entry at rubular.com:
      #
      #   https://rubular.com/r/APT8nRFhe6OPHq
      #
      # ===================================================================== #
      when /(\d+\.?\d*)\s*-\s*(\d+\.?\d*)/
        set_start($1)
        set_end($2)
      # ===================================================================== #
      # === Only numbers
      #
      # This entry point exists when the user only inputted numbers.
      #
      # The second regex checks for Floats specifically.
      # ===================================================================== #
      when /^\d+$/,
           /^\d+\.?\d*$/,
           /^\d+,?\d*$/
        user_did_input_only_numbers(i)
      # ===================================================================== #
      # === 06:11.6
      #
      # This entry point allows us to more easily use positions that
      # directly come from mplayer, e. g. "06:11.6" meaning "371.6"
      # seconds. This will be used for the start position of the
      # audio file at hand.
      # ===================================================================== #
      when /(\d{1,2}:\d{1,2}\.\d{0,1})$/ # See: http://rubular.com/r/bQMecScBBd
        determine_start_position($1.to_s.dup)
      # ===================================================================== #
      # === 5min
      # ===================================================================== #
      when /^(\d{0,2})min$/
        chop_off_the_first_n_minutes($1.to_s)
      # ===================================================================== #
      # === 6*60
      #
      # This entry point allows us to quickly calculate results for this.
      # ===================================================================== #
      when /^(\d+)\*(\d+)$/
        result = eval(
          $1.to_s.dup+' * '+$2.to_s.dup
        )
        e "#{i} is: #{sfancy(result)}"
      # ===================================================================== #
      # === end
      # ===================================================================== #
      when 'end',
           /^set(_|-)?end(_|-)?position$/,
           'end_position'
        self.end_position = :end
        e 'Setting end position to '+simp(end?.to_s)+rev+'.'
      # ===================================================================== #
      # === gui
      # ===================================================================== #
      when 'gui',
           /^-?-?gtk3?/i
        start_the_graphical_user_interface
      # ===================================================================== #
      # === file?
      # ===================================================================== #
      when 'file?',
           'audio?',
           'name?'
        e sfile(work_on_which_file?)
      # ===================================================================== #
      # === assign
      # ===================================================================== #
      when 'assign'
        assign(f)
      # ===================================================================== #
      # === split
      # ===================================================================== #
      when 'split',
           /split(-|_)?in(-|_)?half$/,
           /split(_|-)?audio(_|-)?file(_|-)?in(_|-)?half$/i,
           /half(-|_)?chop/,
           /chop(-|_)?in(-|_)?half/,
           'half',
           'halfy'
        split_audio_file_in_half
      # ===================================================================== #
      # === q
      #
      # Ways to exit this application. Only really applicable when in
      # the interactive mode.
      # ===================================================================== #
      when 'q',
           'exit',
           'quit',
           'break',
           'out',
           'stop',
           nil
        @may_we_exit = true
      # ===================================================================== #
      # === ends?
      # ===================================================================== #
      when 'ends?','end?',
           /^feedback_?all_?end_?points$/i
        feedback_all_end_points
      # ===================================================================== #
      # === --last-file
      #
      # Usage example:
      #
      #   caudio --last-file
      #
      # ===================================================================== #
      when /^-?-?last(_|-)?file$/i,
           /^-?-?try(_|-)?to(_|-)?use(_|-)?the(_|-)?last(_|-)?file$/i
        try_to_use_the_last_file
      # ===================================================================== #
      # === chop_into_segments_of_n_seconds_size
      # ===================================================================== #
      when /chop(_|-)?into(_|-)?segments(_|-)?of(_|-)?n(_|-)?seconds(_|-)?size$/,
           'segments'
        chop_into_segments_of_n_seconds_size(f) # Create n seconds segments.
      # ===================================================================== #
      # === intersect
      #
      # Intersect allows us to cut away a "central" segment, aka a segment
      # that is between two other parts that are interesting to us.
      #
      # In other words - via intersect we REMOVE a part of the audio file,
      # and only RETAIN the two parts that are on the left border and
      # on the right border.
      # ===================================================================== #
      when 'intersect',
           /^remove_?middle_?segment/,
           /^chop_?out/,
           /^segment$/i
        intersect(f)
      # ===================================================================== #
      # === internal_setpoints
      # ===================================================================== #
      when /^internal(_|-)?setpoints$/,
           'isetpoints'
        '' # This is obviously unfinished.
      # ===================================================================== #
      # === lchop
      # ===================================================================== #
      when 'lchop', /left_?chop/,'-chop',/left_?chop_?off/
        left_chop_off(all_arguments) # Chop at the "left" side of a .mp3 file.
      # ===================================================================== #
      # === points?
      # ===================================================================== #
      when 'points?','p?',/report_?points/
        report_points
      # ===================================================================== #
      # === size?
      # ===================================================================== #
      when 'size?','duration?','size','duration','length','d?','length?',
           /report_?duration_?of_?multimedia_?file/,
           /file_?length\??/,
           /duration\??/
        report_duration_of_multimedia_file
      # ===================================================================== #
      # === cut_and_overwrite
      # ===================================================================== #
      when /cut_?and_?overwrite/,/cut_?then_?overwrite/,'superwrite',
           'cwrite',
           /cut_?then_?o/,
           'cuto'
        start_cutting
        overwrite_old_file(:be_verbose)
        # =================================================================== #
        # Since we overwrote it, no need for error reporting here.
        # =================================================================== #
        report_that_we_are_finished_now(:no_error_reporting)
      # ===================================================================== #
      # === chop_off
      # ===================================================================== #
      when /^chop(_|-)?off$/,
           'chopo','chopper','chop'
        chop_off(all_arguments)
      # ===================================================================== #
      # === intersect?
      # ===================================================================== #
      when 'intersect?',
           /^show(_|-)?how(_|-)?to(_|-)?use(_|-)?intersect$/i
        show_how_to_use_intersect
      # ===================================================================== #
      # === player?
      # ===================================================================== #
      when 'player?',
           /^report(_|-)?which(_|-)?player(_|-)?we(_|-)?will(_|-)?use$/i
        report_which_player_we_will_use
      # ===================================================================== #
      # === exist
      # ===================================================================== #
      when 'exist?',
           'exist',
           /^report(_|-)?does(_|-)?the(_|-)?main(_|-)?file(_|-)?exist\??$/i
        report_does_the_main_file_exist?
      # ===================================================================== #
      # === files?
      # ===================================================================== #
      when 'files?',
           'files',
           /^show(_|-)?audio(_|-)?files$/i,
           'output?'
        show_audio_files
      # ===================================================================== #
      # === cut_then_exit
      # ===================================================================== #
      when /^cut(_|-)?then(_|-)?exit$/,
           /^cut(_|-)?and(_|-)?exit$/
        start_cutting
        report_that_we_are_finished_now
        we_may_quit
      # ===================================================================== #
      # === feed
      # ===================================================================== #
      when 'feed',
           'feedback',
           'setpoints?',
           'feedback?',
           'feed?',
           'pos?',
           'positions?',
           'data?',
           'dataset?'
        feedback_cutting
      # ===================================================================== #
      # === overwrite
      # ===================================================================== #
      when 'overwrite',
           /re(_|-)?merge/,
           'rename',
           'original',
           'overwrite_old_file'
        overwrite_old_file(:be_verbose)
        do_reset_all_values_to_their_default_values
      # ===================================================================== #
      # === done
      # ===================================================================== #
      when /^done$/i
        modify_the_main_file_by_prepending_the_upcased_string_done_to_it
      # ===================================================================== #
      # === output_files?
      # ===================================================================== #
      when 'output_files?',
           'outputfiles?' # Debugging here.
        p output_file?
      # ===================================================================== #
      # === clear
      # ===================================================================== #
      when 'clear',
           'reset',
           'reset_everything'
        do_reset_all_values_to_their_default_values
      # ===================================================================== #
      # === delete_the_original_file
      #
      # Only use this entry point if you really want to delete the audio
      # file at hand.
      # ===================================================================== #
      when /^delete(_|-)?the(_|-)?original(_|-)?file$/,
           /^delete(_|-)?old(_|-)?file$/,
           /^remove$/,
           /^remove(_|-)?file$/,
           /^delete(_|-)?file$/,
           'delete',
           /^delete(_|-)?the(_|-)?audio(_|-)?file$/
        delete_the_original_file
      # ===================================================================== #
      # === delete_individual_audio_files
      # ===================================================================== #
      when /^delete(_|-)?individual(_|-)?audio(_|-)?files$/i
        delete_individual_audio_files
      # ===================================================================== #
      # === disable
      # ===================================================================== #
      when 'disable'
        @we_are_allowed_to_run_system_command = false
      # ===================================================================== #
      # === merge
      # ===================================================================== #
      when 'merge',
           'join'
        merge_created_files
      # ===================================================================== #
      # === pwd?
      # ===================================================================== #
      when 'pwd?',
           'pwd',
           'PWD',
           /^report(_|-)?pwd$/i
        report_pwd
      # ===================================================================== #
      # === open
      # ===================================================================== #
      when 'open',
           /^open(_|-)?this(_|-)?file(_|-)?in(_|-)?editor$/i
        open_this_file_in_editor
      # ===================================================================== #
      # === history?
      # ===================================================================== #
      when /^history\??/,
           /^show(_|-)?history$/i,
           'hist',
           'h',
           'his?',
           'input?'
        show_history
      # ===================================================================== #
      # === ll
      # ===================================================================== #
      when 'll',
           'ls',
           'l'
        Dir['*'].each {|entry| e entry }
      # ===================================================================== #
      # === report_status
      # ===================================================================== #
      when /^report(_|-)?status$/,
           '?',
           '??',
           'info?',
           'debug?'
        report_status
      # ===================================================================== #
      # === start?
      # ===================================================================== #
      when 'start?'
        feedback_start_cutting_at
      # ===================================================================== #
      # === mpv
      # ===================================================================== #
      when 'mpv',
           /^do(_|-)?use(_|-)?mpv$/i,
           /^use(_|-)?mpv$/i
        do_use_mpv
      # ===================================================================== #
      # === mplayer
      # ===================================================================== #
      when 'mplayer',
           /^do(_|-)?use(_|-)?mplayer$/i,
           /^use(_|-)?mplayer$/i
        do_use_mplayer
      # ===================================================================== #
      # === show_audio_files
      # ===================================================================== #
      when /show_audio_files/,
           'audiofiles?'
        show_audio_files
      # ===================================================================== #
      # === report_half_the_duration
      # ===================================================================== #
      when 'half?',
           'h?',
           '1/2',
           /^report(_|-)?half(_|-)?the(_|-)?duration$/i
        report_half_the_duration
      # ===================================================================== #
      # === USE_THIS_PLAYER
      # ===================================================================== #
      when /USE_THIS_PLAYER/i,
           /report_which_player_we_will_use/
        report_which_player_we_will_use
      # ===================================================================== #
      # === play
      # ===================================================================== #
      when 'play',
           'mplay',
           'pla','pl','2','p',
           'lp',
           /^play(_|-)?the(_|-)?audio(_|-)?file$/,
           'player',
           'lpay',
           'pkl',
           'mpa'
        play_the_audio_file
      # ===================================================================== #
      # === trim
      #
      # This will trim from the left-end, left-hand side, aka from the
      # beginning.
      # ===================================================================== #
      when /^-?-?trim/i,
           /^-?-?left(-|_| )?chop/i,
           /^-?-?lchop/i
        if first_argument
          trim_n_seconds = first_argument
        else
          e 'Trim n seconds from the start:'
          trim_n_seconds = $stdin.gets.chomp
        end
        @output_file = "output_#{File.basename(work_on_this_file)}"
        _ = 'ffmpeg -y -i '+work_on_this_file+' -ss '+trim_n_seconds+' '\
            '-vcodec copy -acodec copy '+
            @output_file
        e
        esystem _
        e
      # ===================================================================== #
      # === help
      # ===================================================================== #
      when /-?-?help$/i,
           'help?',
           'hlep',
           'status',
           'status?',
           'stat',
           'stat?',
           'stats?',
           'chelp',
           /show_?help_?options/
        show_help
      else # else tag
        case i # case tag
        when 's'
          define_start_point(f)
        when 'e'
          define_end_point(f)
        else
          if i.include? '-' # assume we specified start and end position.
            splitted = i.split('-')
            set_start(splitted.first)
            set_end(splitted[1])
          elsif i.include? ' '
            splitted = i.split ' '
            f = splitted[1] # first argument.
            case splitted.first
            when 'calculate'
              calculate(f)
            end
          elsif i.empty?
            e # Make a newline in this scase.
          else
            e rev+
              'No action is associated with the given '\
              'input `'+steelblue(i.to_s)+'`.'
          end
        end
      end
    end
  end; alias imenu interactive_menu # === imenu

end; end