#!/usr/bin/ruby -w
# Encoding: UTF-8
# frozen_string_literal: true
# =========================================================================== #
# === MultimediaParadise::InteractiveShell
#
# This is an interactive shell, mostly just for demo-purpose.
#
# It will allow us to quickly convert between different formats. 
# =========================================================================== #
require 'multimedia_paradise/base/base.rb'

module MultimediaParadise

class InteractiveShell < ::MultimediaParadise::Base

  begin
    require 'rcfiles'
  rescue LoadError; end

  # ========================================================================= #
  # === initialize
  # ========================================================================= #
  def initialize(
      run_already = true
    )
    reset
    run if run_already
  end

  # ========================================================================= #
  # === reset
  # ========================================================================= #
  def reset
    super()
  end

  # ========================================================================= #
  # === obtain_user_input
  # ========================================================================= #
  def obtain_user_input
    $stdin.gets.chomp
  end

  # ========================================================================= #
  # === menu                                                       (menu tag)
  # ========================================================================= #
  def menu(i)
    case i
    when 'pwd'
      e Dir.pwd
    when 'q','quit','exit'
      :do_exit
    else
      if ::Rcfiles::DirectoryAliases.does_include? i
        cd ::Rcfiles::DirectoryAliases[i]
      end if Object.const_defined? :Rcfiles
    end
  end

  # ========================================================================= #
  # === run
  # ========================================================================= #
  def run
    loop {
      _ = obtain_user_input
      exit if menu(_) == :do_exit
    }
  end

end

# =========================================================================== #
# === MultimediaParadise.shell
# =========================================================================== #
def self.shell
  InteractiveShell.new
end

end

if __FILE__ == $PROGRAM_NAME
   MultimediaParadise.shell
end