#!/usr/bin/ruby -w
# Encoding: UTF-8
# frozen_string_literal: true
# =========================================================================== #
# === MultimediaParadise::MergeMultimedia
#
# This file will delegate towards the multimedia merge functionality
# which is available in MultimediaConversions.merge().
#
# Usage examples:
#
#   MultimediaParadise::MergeMultimedia.new
#   mergemultimedia
#
# =========================================================================== #
# require 'multimedia_paradise/multimedia/merge_multimedia.rb'
# MultimediaParadise::MergeMultimedia.new
# =========================================================================== #
require 'multimedia_paradise/base/base.rb'

module MultimediaParadise

class MergeMultimedia < ::MultimediaParadise::Base # === MultimediaParadise::MergeMultimedia

  require 'multimedia_paradise/toplevel_methods/merge_multimedia_file.rb'

  # ========================================================================= #
  # === initialize
  # ========================================================================= #
  def initialize(
      merge_these_files = nil,
      run_already       = true
    )
    reset
    set_merge_these_files(
      merge_these_files
    )
    run if run_already
  end

  # ========================================================================= #
  # === reset                                                     (reset tag)
  # ========================================================================= #
  def reset
    super()
    # ======================================================================= #
    # === @output_file
    # ======================================================================= #
    @output_file = nil
  end

  # ========================================================================= #
  # === set_merge_these_files
  # ========================================================================= #
  def set_merge_these_files(
      i = ''
    )
    i = i.dup if i
    case i
    when '*'
      i = Dir['*'].select {|entry| is_this_a_multimedia_file?(entry) }
    end
    i.compact! if i.is_a? Array
    @merge_these_files = i
  end

  # ========================================================================= #
  # === check_whether_the_files_exist
  # ========================================================================= #
  def check_whether_the_files_exist(
      these_files = @merge_these_files
    )
    these_files.each {|entry|
      if File.exist? entry
      else
        ewarn 'The file at `'+sfile(entry)+'` does not exist.'
      end
    }
  end

  # ========================================================================= #
  # === which_files?
  # ========================================================================= #
  def which_files?
    @merge_these_files
  end

  # ========================================================================= #
  # === set_output_file
  # ========================================================================= #
  def set_output_file(i)
    @output_file = i
  end

  # ========================================================================= #
  # === output_file?
  # ========================================================================= #
  def output_file?
    @output_file
  end; alias output? output_file? # === output?

  # ========================================================================= #
  # === merge
  # ========================================================================= #
  def merge(
      these_files = which_files?
    )
    if these_files.is_a? Array
      check_whether_the_files_exist
      result = MultimediaParadise.merge_multimedia_file(these_files) # bl conversions.rb
      set_output_file(result)
    end
  end

  # ========================================================================= #
  # === run                                                         (run tag)
  # ========================================================================= #
  def run
    merge
  end

end; end

if __FILE__ == $PROGRAM_NAME
  MultimediaParadise::MergeMultimedia.new(ARGV)
end # merge_multimedia
    # merge_multimedia a.webm b.webm