#!/usr/bin/ruby -w
# Encoding: UTF-8
# frozen_string_literal: true
# =========================================================================== #
# === MultimediaParadise::SimulateYoutubePlaylist
#
# This class can simulate a youtube playlist. It can also generate a
# playlist that can be loaded into other applications.
#
# Usage example:
#
#   MultimediaParadise::SimulateYoutubePlaylist.new
#
# =========================================================================== #
# require 'multimedia_paradise/multimedia/simulate_youtube_playlist.rb'
# =========================================================================== #
require 'multimedia_paradise/base/base.rb'

module MultimediaParadise

class SimulateYoutubePlaylist < ::MultimediaParadise::Base # === MultimediaParadise::SimulateYoutubePlaylist

  # ========================================================================= #
  # === NAMESPACE
  # ========================================================================= #
  NAMESPACE = inspect

  # ========================================================================= #
  # === FILE_YOUTUBE_PLAYLIST
  # ========================================================================= #
  FILE_YOUTUBE_PLAYLIST =
    ::MultimediaParadise.project_base_dir?+
    'yaml/youtube_playlist/youtube_playlist.yml'

  # ========================================================================= #
  # === initialize
  # ========================================================================= #
  def initialize(
      commandline_arguments = nil,
      run_already           = true
    )
    reset
    set_commandline_arguments(
      commandline_arguments
    )
    run if run_already
  end

  # ========================================================================= #
  # === reset                                                     (reset tag)
  # ========================================================================= #
  def reset
    super()
    # ======================================================================= #
    # === @inner_hash
    # ======================================================================= #
    @inner_hash = {}
    # ======================================================================= #
    # === :shall_we_download_these_files
    # ======================================================================= #
    @inner_hash[:shall_we_download_these_files] = true
    # ======================================================================= #
    # === :work_on_this_subsection
    # ======================================================================= #
    set_work_on_this_subsection(:eurodance)
set_work_on_this_subsection(:married_with_children)
  end

  # ========================================================================= #
  # === set_work_on_this_subsection
  # ========================================================================= #
  def set_work_on_this_subsection(i)
    @inner_hash[:work_on_this_subsection] = i.to_s
  end

  # ========================================================================= #
  # === shall_we_download_these_files?
  # ========================================================================= #
  def shall_we_download_these_files?
    @inner_hash[:shall_we_download_these_files]
  end

  # ========================================================================= #
  # === opnn
  # ========================================================================= #
  def opnn(&block)
    super(NAMESPACE, &block)
  end

  # ========================================================================= #
  # === sanitize_dataset
  # ========================================================================= #
  def sanitize_dataset(hash)
    hash.each_pair {|key, value|
      value.map! {|remote_URL|
        unless remote_URL.start_with? 'http'
          remote_URL = beautiful_url(remote_URL)
        end
        remote_URL
      }
    }
    return hash
  end

  begin
    require 'open'
  rescue LoadError; end
  # ========================================================================= #
  # === work_on_this_hash
  # ========================================================================= #
  def work_on_this_hash(
      hash,
      work_on_this_subsection = @inner_hash[:work_on_this_subsection]
    )
    work_on_this_subsection = work_on_this_subsection.to_s
    if hash.has_key? work_on_this_subsection
      work_on_these_remote_URLs = hash[work_on_this_subsection]
      e rev+
        'This playlist contains '+
        steelblue(work_on_these_remote_URLs.size.to_s)+
        ' entries.'
      # ===================================================================== #
      # Consider downloading these remote files next:
      # ===================================================================== #
      if shall_we_download_these_files?
        e 'Now downloading these files.'
        cd log_dir?
        e 'Trying to create a playlist next:'
        require 'multimedia_paradise/audio/create_m3u_playlist.rb'
        MultimediaParadise.create_m3u_playlist(
          work_on_these_remote_URLs
        )
        mkdir 'youtube_playlist' unless File.directory? 'youtube_playlist'
        cd 'youtube_playlist'
        e 'Working from the directory `'+sdir(return_pwd)+'`.'
        work_on_these_remote_URLs.each_with_index {|remote_URL, index| index += 1
          e 'Working on the remote URL `'+steelblue(remote_URL)+'` next.'
          download_command = "youtube-dl #{remote_URL}"
          e lightblue(download_command)
          result = `#{download_command}`
          # ================================================================= #
          # Try to rename it next:
          # ================================================================= #
          if result.include? 'Merging formats into'
            splitted = result.split("\n")
            new_filename = splitted.select {|line|
              line.include? 'Merging formats into '
            }
            new_filename = new_filename.first if new_filename.is_a? Array
            filename     = '"'+new_filename.dup+"'"
            regex_to_use =
              /(Merging formats into) "([-A-Za-z0-9._ ]+)"/ # See: https://rubular.com/r/uLTQUI5FbxDssJ
            new_filename =~ regex_to_use
            if $2
              filename = $2.to_s.dup
              new_filename = '00'+index.to_s+'_'+$2.to_s.
                             tr(' ','_')
            end
            new_filename.delete!("'")
            new_filename.squeeze!('_')
            unless File.exist? new_filename
              e 'Now renaming from '+
                sfile(filename)+' to '+
                sfile(new_filename)+
                '.'
              rename_file(filename, new_filename)
            end
          end
        }
      end
      work_on_these_remote_URLs.each {|remote_URL|
        e sfancy(remote_URL)
        Open.in_browser(remote_URL) { :do_not_open_in_the_background }
      }
    else
      e 'No subsection called '+work_on_this_subsection+' found.'
    end
  end

  # ========================================================================= #
  # === run                                                         (run tag)
  # ========================================================================= #
  def run
    if File.exist? FILE_YOUTUBE_PLAYLIST
      dataset = YAML.load_file(FILE_YOUTUBE_PLAYLIST)
      # ===================================================================== #
      # First sanitize the dataset.
      # ===================================================================== #
      dataset = sanitize_dataset(dataset)
      # ===================================================================== #
      # Now we assume the dataset has been sanitized. Delegate to another
      # method next.
      # ===================================================================== #
      work_on_this_hash(dataset)
    end
  end

  # ========================================================================= #
  # === MultimediaParadise::SimulateYoutubePlaylist[]
  # ========================================================================= #
  def self.[](i = '')
    new(i)
  end

end; end

if __FILE__ == $PROGRAM_NAME
  MultimediaParadise::SimulateYoutubePlaylist.new(ARGV)
end # simulateyoutubeplaylist