#!/usr/bin/ruby -w
# Encoding: UTF-8
# frozen_string_literal: true
# =========================================================================== #
# === MultimediaParadise::Sinatra
# =========================================================================== #
# require 'multimedia_paradise/sinatra/app.rb'
# MultimediaParadise.start_sinatra
# MultimediaParadise.run_app
# =========================================================================== #
module MultimediaParadise

begin

require 'sinatra'

class Sinatra < ::Sinatra::Base

  require 'multimedia_paradise/constants/encodings.rb'
  require 'multimedia_paradise/toplevel_methods/files_and_directories.rb'
  require 'multimedia_paradise/toplevel_methods/n_local_videos.rb'
  require 'multimedia_paradise/toplevel_methods/output_directory.rb'
  require 'multimedia_paradise/toplevel_methods/return_all_video_files.rb'

  begin
    require 'beautiful_url'
  rescue LoadError; end

  begin
    require 'open-uri'
  rescue LoadError; end

  begin
    require 'json'
  rescue LoadError; end

  include Colours::E
  include Colours
  require 'multimedia_paradise/sinatra/embeddable_interface.rb'
  include MultimediaParadise::EmbeddableInterface

  # require 'sinatra/reloader'
  # register ::Sinatra::Reloader

  configure do
    mime_type :avi, 'video/avi'
  end

  set :public_folder, BASE_DIR_TO_THE_REALVIDS
  # available_simpsons_directories = Dir['/home/x/video/Cartoons/*'].select {|entry|
  #   File.directory?(entry)
  # }.map {|inner_entry| "#{inner_entry}/".squeeze('/') }

  # @random_simpsons_directory = available_simpsons_directories.sample
  # Sinatra does not allow for multiple public folder directories.
  # set :public_folder, '/home/x/video/Cartoons/Simpsons-03/'
  # set :public_folder, '/home/x/video/Cartoons/Simpsons-04/'
  # set :public_folder, '/home/x/video/Cartoons/Simpsons-05/'
  # set :public_folder, '/home/x/video/Cartoons/Simpsons-06/'
  # set :public_folder, '/home/x/video/Cartoons/Simpsons-08/'
  # set :public_folder, '/home/x/video/Cartoons/Simpsons-09/'
  # set :public_folder, '/home/x/video/Cartoons/Simpsons-11/'
  # set :public_folder, '/home/x/video/Cartoons/Simpsons-12/'
  # set :public_folder, '/home/x/video/Cartoons/Simpsons-13/'
  # set :public_folder, '/home/x/video/Cartoons/Simpsons-14/'
  # set :public_folder, '/home/x/video/Cartoons/Simpsons-15/'
  # set :public_folder, '/home/x/video/Cartoons/Simpsons-16/'
  # set :public_folder, '/home/x/video/Cartoons/Simpsons-18/'
  # set :public_folder, @random_simpsons_directory
  # ^^^ Unfortunately the above does not work - sinatra wants only
  #     one public directory.

  # ========================================================================= #
  # === MultimediaParadise::Sinatra.simpsons_directory?
  # ========================================================================= #
  def self.simpsons_directory?
    @random_simpsons_directory
  end

  set :port, USE_THIS_PORT

  # ========================================================================= #
  # Try to enable sessions:
  # ========================================================================= #
  enable :sessions

  # ========================================================================= #
  # === before                                                     (before tag)
  # ========================================================================= #
  before {
    content_type(:html, 'charset' => USE_THIS_ENCODING )
  }

  # ========================================================================= #
  # === IS_ROEBE
  # ========================================================================= #
  IS_ROEBE = true

  # ========================================================================= #
  # === initialize
  # ========================================================================= #
  def initialize
    super()
    reset
    if File.directory? ::MultimediaParadise.log_dir?
      MultimediaParadise.cd(::MultimediaParadise.log_dir?)
    end
    if IS_ROEBE
      require 'open'
      # ===================================================================== #
      # Tell us which port will be used:
      # ===================================================================== #
      target = "http://localhost:#{USE_THIS_PORT}/"
      e sfancy(target)
      Thread.new {
        sleep 1.0
        Open.in_browser(target)
      }
    end
  end

  # ========================================================================= #
  # === reset
  # ========================================================================= #
  def reset
  end

  # ========================================================================= #
  # === /view                                                     (view tag)
  #
  # http://localhost:5580/view
  # ========================================================================= #
  get('/view') {
    return_view
  }

  # ========================================================================= #
  # === /n_local_videos
  #
  # http://localhost:5580/n_local_videos
  # ========================================================================= #
  get('/n_local_videos') {
    return_n_local_videos
  }

  # ========================================================================= #
  # === /view                                                      (view tag)
  #
  # http://localhost:5580/view/php
  # ========================================================================= #
  get('/view/*'){
    # use_these_params = params[:splat]
    # if params.has_key?('user_input')
    #   use_these_params = params.fetch('user_input')
    # end
    result = ''.dup
    result << html_and_body_start
    result << 'Hello world!'
    result
  }

  # ========================================================================= #
  # === file exists
  #
  # This one must come last, so that the earlier APIs continue to work.
  # If the file exists as-is then this is not necessary, though. But
  # we want to allow something like "027" or "27" as well.
  # Actually, it does not have to come last AT ALL. http://localhost:5580/027_StarTrekII.mp4
  #
  # The <video> element allows us to embed video files into an HTML, very
  # similar to the way images are embedded.
  #
  # Usage examples:
  #
  #   http://localhost:5580/file_exists/252
  #   http://localhost:5580/file_exists/229_Ronal_the_Barbarian.mp4
  #
  # ========================================================================= #
  get('/file_exists/*') {
    _ = params[:splat]
    _ = _.join(' ').strip if _.is_a? Array
    result = ''.dup
    result << html_and_body_start
    if _ =~ /^\d+/i # If it is a number.
      dataset = YAML.load_file(MultimediaParadise.file_video_collection)
      result = MultimediaParadise.return_filename_of_this_video_file(_.to_i) # => "024_Convoy"
      if result
        file_location = Dir["*#{result}*"]
        file_location = file_location.first if file_location.is_a? Array
      end
      _ = _.to_i
      if dataset.has_key?(_)
        result << p('Is this available: yes')
        result << p('Does the file at <b>'+file_location.to_s+
                    '</b> exist: '+File.exist?(file_location).to_s)
        result << '<video controls autoplay><source src="'+File.basename(file_location)+'"></video>'
      else
        result << p('Is this available: no')
      end
    end
    result << return_to_root_link
    result
  }

  # ========================================================================= #
  # === / Entry point
  #
  # This is the root of our web-application.
  # ========================================================================= #
  get('/') {
    return_main_index
  }

  # ========================================================================= #
  # === /available_simpsons_files
  #
  # http://localhost:5580/available_simpsons_files
  # ========================================================================= #
  get('/available_simpsons_files') {
    return_available_simpsons_files
  }

  # ========================================================================= #
  # === /available_video_files
  #
  # http://localhost:5580/available_video_files
  # ========================================================================= #
  get('/available_video_files') {
    return_available_video_files
  }

  Dir[BASE_DIR_TO_THE_REALVIDS+'*'].each {|this_route|
    get('/'+File.basename(this_route)) {
      result = ''.dup
      result << html_and_body_start
      send_file(this_route)
    }
  }

  # ========================================================================= #
  # === /listing
  #
  # This is definitely shorter than the other more verbose variant.
  #
  # http://localhost:5580/listing
  # ========================================================================= #
  get('/listing') {
    return_listing
  }

  # ========================================================================= #
  # === /random_video
  # ========================================================================= #
  get('/random_video') {
    return_random_video
  }

  # ========================================================================= #
  # === /play
  # ========================================================================= #
  get('/play') {
    return_play_splat
  }

  # ========================================================================= #
  # === /play/*                                                    (play tag)
  #
  # http://localhost:5580/play
  # http://localhost:5580/play/003
  # http://localhost:5580/play/228_Apocalypto.mp4
  # ========================================================================= #
  get('/play/*') {
    return_play_splat
  }

  require 'multimedia_paradise/toplevel_methods/to_mp4.rb'
  # ========================================================================= #
  # === /to_mp4/*                                                 (play tag)
  #
  # Usage example:
  #
  #   http://localhost:5580/to_mp4/223_Conan_the_Barbarian.avi
  #
  # ========================================================================= #
  get('/to_mp4/*') {
    _ = params[:splat]
    _ = _.join(' ').strip if _.is_a? Array
    target = BASE_DIR_TO_THE_REALVIDS+_
    result = MultimediaParadise.to_mp4(target)
    if File.exist? result
      Cyberweb::HtmlTemplate[
        title: result,
        body: 'The file is now available at: '+result,
        body_css_style: 'font-size: 1.2em;',
        css_classes: 'darkblue'
      ].to_s
    end
  }

end

# =========================================================================== #
# === MultimediaParadise.run_app
# =========================================================================== #
def self.run_app
  MultimediaParadise::Sinatra.run!
end; self.instance_eval { alias start_sinatra run_app } # === MultimediaParadise.start_sinatra

end

rescue LoadError

end

if __FILE__ == $PROGRAM_NAME
  MultimediaParadise.run_app
end # multimedia_paradise_run_app