#!/usr/bin/ruby -w
# Encoding: UTF-8
# frozen_string_literal: true
# =========================================================================== #
# === MultimediaParadise::Statistics::Video
#
# This class will show some statistical information about the
# videos.
#
# Usage example:
#
#   MultimediaParadise::Statistics::Video.new
#
# =========================================================================== #
# require 'multimedia_paradise/video/encode_video'
# =========================================================================== #
require 'multimedia_paradise/base/base.rb'

module MultimediaParadise

module Statistics

class Video < ::MultimediaParadise::Base # === MultimediaParadise::Statistics::Video

  require 'multimedia_paradise/video/store_available_video_files.rb'

  # ========================================================================= #
  # === initialize
  # ========================================================================= #
  def initialize(
      i           = ARGV,
      run_already = true
    )
    reset
    set_commandline_arguments(i)
    run if run_already
  end

  # ========================================================================= #
  # === reset                                                     (reset tag)
  # ========================================================================= #
  def reset
    super()
    infer_the_namespace
  end

  # ========================================================================= #
  # === run                                                         (run tag)
  # ========================================================================= #
  def run
    opne 'Next showing some information about the local video files:'
    all_video_files = Dir['/home/x/video/**/**'].select {|entry|
      is_a_video_file?(entry)
    }
    e 'There are a total of '+steelblue(all_video_files.size)+
      ' video files existing locally.'
    e
    e '  How many real videos exist? '.ljust(36)+steelblue(
        all_video_files.select {|inner_entry|
          inner_entry.include?('/Realvids')
        }.size.to_s.rjust(3)
      )
    e '  How many real life videos exist? '.ljust(36)+steelblue(
        all_video_files.select {|inner_entry|
          inner_entry.include?('/Reallife')
        }.size.to_s.rjust(3)
      )
    e '  How many MMA videos exist? '.ljust(36)+steelblue(
        all_video_files.select {|inner_entry|
          inner_entry.include?('/MMA')
        }.size.to_s.rjust(3)
      )
    # ======================================================================= #
    # Delegate to class MultimediaParadise::StoreAvailableVideoFiles next.
    # ======================================================================= #
    MultimediaParadise::StoreAvailableVideoFiles.new
    e
  end

end; end; end

if __FILE__ == $PROGRAM_NAME
  MultimediaParadise::Statistics::Video.new(ARGV)
end # videostatistics