#!/usr/bin/ruby -w
# Encoding: UTF-8
# frozen_string_literal: true
# =========================================================================== #
# The idea for this file is to collect codec-related code. 
# =========================================================================== #
# require 'multimedia_paradise/toplevel_methods/codecs.rb'
# =========================================================================== #
module MultimediaParadise

  require 'multimedia_paradise/toplevel_methods/esystem.rb'

  # ========================================================================= #
  # === MultimediaParadise.codec_conversion
  #
  # This method can be used to convert, e. g. from a .mp3 file to a
  # .ogg file. Note that this does not work right now (November 2020).
  #
  # In raw ffmpeg this would be like so:
  #
  #   ffmpeg -i input.mp3 -c:a libvorbis output.ogg
  #
  # Usage example:
  #
  #   MultimediaParadise.codec_conversion
  #
  # ========================================================================= #
  def self.codec_conversion(
      of_this_input_file,
      use_this_codec = :libvorbis
    )
    if of_this_input_file.is_a? Array
      of_this_input_file = of_this_input_file.join(' ').strip
    end
    output_file = of_this_input_file.
                  sub(/#{File.extname(of_this_input_file)}/,'')+
                  '.ogg'
    _ = 'ffmpeg -i '+of_this_input_file+' -c:a '+
         use_this_codec.to_s+
         output_file
    e
    esystem _
    e
  end

end

if __FILE__ == $PROGRAM_NAME
  MultimediaParadise.codec_conversion(ARGV)
end