#!/usr/bin/ruby -w
# Encoding: UTF-8
# frozen_string_literal: true
# =========================================================================== #
# require 'multimedia_paradise/toplevel_methods/create_video_from_this_audio.rb'
# =========================================================================== #
require 'multimedia_paradise/toplevel_methods/e.rb'
require 'multimedia_paradise/toplevel_methods/esystem.rb'
require 'multimedia_paradise/toplevel_methods/is_image_file.rb'

module MultimediaParadise

  # ========================================================================= #
  # === MultimediaParadise.create_video_from_this_audio
  #
  # This method can turn a, for example, .mp3 file into a video with
  # a static image.
  #
  # The second argument should be the image that you wish to show for
  # the video. If this image can NOT be found then the method will
  # try to find any other image file in the current directory - this
  # is the behaviour I needed, so it was added. Ideally provide the
  # image, though. 
  # ========================================================================= #
  def self.create_video_from_this_audio(
      this_audio_file = ARGV,
      use_this_image  = 'CAT.png'
    )
    if this_audio_file.is_a? Array
      this_audio_file.each {|entry|
        MultimediaParadise.create_video_from_this_audio(entry)
      }
    else
      unless File.exist? use_this_image
        e "No file called `#{use_this_image}` exists."
        e 'Trying to find any other image in this directory next.'
        all_image_files = Dir['*'].select {|entry| is_an_image_file?(entry) }
        # =================================================================== #
        # Must check whether it is empty or not.
        # =================================================================== #
        if all_image_files.empty?
          e 'No image could be found, thus no image was supplied. This is'
          e 'not allowed, so we exit here.'
          return
        else
          # ================================================================= #
          # Ok, use that image in this case - the first one.
          # ================================================================= #
          use_this_image = all_image_files.first.to_s
        end
      end
      output_file = this_audio_file.sub(/#{File.extname(this_audio_file)}/,'')+'.mp4'
      _ = 'ffmpeg -loop 1 '\
          '-i '+use_this_image.to_s+' '\
          '-i '+this_audio_file+' -c:a copy -c:v libx264 '\
          '-shortest '+output_file
      esystem(_)
    end
  end

end

if __FILE__ == $PROGRAM_NAME
  MultimediaParadise.create_video_from_this_audio(ARGV)
end # create_video_from_this_audio