#!/usr/bin/ruby -w
# Encoding: UTF-8
# frozen_string_literal: true
# =========================================================================== #
# require 'multimedia_paradise/toplevel_methods/encode_this_video.rb'
# MultimediaParadise.encode_this_video('foobar.avi')
# =========================================================================== #
require 'multimedia_paradise/toplevel_methods/esystem.rb'

module MultimediaParadise

  # ========================================================================= #
  # === MultimediaParadise.encode_this_video
  #
  # The default mode for this method is CRF, which stands for:
  #
  #   Constant Rate Factor
  #
  # This method is similar to the following commandline:
  #
  #   ffmpeg -i input.avi -c:v libx264 -preset slow -crf 22 -c:a copy output.mkv
  #
  # ========================================================================= #
  def self.encode_this_video(
      i = ARGV, use_this_mode = :CRF
    )
    [i].flatten.compact.each {|this_video|
      if File.exist? this_video
        output_filename = 'output_'+
                          this_video.
                          sub(/#{File.extname(this_video)}$/,'')+
                          '.mkv'
        case use_this_mode
        when :CRF, :default
          _ = 'ffmpeg -i '+this_video+' -c:v libx264 '\
              '-preset slow -crf 22 -c:a copy '+output_filename
          esystem _
        end
      else
        e 'No file exists at '+this_video+'.'
      end
    }
  end

end

if __FILE__ == $PROGRAM_NAME
  MultimediaParadise.encode_this_video(ARGV)
end # encodethisvideo