#!/usr/bin/ruby -w
# Encoding: UTF-8
# frozen_string_literal: true
# =========================================================================== #
# Code in this file will deal with files and directories in general, including
# cd-related actions, removal of files, creation of files and so forth.
# =========================================================================== #
# require 'multimedia_paradise/toplevel_methods/files_and_directories.rb'
# MultimediaParadise.chdir
# =========================================================================== #
module MultimediaParadise

  require 'fileutils'
  require 'multimedia_paradise/toplevel_methods/e.rb'
  require 'multimedia_paradise/colours/colours.rb'

  # ========================================================================= #
  # === MultimediaParadise.move_file
  # ========================================================================= #
  def self.move_file(old, new)
    FileUtils.mv(old, new)
  end; self.instance_eval { alias mv          move_file } # === MultimediaParadise.mv
       self.instance_eval { alias rename_file move_file } # === MultimediaParadise.rename_file

  # ========================================================================= #
  # === MultimediaParadise.remove_file
  #
  # Remove a given file, without any further checks.
  # ========================================================================= #
  def self.remove_file(i)
    File.delete(i) if File.exist?(i)
  end; self.instance_eval { alias delete_file remove_file } # === MultimediaParadise.delete_file

  # ========================================================================= #
  # === remove_file
  # ========================================================================= #
  def remove_file(i)
    MultimediaParadise.remove_file(i)
  end

  # ========================================================================= #
  # === MultimediaParadise.mkdir
  #
  # This method can be used to create a directory on the local computer
  # system.
  # ========================================================================= #
  def self.mkdir(i)
    FileUtils.mkdir_p(i)
  end

  # ========================================================================= #
  # === mkdir
  # ========================================================================= #
  def mkdir(i)
    MultimediaParadise.mkdir(i)
  end

  # ========================================================================= #
  # === MultimediaParadise.rename
  #
  # This perform a rename-action, using mv.
  # ========================================================================= #
  def self.rename(from, to)
    if File.exist? from
      e rev+'Now renaming `'+sfile(from.to_s)+rev+'` to `'+sfile(to.to_s)+rev+'`.'
      FileUtils.mv(from, to)
    else
      e "#{rev}Can not rename `#{sfancy(from)}` as it does not exist."
    end
  end; self.instance_eval { alias mv          rename } # === MultimediaParadise.mv (mv tag)
       self.instance_eval { alias rename_file rename } # === MultimediaParadise.rename_file

  # ========================================================================= #
  # === chdir                                                        (cd tag)
  # ========================================================================= #
  def chdir(i)
    MultimediaParadise.chdir(i)
  end; alias cd chdir # === cd

  # ========================================================================= #
  # === MultimediaParadise.chdir
  #
  # Use this method if you wish to change directory to another target
  # directory.
  # ========================================================================= #
  def self.chdir(i)
    Dir.chdir(i) if File.directory? i
  end; self.instance_eval { alias cd chdir } # === MultimediaParadise.cd

end