#!/usr/bin/ruby -w
# Encoding: UTF-8
# frozen_string_literal: true
# =========================================================================== #
# This file will allow us to flip a video file horizontally
# or vertically.
# =========================================================================== #
# require 'multimedia_paradise/toplevel_methods/flip_and_rotate.rb'
# =========================================================================== #
require 'multimedia_paradise/ffmpeg/ffmpeg.rb'

module MultimediaParadise

  # ========================================================================= #
  # === MultimediaParadise.vertical_flip
  #
  # This method will make use of the video-filter called "vflip". A
  # vertical flip will flip the video from head-to-bottom, so people
  # will suddenly appear as if they are hanging from the ceiling. :)
  #
  # Invocation example:
  #
  #   MultimediaParadise.vertical_flip '/Depot/Video/Reallife/O.J.Simpson_TV_News_etwa_1995.avi'
  #
  # ========================================================================= #
  def self.vertical_flip(this_video_file)
    ffmpeg_vertical_flip(this_video_file)
  end; self.instance_eval { alias flip_upside_down vertical_flip } # === MultimediaParadise.flip_upside_down

  # ========================================================================= #
  # === MultimediaParadise.horizontal_flip
  #
  # This method will make use of the video-filter called "hflip".
  # ========================================================================= #
  def self.horizontal_flip(this_video_file)
    ffmpeg_horizontal_flip(this_video_file)
  end

  # ========================================================================= #
  # === MultimediaParadise.rotate_this_video_file_by_90_degrees
  #
  # For a complete description of this method, please have a look at
  # the method MultimediaParadise.ffmpeg_rotate_this_video_file_by_90_degrees
  # ========================================================================= #
  def self.rotate_this_video_file_by_90_degrees(
      this_video_file
    )
    ffmpeg_rotate_this_video_file_by_90_degrees(
      this_video_file
    )
  end

end

if __FILE__ == $PROGRAM_NAME
  # MultimediaParadise.horizontal_flip(ARGV)
  MultimediaParadise.vertical_flip(ARGV)
end # horizontal_flip