#!/usr/bin/ruby -w
# Encoding: UTF-8
# frozen_string_literal: true
# =========================================================================== #
# require 'multimedia_paradise/toplevel_methods/increase_volume_of_this_audio_file.rb'
# =========================================================================== #
require 'multimedia_paradise/ffmpeg/ffmpeg.rb'

module MultimediaParadise

  # ========================================================================= #
  # === MultimediaParadise.increase_volume_of_this_audio_file
  #
  # The method method MultimediaParadise.increase_volume_of_this_audio_file
  # can boost the volume of an audio file, by making use of sox or ffmpeg.
  #
  # Sox is currently the default, whereas ffmpeg is used if the input
  # given ends with 'db', which is short for decibel. The latter variant
  # allows you to just input an increase or decrease in decibel.
  #
  # The sox command can modify the audio volume, up or down, using the -v
  # flag, followed by a multiplier. The following example doubles (x2)
  # the audio volume of a file.
  #
  #   sox -v 2.0 original-music.ogg louder-music.ogg 
  #
  # ========================================================================= #
  def self.increase_volume_of_this_audio_file(
      input_file,
      modify_volume_by_this_value = '5.0' # or 3.0 or 4.0 and so forth.
    )
    unless input_file.is_a? Array
      input_file = [input_file]
    end
    if modify_volume_by_this_value.to_s.downcase.end_with? 'db'
      ffmpeg_increase_volume_of_this_audio_file(
        input_file, modify_volume_by_this_value
      )
    else
      case modify_volume_by_this_value # case tag
      when '1/4', :one_fourth,'25%'
        modify_volume_by_this_value = 0.25
      end
      # ======================================================================= #
      # Batch-process the Array next.
      # ======================================================================= #
      input_file.each {|this_file|
        value = modify_volume_by_this_value.to_s
        output_file = "modified_file_by_v#{value}_#{this_file}"
        _ = 'sox -v '+value+
            ' '+this_file+' '+output_file
        esystem _
      }
    end
  end

end

if __FILE__ == $PROGRAM_NAME
  MultimediaParadise.increase_volume_of_this_audio_file(ARGV, '+20dB')
end # increase_volume_of_this_audio_file