#!/usr/bin/ruby -w
# Encoding: UTF-8
# frozen_string_literal: true
# =========================================================================== #
# This file here will simply report how many videos are in the main
# video directory (locally).
#
# This currently depends on videos being there, but in theory, we could
# also simply query from a file that keeps track of how many videos
# SHOULD be there.
# =========================================================================== #
# require 'multimedia_paradise/toplevel_methods/n_local_videos.rb'
# =========================================================================== #
require 'multimedia_paradise/colours/colours.rb'
require 'multimedia_paradise/constants/file_constants.rb'
require 'multimedia_paradise/constants/directory_constants.rb'
require 'multimedia_paradise/toplevel_methods/e.rb'
require 'multimedia_paradise/toplevel_methods/is_video_file.rb'
require 'multimedia_paradise/toplevel_methods/is_on_roebe.rb'
require 'multimedia_paradise/toplevel_methods/opn.rb'

module MultimediaParadise

  # ========================================================================= #
  # === MultimediaParadise.n_local_videos?
  # ========================================================================= #
  def self.n_local_videos?(
      from_this_directory = realvids_directory?
    )
    these_files = Dir["#{from_this_directory}*"].select {|entry|
      ::MultimediaParadise.is_video_file?(entry)
    } 
    n_local_videos_are_available = these_files.size
    return n_local_videos_are_available
  end

  # ========================================================================= #
  # === MultimediaParadise.report_n_local_videos?
  #
  # This is the method that will report how many videos are available.
  # ========================================================================= #
  def self.report_n_local_videos?
    opn; e 'There are '+sfancy(n_local_videos?.to_s)+' videos available '\
           'locally, at `'+sdir(realvids_directory?)+'`.'
    if is_on_roebe? and File.exist?(FILE_VIDEO_COLLECTION)
      dataset = YAML.load_file(FILE_VIDEO_COLLECTION)
      opn; e 'A total of '+sfancy(dataset.keys.size.to_s)+' videos are registered '\
             'in total in the file '+sfile('video_collection.yml')+'.'
    end
  end

end

if __FILE__ == $PROGRAM_NAME
  MultimediaParadise.report_n_local_videos?
end # n_local_videos?