#!/usr/bin/ruby -w
# Encoding: UTF-8
# frozen_string_literal: true
# =========================================================================== #
# require 'multimedia_paradise/toplevel_methods/player_in_use.rb'
# =========================================================================== #
module MultimediaParadise

  require 'multimedia_paradise/toplevel_methods/is_on_roebe.rb'

  # ========================================================================= #
  # === @player_in_use
  #
  # This toplevel instance variable denotes which video-player we will
  # use for the MultimediaParadise project.
  # ========================================================================= #
  @player_in_use = nil

  # ========================================================================= #
  # === WHICH_VIDEO_PLAYER_TO_USE
  #
  # This constant determines which video player to use. An environment
  # variable can overrule this, called USE_THIS_VIDEO_PLAYER.
  #
  # Since as of December 2018 this should no longer be the primary
  # means to set the multimedia-player in use for the project -
  # instead, make use of MultimediaParadise.set_player_in_use().
  # ========================================================================= #
  if ENV.has_key? 'USE_THIS_VIDEO_PLAYER'
    WHICH_VIDEO_PLAYER_TO_USE = "#{ENV['USE_THIS_VIDEO_PLAYER']}"
  else
    WHICH_VIDEO_PLAYER_TO_USE = 'mplayer' # DATA.read
  end

  # ========================================================================= #
  # === MultimediaParadise.player_in_use?
  #
  # Query which audio/video player is in use for this project.
  # ========================================================================= #
  def self.player_in_use?
    @player_in_use
  end; self.instance_eval { alias which_video_player_to_use?   player_in_use? } # === MultimediaParadise.which_video_player_to_use?
       self.instance_eval { alias use_this_player              player_in_use? } # === MultimediaParadise.use_this_player
       self.instance_eval { alias use_which_multimedia_player? player_in_use? } # === MultimediaParadise.use_which_multimedia_player?
       self.instance_eval { alias use_this_multimedia_player?  player_in_use? } # === MultimediaParadise.use_this_multimedia_player?
       self.instance_eval { alias player?                      player_in_use? } # === MultimediaParadise.player?
       self.instance_eval { alias video_player?                player_in_use? } # === MultimediaParadise.video_player?
       self.instance_eval { alias use_which_video_player?      player_in_use? } # === MultimediaParadise.use_which_video_player?

  # ========================================================================= #
  # === MultimediaParadise.set_player_in_use
  # ========================================================================= #
  def self.set_player_in_use(
      i = :default
    )
    i = i.to_s.dup.strip # Make sure we have a String past this point.
    case i
    when 'default',''
      # ====================================================================== #
      # Determine the default video-player in use for the MultimediaParadise
      # project. A constant keeps track of this value. Since as of June
      # 2020 a .yml file exists, and it will be used if it is available
      # AND if we are on a roebe system..
      # ====================================================================== #
      if File.exist?(FILE_USE_THIS_VIDEO_PLAYER) and is_on_roebe?
        i = YAML.load_file(FILE_USE_THIS_VIDEO_PLAYER)
      else
        i = WHICH_VIDEO_PLAYER_TO_USE
      end
    end
    @player_in_use = i
  end; self.instance_eval { alias set_use_this_multimedia_player set_player_in_use } # === MultimediaParadise.set_use_this_multimedia_player

  # ========================================================================= #
  # And call it at once with the default, to have a proper state for the
  # variable.
  # ========================================================================= #
  set_player_in_use

end