#!/usr/bin/ruby -w
# Encoding: UTF-8
# frozen_string_literal: true
# =========================================================================== #
# Internet-radio related code will reside in this file here.
# =========================================================================== #
# require 'multimedia_paradise/toplevel_methods/radio.rb'
# MultimediaParadise.register_title_of_this_audio_file
# MultimediaParadise.title_of_the_audio_file?
# =========================================================================== #
module MultimediaParadise

  require 'multimedia_paradise/constants/constants.rb'
  require 'multimedia_paradise/constants/encodings.rb'
  require 'multimedia_paradise/constants/file_constants.rb'
  require 'multimedia_paradise/colours/colours.rb'
  require 'multimedia_paradise/toplevel_methods/e.rb'
  require 'multimedia_paradise/toplevel_methods/esystem.rb'
  require 'multimedia_paradise/toplevel_methods/player_in_use.rb'

  begin
    require 'roebe/toplevel_methods/unicode/popular_unicode_symbols.rb'
  rescue LoadError; end

  # ========================================================================= #
  # === @title_of_the_audio_file
  # ========================================================================= #
  @title_of_the_audio_file = nil

  # ========================================================================= #
  # === MultimediaParadise.try_to_find_a_match_for_a_similar_radio_station
  #
  # This method will attempt to find a matching URL from the given input.
  # ========================================================================= #
  def self.try_to_find_a_match_for_a_similar_radio_station(i)
    i = i.to_s.downcase.delete(' ')
    all_radio_stations = MultimediaParadise.radio_stations?
    selection = all_radio_stations.select {|name_of_the_radio_station, remote_URL|
      name_of_the_radio_station.downcase.delete(' ').include? i
    }
    return selection
  end

  # ========================================================================= #
  # === MultimediaParadise.play_radio_from_this_url
  #
  # This method can be used to play radio from a remote URL.
  #
  # It will make use of Process.spawn rather than system(), because
  # we need to return the PID.
  #
  # The second argument to this method, if given, will log the
  # captured stream into a local file. This depends on mpv mostly,
  # and is a bit limited, so don't expect too much of that
  # feature.
  # ========================================================================= #
  def self.play_radio_from_this_url(
      url                    = 'http://www.radioking.com/play/generationhit80',
      capture_into_this_file = nil,
      &block
    )
    _ = video_player?
    # ======================================================================= #
    # === Handle blocks given to this method next
    # ======================================================================= #
    if block_given?
      yielded = yield
      if yielded.is_a? Hash
        # =================================================================== #
        # === :use_this_player
        # =================================================================== #
        if yielded.has_key? :use_this_player
          _ = yielded.delete(:use_this_player)
        end
      end
    end
    case _.to_s
    # ======================================================================= #
    # === mpv
    # ======================================================================= #
    when 'mpv'
      cmd_to_run = "mpv #{url}".dup
      if capture_into_this_file
        cmd_to_run << ' --stream-record="'+capture_into_this_file+'"'
      end
      e cmd_to_run
      # ===================================================================== #
      # The line below was in use until September 2020; it is kept in the
      # event that we wish to revert to it.
      #   pid = Process.spawn(cmd_to_run)
      # ===================================================================== #
      io_object = IO.popen(cmd_to_run, 'r+')
      pid = io_object.pid
      set_pid(pid) # Keep track of the pid here.
      io_object.each { |line|
        if line.start_with? ' icy-title: '
          register_title_of_this_audio_file(line)
        end
        e line
      }
      io_object.close
      return pid
    # ======================================================================= #
    # === mplayer
    # ======================================================================= #
    when 'mplayer'
      cmd_to_run = "mplayer #{url}".dup
      if capture_into_this_file
        cmd_to_run << ' -dumpstream -dumpfile "'+capture_into_this_file+'"'
      end
      e cmd_to_run
      pid = Process.spawn(cmd_to_run)
      set_pid(pid) # Keep track of the pid here.
      return pid
    else
      e "No video player that is known (#{_}). The "\
        "URL would have been: #{sfancy(url)}"
    end
  end

  # ========================================================================= #
  # === MultimediaParadise.radio_stations?
  #
  # The file that is used by this method by default is usually the one
  # that is distributed via the MultimediaParadise project, called
  # 'radio/radio_stations.yml'.
  # ========================================================================= #
  def self.radio_stations?(i = FILE_RADIO_STATIONS)
    YAML.load_file(i)
  end

  # ========================================================================= #
  # === MultimediaParadise.show_internet_radio_URLs
  #
  # This method will show various URLs to internet-radio. This depends
  # on prior registration in a yaml file.
  #
  # The first argument should provide an existing file to where the
  # radio stations are registered. By default, MultimediaParadise
  # will come with a file where some radio URL stations are registered.
  #
  # The second argument controls how this dataset is displayed. The
  # :default style means that we will use a long display format, with
  # more newlines. This may be easier to read - but there is also a
  # :compact variant, which will display the radio stations along
  # with the registered URL in a compact manner, all on the same
  # line. Do note that :in_a_compact_form is simply an alias
  # to :compact.
  #
  # To invoke this, do:
  #
  #   mparadise --radio-compact
  #   mparadise --rhythmbox
  #
  # ========================================================================= #
  def self.show_internet_radio_URLs(
      i             = file_radio_stations,
      display_style = :default
    )
    # ======================================================================= #
    # First, load up the dataset from the given file.
    # ======================================================================= #
    dataset = YAML.load_file(i)
    e
    if block_given?
      yielded = yield
      case yielded
      # ===================================================================== #
      # This instructions will show the remote URL listing of radio
      # stations in a more compact version.
      # ===================================================================== #
      when :compact,
           :in_a_compact_form
        display_style = :compact
      end
    end
    dataset.each_pair {|name_of_the_radio_station_at_hand, value|
      result = '  '.dup
      if TRY_TO_USE_UNICODE_SYMBOLS and Object.const_defined?(:Roebe) and
         Roebe.respond_to?(:wheel_of_dharma)
        result << COLOURS.darkslateblue(Roebe.wheel_of_dharma+' ')
      end
      entry = value['url'] # Define "entry" here.
      properly_formatted_name_of_the_radio_station_at_hand =
        COLOURS.royalblue(
          (name_of_the_radio_station_at_hand+':').ljust(25)
        )
      result << properly_formatted_name_of_the_radio_station_at_hand
      unless display_style == :compact
        result << "\n"
      end
      result << '  '+sfancy("    #{entry}")
      e result
    }
    e
  end; self.instance_eval { alias show_available_radio_stations show_internet_radio_URLs } # === MultimediaParadise.show_available_radio_stations

  # ========================================================================= #
  # === MultimediaParadise.set_pid
  # ========================================================================= #
  def self.set_pid(i)
    @pid = i
  end

  # ========================================================================= #
  # === MultimediaParadise.register_title_of_this_audio_file
  # ========================================================================= #
  def self.register_title_of_this_audio_file(i)
    @title_of_the_audio_file = nil # Always reset it here.
    use_this_regex = /^ icy-title: (.+)\n/
    if i and i.start_with?(' icy-title: ')
      i = i.scan(use_this_regex).flatten
      if i.is_a? Array
        i = i.first.to_s
      end
      @title_of_the_audio_file = i.strip
    end
  end

  # ========================================================================= #
  # === MultimediaParadise.title_of_the_audio_file?
  # ========================================================================= #
  def self.title_of_the_audio_file?
    @title_of_the_audio_file
  end
  
  # ========================================================================= #
  # === MultimediaParadise.pid?
  #
  # Keep track of the pid through this toplevel variable.
  # ========================================================================= #
  def self.pid?
    @pid
  end

  # ========================================================================= #
  # === MultimediaParadise.play_radio
  # ========================================================================= #
  def self.play_radio(
      remote_url = 'bbc3'
    )
    unless remote_url.include? 'http'
      remote_url = return_radio_station_url_from_this_input(remote_url)
    end
    play_radio_from_this_url(remote_url)
  end

  # ========================================================================= #
  # === MultimediaParadise.return_iso_encoding_of
  # ========================================================================= #
  def self.return_iso_encoding_of(i)
    i = i.dup if i.frozen?
    i.force_encoding(ISO_ENCODING)
  end

  # ========================================================================= #
  # === MultimediaParadise.return_radio_station_url_from_this_input
  #
  # This method will try to return a corresponding URL from the given
  # input at hand.
  # ========================================================================= #
  def self.return_radio_station_url_from_this_input(
      i,
      use_this_yaml_file = FILE_RADIO_STATIONS
    )
    dataset = YAML.load_file(use_this_yaml_file)
    # ======================================================================= #
    # This file may contain umlauts. We could convert these into ISO
    # encoding, but since as of May 2019 we will default to UTF-8
    # anyway.
    # ======================================================================= #
    # dataset.each_pair {|key, value|
    #   hash[return_iso_encoding_of(key)] = value
    # }; dataset = hash
    i = i.to_s.delete('-').downcase
    i = case i
    # ======================================================================= #
    # === oe3
    # ======================================================================= #
    when 'oe3','ö3','o3'
      dataset['Hitradio Ö3']['url']
    # ======================================================================= #
    # === fm4
    # ======================================================================= #
    when 'fm4'
      dataset['FM4']['url']
    # ======================================================================= #
    # == orf-radio-wien
    # ======================================================================= #
    when /^-?-?orf(_|-)?radio(_|-)?wien$/i
      dataset['ORF Radio Wien']['url']
    # ======================================================================= #
    # === energy
    # ======================================================================= #
    when 'energy'
      dataset['Energy 104,2']['url']
    # ======================================================================= #
    # === trance
    # ======================================================================= #
    when 'trance'
      dataset['PulsRadio TRANCE']['url']
    # ======================================================================= #
    # === germany
    # ======================================================================= #
    when 'germany'
      dataset['Germany Radio']['url']
    # ======================================================================= #
    # === bbc1
    # ======================================================================= #
    when /^bbc1$/i
      dataset['BBC Radio 1']['url']
    # ======================================================================= #
    # === bbc2
    # ======================================================================= #
    when /^bbc2$/i
      dataset['BBC Radio 2']['url']
    # ======================================================================= #
    # === bbc4
    # ======================================================================= #
    when /^bbc4/i
      dataset['BBC Radio 4']['url']
    # ======================================================================= #
    # === kronehit
    # ======================================================================= #
    when 'kronehit'
      dataset['Kronehit Dance']['url']
    # ======================================================================= #
    # === oe1
    # ======================================================================= #
    when 'oe1','ö1'
      dataset['OE 1']['url']
    # ======================================================================= #
    # === radiowien
    # ======================================================================= #
    when /^radio(-|_| )?wien$/
      dataset['ORF Radio Wien']['url']
    # ======================================================================= #
    # === Magic FM Germany
    # ======================================================================= #
    when /^Magic(-|_| )?FM(-|_| )?Germany$/i
      dataset['Magic FM Germany']['url']
    # ======================================================================= #
    # === bbc3
    # ======================================================================= #
    when /^bbc3$/i,
         /^BBC(-|_| )?Radio(-|_| )?3$/i
      dataset['BBC Radio 3']['url']
    else
      i
    end
    i
  end

  # ========================================================================= #
  # === MultimediaParadise.display_radio_stations
  # ========================================================================= #
  def self.display_radio_stations
    e sfile(
        FILE_RADIO_STATIONS
      )
  end

end

if __FILE__ == $PROGRAM_NAME
  alias e puts
  MultimediaParadise.display_radio_stations
  # e 'The available radio stations are:'
  # e
  # pp MultimediaParadise.radio_stations?
  if ARGV.empty?
    use_this_radio_station = 'BBC3'
  else
    use_this_radio_station = ARGV.first
  end
  e
  e 'Playing some Radio next (for BBC3):'
  e
  MultimediaParadise.play_radio(use_this_radio_station)
  # MultimediaParadise.play_radio_from_this_url 'https://stream.laut.fm/magicfm'
end # display_radio_stations http://listen.radionomy.com:80/YourTranceRadio