#!/usr/bin/ruby -w
# Encoding: UTF-8
# frozen_string_literal: true
# =========================================================================== #
# require 'multimedia_paradise/toplevel_methods/return_all_video_files.rb'
# MultimediaParadise.return_all_video_files
# =========================================================================== #
module MultimediaParadise

  require 'yaml'
  require 'multimedia_paradise/constants/file_constants.rb'
  require 'multimedia_paradise/toplevel_methods/is_video_file.rb'

  # ========================================================================= #
  # === MultimediaParadise.available_video_files
  #
  # This method will only return all video-files under ther ealvids
  # directory.
  # ========================================================================= #
  def self.available_video_files
    Dir["#{realvids_directory?}**"].select {|entry| is_a_video_file?(entry) }.sort
  end; self.instance_eval { alias array_available_realvids_videos available_video_files } # === MultimediaParadise.array_available_realvids_videos

  # ========================================================================= #
  # === MultimediaParadise.return_all_video_files
  #
  # Use this method if you wish to return all video files registered
  # in the main video_collection yaml file.
  #
  # This will return a Hash that has entries similar to this one here:
  #
  #   275=>{"title"=>"Shaun Of The Dead", "genre"=>"Comedy", "addition_date"=>"21.12.2009", "language"=>"english"},
  #
  # ========================================================================= #
  def self.return_all_video_files
    _ = file_video_collection
    if File.exist? _
      begin
        YAML.load_file(_)
      rescue Psych::SyntaxError => error
        puts "Psych::SyntaxError for the file #{_} in the method "\
             "MultimediaParadise.return_all_video_files()"
        pp error
      end
    else
      {} # Or we will return an empty Hash.
    end
  end

  # ========================================================================= #
  # === return_all_video_files
  #
  # Return all video files that are defined in the file
  # video_collection.yml.
  # ========================================================================= #
  def return_all_video_files
    MultimediaParadise.return_all_video_files # Defined in this file here.
  end

end

if __FILE__ == $PROGRAM_NAME
  pp MultimediaParadise.return_all_video_files
end