#!/usr/bin/ruby -w
# Encoding: UTF-8
# frozen_string_literal: true
# =========================================================================== #
# require 'multimedia_paradise/toplevel_methods/return_full_name_for_video_at_this_position.rb'
# =========================================================================== #
require 'multimedia_paradise/toplevel_methods/video_dataset.rb'

module MultimediaParadise

  # ========================================================================= #
  # === MultimediaParadise.return_full_name_for_video_at_this_position
  #
  # Pass the position of the video file, such as '5'. Then we will
  # return a full string such as "005_BowlingforColumbine".
  #
  # The second argument will be the name of the file extension in question.
  # ========================================================================= #
  def self.return_full_name_for_video_at_this_position(
      position_of_the_video_file,
      use_this_extension = 'avi'
    )
    renamed_title = obtain_title_at_position(
      position_of_the_video_file.to_i
    ).to_s.delete(' ')
    unless use_this_extension.start_with? '.'
      use_this_extension = use_this_extension.dup if use_this_extension.frozen?
      use_this_extension.prepend '.'
    end
    # ======================================================================= #
    # We need to pad-up with leading numbers such as '0' and so on.
    # ======================================================================= #
    new_name = (position_of_the_video_file.to_s).rjust(3, '0')+
                '_'+renamed_title+use_this_extension
    return new_name
  end

  # ========================================================================= #
  # === return_full_name_for_video_at_this_position
  #
  # This method may return a String such as '001_TheAnimatrix.avi'.
  # ========================================================================= #
  def return_full_name_for_video_at_this_position(a = 1, b = 'avi')
    ::MultimediaParadise.return_full_name_for_video_at_this_position(a, b)
  end

end

if __FILE__ == $PROGRAM_NAME
  puts MultimediaParadise.return_full_name_for_video_at_this_position(1)
  puts MultimediaParadise.return_full_name_for_video_at_this_position(5)
  puts MultimediaParadise.return_full_name_for_video_at_this_position(6)
  puts MultimediaParadise.return_full_name_for_video_at_this_position(7)
end # return_full_name_for_video_at_this_position