#!/usr/bin/ruby -w
# Encoding: UTF-8
# frozen_string_literal: true
# =========================================================================== #
# require 'multimedia_paradise/toplevel_methods/return_random_video_file_from_the_video_collection.rb'
# MultimediaParadise.return_random_video
# =========================================================================== #

module MultimediaParadise

  require 'multimedia_paradise/constants/file_constants.rb'
  require 'multimedia_paradise/toplevel_methods/e.rb'
  require 'multimedia_paradise/toplevel_methods/is_video_file.rb'
  require 'multimedia_paradise/toplevel_methods/return_all_video_files.rb'

  # ========================================================================= #
  # === MultimediaParadise.return_random_video_file_from_the_video_collection
  #
  # This method will return a random video file from the video collection.
  #
  # This can then be used to e. g. play said file.
  #
  # Do note that for this to work, the file must exist at the target
  # position - otherwise the method will return nil.
  # ========================================================================= #
  def self.return_random_video_file_from_the_video_collection(
      shall_we_notify_the_user_if_the_video_file_is_missing = false
    )
    case shall_we_notify_the_user_if_the_video_file_is_missing
    when :notify_when_missing
      shall_we_notify_the_user_if_the_video_file_is_missing = true
    end
    # ======================================================================= #
    # First, obtain the full collection.
    # ======================================================================= #
    _ = file_video_collection
    video_collection = YAML.load_file(_)
    keys = video_collection.keys
    sample_key = keys.sample
    target = video_collection[sample_key]['title'] # This is the sample.
    path = ::MultimediaParadise.realvids_directory?+'*'+target.tr(' ','*')+'*'
    result = Dir[path]
    if result.is_a?(Array) and (result.size > 1)
      # ===================================================================== #
      # This here may be the case for a two-member Array such
      # as ['058_IceAge.mp3','058_IceAge.avi']
      # ===================================================================== #
      result = is_video_file?(result)
    end
    if result.empty? and shall_we_notify_the_user_if_the_video_file_is_missing
      e 'No file could be found at '+path+'.'
    end
    if result.is_a? Array
      result = result.first
    end
    return result
  end; self.instance_eval { alias return_random_video return_random_video_file_from_the_video_collection } # === MultimediaParadise.return_random_video

  # ========================================================================= #
  # === MultimediaParadise.return_random_video_that_exists
  # ========================================================================= #
  def self.return_random_video_that_exists
    return Dir[::MultimediaParadise.realvids_directory?+'*'].select {|entry|
      MultimediaParadise.is_this_a_video_file?(entry)
    }.sample
  end

end

if __FILE__ == $PROGRAM_NAME
  _ = MultimediaParadise.return_random_video_file_from_the_video_collection(:notify_when_missing)
  puts _ if _  
end # return_random_video_file_from_the_video_collection