#!/usr/bin/ruby -w
# Encoding: UTF-8
# frozen_string_literal: true
# =========================================================================== #
# require 'multimedia_paradise/toplevel_methods/set_title_of.rb'
# =========================================================================== #
module MultimediaParadise

  require 'multimedia_paradise/toplevel_methods/esystem.rb'
  require 'multimedia_paradise/colours/colours.rb'

  # ========================================================================= #
  # === MultimediaParadise.set_title_of
  #
  # This toplevel-method can be used specifically to set the title of a
  # .mp3 or a .mp4 file or any other multimedia-file that allows for
  # meta-data.
  #
  # For some .mp3 files this requires re-encoding, since the total
  # filesize will change (as the header will be at the beginning
  # of the file, in order to allow for simpler streaming support).
  #
  # The second argument to this method should be the title that we are
  # going to use for the given multimedia file at hand.
  # ========================================================================= #
  def self.set_title_of(
      these_audio_files = ARGV,
      use_this_title    = nil
    )
    [these_audio_files].flatten.compact.each {|this_audio_file|
      if use_this_title
        case use_this_title
        # =================================================================== #
        # === :automatic
        # =================================================================== #
        when :automatic
          extname = File.extname(this_audio_file)
          use_this_title = File.basename(this_audio_file).sub(/#{extname}$/,'')
        end
        new_filename = "output_#{File.basename(this_audio_file)}"
        # =================================================================== #
        # Build up our main command next, via ffmpeg:
        # =================================================================== #
        _ = 'ffmpeg -i '+
            this_audio_file+
            ' -metadata title="'+use_this_title+'" '\
            '-codec copy '+new_filename
        esystem _
        if File.exist? new_filename
          e "#{rev}Stored at `#{sfile(new_filename)}`."
        end
      end
    }
  end; self.instance_eval { alias set_title_of_this_mp3_file set_title_of } # === MultimediaParadise.set_title_of_this_mp3_file

end

if __FILE__ == $PROGRAM_NAME
  # ========================================================================= #
  # First argument should be the name of the .mp3 file; second argument
  # should be the new title that is to be used.
  # ========================================================================= #
  MultimediaParadise.set_title_of(ARGV[0], ARGV[1])
end # mp3_set_title_of /Depot/j/foobar.mp3 BamBoleO
    # mp3settitleof 123_Gremlins.mkv Gremlins