#!/usr/bin/ruby -w
# Encoding: UTF-8
# frozen_string_literal: true
# =========================================================================== #
# This file will gather code that can be used to do a screencast on the
# xorg-server.
# =========================================================================== #
# require 'multimedia_paradise/toplevel_methods/screencast.rb'
# =========================================================================== #
module MultimediaParadise

  require 'multimedia_paradise/ffmpeg/ffmpeg.rb'

  # ========================================================================= #
  # === MultimediaParadise.start_screencast
  #
  # This method can be used to "record" the desktop, by making use of
  # x11grab and the huffyuv codec.
  #
  # If more flexibility is required then this may be added at a later time.
  #
  # Keep in mind that this will create HUGE files depending on the video
  # codec in use - like 20 seconds may occupy 2 gigabytes for the
  # lossless huffyuv codec.
  #
  # If you wish to use different video or audio codecs, you can use
  # these commandline flags:
  #
  #   -vcodec: video encoding codec
  #   -acodec: audio encoding codec
  #
  # To record the desktop only, one could use a commandline flag such as
  # this one here:
  #
  #   ffmpeg -f x11grab -s 1366x768 -i :0.0 -r 25 -vcodec libx264 output.mkv
  #   ffmpeg -f x11grab -r 25 -s 1920x1080 -i :0.0 -vcodec huffyuv screencast.avi
  #
  # For some useful links (should these persist), have a look at:
  #
  #   https://www.pcsuggest.com/linux-screen-recorder-ffmpeg/
  #
  # ========================================================================= #
  def self.start_screencast(
      use_this_screen_resolution = return_screen_resolution, # ← We will use the default screen-resolution here.
      output_filename            = 'screencast',
      use_this_framerate         = '25',     # ← This is the option -r to ffmpeg.
      use_this_video_codec       = 'huffyuv' # ← or libx264, for instance.
    )
    ffmpeg_start_screencast(
      use_this_screen_resolution,
      output_filename,
      use_this_framerate,
      use_this_video_codec
    )
  end

  # ========================================================================= #
  # === MultimediaParadise.record_my_desktop
  #
  # This is mostly an ad-hoc method.
  # ========================================================================= #
  def self.record_my_desktop
    esystem 'ffmpeg -video_size 1920x1080 -framerate 30 -f x11grab -i :0.0 -c:v libx264 -qp 0 -preset ultrafast capture.mkv'
  end

end

if __FILE__ == $PROGRAM_NAME
  MultimediaParadise.start_screencast  
end # startscreencast