#!/usr/bin/ruby -w
# Encoding: UTF-8
# frozen_string_literal: true
# =========================================================================== #
# require 'multimedia_paradise/toplevel_methods/subtitles.rb'
# MultimediaParadise.remove_subtitles_from_this_video_file
# MultimediaParadise.embed_this_subtitle_onto_that_video
# =========================================================================== #
module MultimediaParadise

  require 'multimedia_paradise/toplevel_methods/esystem.rb'
  # ========================================================================= #
  # === MultimediaParadise.remove_subtitles_from_this_video_file
  #
  # This method can be used to remove a subtitle from a video file.
  #
  # Typical commandline usage in ffmpeg would look like this:
  #
  #   ffmpeg -i video.mkv -vcodec copy -acodec copy -sn video-no-subs.mkv
  #
  # ========================================================================= #
  def self.remove_subtitles_from_this_video_file(
      this_video_file = 'foobar.mp4'
    )
    if this_video_file.is_a? Array
      this_video_file = this_video_file.join(' ').strip
    end
    output_file = "output_video_without_subs_#{this_video_file}"
    _ = 'ffmpeg -i '+this_video_file.to_s+' -vcodec copy '\
        '-acodec copy '\
        '-sn '+
        output_file
    e
    esystem _
    e
  end

  # ========================================================================= #
  # === MultimediaParadise.embed_this_subtitle_onto_that_video
  #
  # This method can be used to "attach" a subtitle to a particular
  # video.
  # ========================================================================= #
  def self.embed_this_subtitle_onto_that_video(
      this_subtitle, that_video
    )
    output_file = 'output.mp4'
    _ = 'ffmpeg -i '+that_video+' -i '+this_subtitle+
        ' -c copy -c:s mov_text '+output_file
    e
    esystem _
    e
  end

end

if __FILE__ == $PROGRAM_NAME
  MultimediaParadise.embed_this_subtitle_onto_that_video(ARGV[0], ARGV[1])
end # embedthissubtitleontothatvideo