#!/usr/bin/ruby -w
# Encoding: UTF-8
# frozen_string_literal: true
# =========================================================================== #
# === MultimediaParadise::AllVideos
#
# This class will output all Videofiles, as found from the yaml file itself.
#
# Note that FindVideo also does something very similar.
#
# Usage examples:
#   MultimediaParadise::AllVideos.new
# =========================================================================== #
# require 'multimedia_paradise/video/all_videos.rb'
# =========================================================================== #
require 'multimedia_paradise/base/base.rb'
require 'multimedia_paradise/toplevel_methods/return_all_video_files.rb'

module MultimediaParadise

class AllVideos < ::MultimediaParadise::Base # === MultimediaParadise::AllVideos.new

  include MultimediaParadise

  # ========================================================================= #
  # === NAMESPACE
  # ========================================================================= #
  NAMESPACE = inspect

  # ========================================================================= #
  # === initialize
  # ========================================================================= #
  def initialize(
      i           = nil,
      run_already = true
    )
    reset
    run if run_already
  end

  # ========================================================================= #
  # === reset
  # ========================================================================= #
  def reset
    super()
    @array_unknown_tags = []
  end

  # ========================================================================= #
  # === report_all_known_videofiles
  # ========================================================================= #
  def report_all_known_videofiles
    e; opnn; e 'All known (registered) videofiles are:'; e
    _ = return_all_video_files # This method is defined in shared.rb.
    _.each_pair {|key, value|
      title = value['title']
      if title.nil? # Add the unknown tag.
        title = swarn('<unknown>')
        @array_unknown_tags << key
      end
      e simp(('  %03d' % key).to_s)+' is '+sfancy(title)+'.' 
    }; e
  end

  # ========================================================================= #
  # === consider_reporting_unknown_tags
  # ========================================================================= #
  def consider_reporting_unknown_tags
    unless @array_unknown_tags.empty?
      opnn; e 'These entries are empty right now.'
      pp @array_unknown_tags
    end
  end

  # ========================================================================= #
  # === opnn
  # ========================================================================= #
  def opnn
    super(NAMESPACE) if show_namespace?
  end

  # ========================================================================= #
  # === show_namespace?
  # ========================================================================= #
  def show_namespace?
    false
  end

  # ========================================================================= #
  # === run                                                         (run tag)
  # ========================================================================= #
  def run
    report_all_known_videofiles
    consider_reporting_unknown_tags
  end

end; end

if __FILE__ == $PROGRAM_NAME
  MultimediaParadise::AllVideos.new
end # allvideos