#!/usr/bin/ruby -w
# Encoding: UTF-8
# frozen_string_literal: true
# =========================================================================== #
# === MultimediaParadise::Video::CheckNumbers
#
# The sole purpose of this class is to check that the numbers will match.
# It will NOT correct anything.
#
# Usage example:
#   MultimediaParadise::Video::CheckNumbers.new
# =========================================================================== #
# require 'multimedia_paradise/video/check_numbers.rb'
# =========================================================================== #
require 'multimedia_paradise/base/base.rb'

module MultimediaParadise

module Video

class CheckNumbers < ::MultimediaParadise::Base # === MultimediaParadise::Video::CheckNumbers.new

  # ========================================================================= #
  # === NAMESPACE
  # ========================================================================= #
  NAMESPACE = inspect

  # ========================================================================= #
  # === DATASET_CONTAINING_ALL_VIDEOS
  # ========================================================================= #
  _ = FILE_VIDEO_COLLECTION
  if File.exist? _
    begin
      DATASET_CONTAINING_ALL_VIDEOS = YAML.load_file(FILE_VIDEO_COLLECTION)
    rescue Psych::SyntaxError => error
      pp error
      DATASET_CONTAINING_ALL_VIDEOS = {}
    end
  else
    DATASET_CONTAINING_ALL_VIDEOS = {} # Just a safeguard.
  end

  # ========================================================================= #
  # === initialize
  # ========================================================================= #
  def initialize(
      check_this_entity = Dir.pwd,
      run_already       = true
    )
    reset
    if block_given?
      yielded = yield
      case yielded
      when :do_not_use_opn
        @do_use_opn = false
      end 
    end
    set_check_this_entity(check_this_entity)
    run if run_already
  end

  # ========================================================================= #
  # === reset                                                     (reset tag)
  # ========================================================================= #
  def reset
    super()
    @do_use_opn = true
  end

  # ========================================================================= #
  # === set_check_this_entity
  #
  # The "entity" can be either a directory or a file.
  # ========================================================================= #
  def set_check_this_entity(i = '')
    i = i.first if i.is_a? Array
    i = i.to_s.dup
    if File.directory? i
      i << '/' unless i.end_with? '/'
    end
    @check_this_entity = i
  end

  # ========================================================================= #
  # === check_this_entity?
  # ========================================================================= #
  def check_this_entity?
    @check_this_entity
  end; alias entity? check_this_entity? # === entity?

  # ========================================================================= #
  # === select_only_video_files_from
  # ========================================================================= #
  def select_only_video_files_from(array)
    array.select {|entry| is_video_file? entry }
  end

  # ========================================================================= #
  # === handle_given_input
  # ========================================================================= #
  def handle_given_input
    _ = entity?
    if File.directory? _
      # ======================================================================= #
      # If it is a directory, we obtain all files.
      # ======================================================================= #
      these_video_files = select_only_video_files_from(Dir[_+'*'])
      # ======================================================================= #
      # Sanitize them via File.basename() next.
      # ======================================================================= #
      these_video_files.map! {|entry| File.basename(entry) }
      # ======================================================================= #
      # Sort them:
      # ======================================================================= #
      these_video_files = these_video_files.sort_by {|entry|
        first_number_part = entry.split('_').first.to_i
        first_number_part
      }
      these_video_files.each {|entry|
        check_whether_this_file_is_included(entry)
      }
      opnne 'All '+simp(these_video_files.size.to_s)+' videos are correct '\
            'from the names alone, at the least.'
    elsif File.file? _
      check_whether_this_file_is_included(_)
    else
      opnn; e 'No idea how to proceed with '+simp(_)+'.'
      unless File.exist? _
        opnn; e 'The input does not exist. Perhaps you have'
        opnn; e 'misspelled the name?'
      end
    end
  end

  # ========================================================================= #
  # === dataset?
  # ========================================================================= #
  def dataset?
    DATASET_CONTAINING_ALL_VIDEOS
  end

  # ========================================================================= #
  # === check_whether_this_file_is_included
  # ========================================================================= #
  def check_whether_this_file_is_included(i)
    # ======================================================================= #
    # Chop off '/' if they are part of the input.
    # ======================================================================= #
    i = File.basename(i) if i.include? '/'
    # Sanitize the input.
    if i.include? '_VHS-RIP'
      i.sub!(/_VHS-RIP/,'')
    end
    if i.include? '_' # <- Only these are assumed to be correct.
      dataset = dataset?
      splitted = i.split('_')
      position = splitted.first.to_i
      if dataset.has_key? position
        title    = dataset[position]['title']
        compact_title = title.delete(' ')
        n_pad    = 45
        if i.delete('_').include? compact_title
          opnn; e 'Yes, '+sfancy(i.ljust(n_pad))+' is included. The position '\
                  'is '+simp(position.to_s.rjust(3))+'.'
        else # Here when the input was NOT found.
          opnn; e ' No, '+sfancy(i.ljust(n_pad))+' is '+swarn('NOT')+
                ' included. The position is '+simp(position.to_s.rjust(3))+'.'
          e '---'
          pp i
          pp compact_title
          e '---'
          exit
        end
      else
        opnne swarn('The position ')+sfancy(position)+swarn(' was not found in '\
              'the dataset.')+' This is the input `'+simp(i)+'`.'
        opnne 'Perhaps it can be deleted.'
        exit # Easier to exit here, then fix the problem. 
      end
    end
  end

  # ========================================================================= #
  # === opnne
  # ========================================================================= #
  def opnne(i)
    opnn; e i
  end

  # ========================================================================= #
  # === opnn
  # ========================================================================= #
  def opnn
    super(NAMESPACE) if @do_use_opn
  end

  # ========================================================================= #
  # === run                                                         (run tag)
  # ========================================================================= #
  def run
    handle_given_input
  end

  # ========================================================================= #
  # === MultimediaParadise::Video::CheckNumbers[]
  # ========================================================================= #
  def self.[](i = '')
    new(i)
  end

end; end; end

if __FILE__ == $PROGRAM_NAME
  MultimediaParadise::Video::CheckNumbers.new(ARGV) { :do_not_use_opn }
end # check_numbers