#!/usr/bin/ruby -w
# Encoding: UTF-8
# frozen_string_literal: true
# =========================================================================== #
# === MultimediaParadise::Columbo
#
# This class will play my custom, local Columbo-files.
#
# Usage example:
#
#   MultimediaParadise::Columbo.new(ARGV)
#
# =========================================================================== #
# require 'multimedia_paradise/video/columbo/columbo.rb'
# MultimediaParadise::Columbo.new(ARGV)
# =========================================================================== #
require 'multimedia_paradise/base/base.rb'

module MultimediaParadise

class Columbo < ::MultimediaParadise::Base

  # ========================================================================= #
  # === initialize
  # ========================================================================= #
  def initialize(
      commandline_arguments = nil,
      run_already           = true
    )
    reset
    set_commandline_arguments(
      commandline_arguments
    )
    if block_given?
      yielded = yield
      # ===================================================================== #
      # === Handle Hashes next
      # ===================================================================== #
      if yielded.is_a? Hash
        # =================================================================== #
        # === :use_this_base_directory
        # =================================================================== #
        if yielded.has_key? :use_this_base_directory
          set_use_this_base_directory(yielded[:use_this_base_directory])
        end
        # =================================================================== #
        # === :play_n_files
        # =================================================================== #
        if yielded.has_key? :play_n_files
          set_use_this_base_directory(yielded[:play_n_files])
        end
      end
    end
    run if run_already
  end

  # ========================================================================= #
  # === reset                                                     (reset tag)
  # ========================================================================= #
  def reset
    super()
    # ======================================================================= #
    # === @use_this_base_directory
    # ======================================================================= #
    @use_this_base_directory = :default
    # ======================================================================= #
    # === @play_n_files
    # ======================================================================= #
    @play_n_files = nil
  end

  # ========================================================================= #
  # === set_play_n_files
  # ========================================================================= #
  def set_play_n_files(i)
    if i.is_a? Array
      i = i.first
    end
    i = i.to_i
    @play_n_files = i
  end

  # ========================================================================= #
  # === set_use_this_base_directory
  # ========================================================================= #
  def set_use_this_base_directory(i)
    @use_this_base_directory = i
  end

  # ========================================================================= #
  # === run                                                         (run tag)
  # ========================================================================= #
  def run
    _ = first_argument?
    if _ and _.is_a?(String) and (_ =~ /\d{1,3}/)
      set_play_n_files(_)
    end
    case @use_this_base_directory
    # ======================================================================= #
    # === :default
    # ======================================================================= #
    when :default
      @use_this_base_directory = '/home/x/video/'
    end
    # ========================================================================= #
    # First we must obtain all columbo files.
    # ========================================================================= #
    all_columbo_files = Dir[@use_this_base_directory+'/Columbo*/**/**'].shuffle
    if @play_n_files and @play_n_files.is_a? Numeric
      all_columbo_files = all_columbo_files[0 .. (@play_n_files - 1)]
    end
    e 'We will next play these '+all_columbo_files.size.to_s+' video files:'
    all_columbo_files.each {|file|
      play_this_file(file)
    }
  end

  # ========================================================================= #
  # === play_this_file
  # ========================================================================= #
  def play_this_file(file)
    if file.include?("'")
      file = '"'+file+'"'
    end
    esystem("mplayer -vo x11 #{file}")
  end

  # ========================================================================= #
  # === MultimediaParadise::Columbo[]
  # ========================================================================= #
  def self.[](i = '')
    new(i)
  end

end; end

if __FILE__ == $PROGRAM_NAME
  MultimediaParadise::Columbo.new(ARGV)
end # columbo 15