#!/usr/bin/ruby -w
# Encoding: UTF-8
# frozen_string_literal: true
# =========================================================================== #
# === MultimediaParadise::CopyRandomVideo
#
# This class will copy a random video file from my collection to the
# current working directory.
#
# Usage example:
#
#   MultimediaParadise::CopyRandomVideo.new(ARGV)
#
# =========================================================================== #
# require 'multimedia_paradise/video/copy_random_video.rb'
# MultimediaParadise::CopyRandomVideo.new(ARGV)
# =========================================================================== #
require 'multimedia_paradise/base/base.rb'

module MultimediaParadise

class CopyRandomVideo < Base # === MultimediaParadise::CopyRandomVideo

  # ========================================================================= #
  # === initialize
  # ========================================================================= #
  def initialize(
      i           = ARGV,
      run_already = true
    )
    reset
    set_commandline_arguments(i)
    run if run_already
  end

  # ========================================================================= #
  # === reset                                                     (reset tag)
  # ========================================================================= #
  def reset
    super()
    infer_the_namespace
  end

  # ========================================================================= #
  # === obtain_all_videos_from_my_local_video_collection
  # ========================================================================= #
  def obtain_all_videos_from_my_local_video_collection
    @all_files = select_only_video_files_from(
      Dir['/home/x/video/**/**']
    ).sort
  end

  # ========================================================================= #
  # === copy_this_selected_video_file
  # ========================================================================= #
  def copy_this_selected_video_file
    opnn; e 'Next copying the video file '+sfile(@random_video)
    opnn; e 'to the current directory (at '+sdir(return_pwd)+').'
    copy_file(@random_video, return_pwd)
  end

  # ========================================================================= #
  # === select_a_random_video
  # ========================================================================= #
  def select_a_random_video
    @random_video = @all_files.sample
  end

  # ========================================================================= #
  # === run                                                         (run tag)
  # ========================================================================= #
  def run
    obtain_all_videos_from_my_local_video_collection
    select_a_random_video
    copy_this_selected_video_file
  end

  # ========================================================================= #
  # === MultimediaParadise::CopyRandomVideo[]
  # ========================================================================= #
  def self.[](i = '')
    new(i)
  end

end

# =========================================================================== #
# === MultimediaParadise.copy_random_video
# =========================================================================== #
def self.copy_random_video(i)
  MultimediaParadise::CopyRandomVideo.new(i)
end

end

if __FILE__ == $PROGRAM_NAME
  MultimediaParadise::CopyRandomVideo.new(ARGV)
end # copyrandomvideo