#!/usr/bin/ruby -w
# Encoding: UTF-8
# frozen_string_literal: true
# =========================================================================== #
# === MultimediaParadise::FfmpegOptions
#
# This little class just translates a Ffmpeg option into the
# verbose format for now. This may be more convenient to use
# than the shorter variants.
# =========================================================================== #
require 'multimedia_paradise/base/base.rb'

module MultimediaParadise

class FfmpegOptions <  Base

  # ========================================================================= #
  # Next, try to load up the file video_filter_aliases.yml
  #   bl /Users/x/DATA/VIDEO/YAML/video_filter_aliases.yml
  # ========================================================================= #
  _ = MY_DATA_DIR+'VIDEO/YAML/video_filter_aliases.yml' # bl $DATA/VIDEO/YAML/video*fil*

  # ========================================================================= #
  # === VIDEO_FILTER_ALIASES
  # ========================================================================= #
  if File.exist? _
    begin
      VIDEO_FILTER_ALIASES = YAML.load_file(_)
    rescue Psych::SyntaxError => error
      VIDEO_FILTER_ALIASES = {}
      puts 'Constant VIDEO_FILTER_ALIASES can not be loaded.'
      pp error
    end
  elsif File.exist? (_ = MY_DATA_DIR+'VIDEO/YAML/video_filter_aliases.yml') # Hardcoded attempt here.
    VIDEO_FILTER_ALIASES = YAML.load_file(_)
  else
    VIDEO_FILTER_ALIASES = {}
  end

  # ========================================================================= #
  # === REVERTED_VIDEO_FILTER_ALIASES
  # ========================================================================= #
  REVERTED_VIDEO_FILTER_ALIASES = VIDEO_FILTER_ALIASES.invert

  # ========================================================================= #
  # === DEFAULT_STRING
  # ========================================================================= #
  DEFAULT_STRING = 'ffmpeg -i jokes.flv -ab 56 -ar 22050 -b 500 -s 320×240 jokes.mpg'

  # ========================================================================= #
  # === PADDING
  # ========================================================================= #
  PADDING = '  '

  # ========================================================================= #
  # === initialize
  # ========================================================================= #
  def initialize(
      i           = DEFAULT_STRING,
      run_already = true
    )
    reset
    set_string(i)
    run if run_already
  end

  # ========================================================================= #
  # === reset
  # ========================================================================= #
  def reset
    super()
  end

  # ========================================================================= #
  # === set_keys
  # ========================================================================= #
  def set_keys
    @keys = REVERTED_VIDEO_FILTER_ALIASES
  end

  # ========================================================================= #
  # === set_string
  # ========================================================================= #
  def set_string(i)
    i = i.join(' ') if i.is_a? Array
    if i.empty?
      e 'Assigning default value to the string, as the passed string'+
        'is '+sfancy('empty')+'.'
      i = DEFAULT_STRING
    else
      case i
      when 'help' # Feedback help about codecs, then exit.
        system 'ffmpeg -codecs'
        exit
      end
    end
    i = i.strip
    @string = i
  end

  # ========================================================================= #
  # === sanitize_string
  # ========================================================================= #
  def sanitize_string
    _ = ''.dup
    @string.split(' ').each { |x|
      if x.include? '-'
        if @keys.include? x[1..-1]
          _ << '-'+@keys[x[1..-1]]
        else
          _ << x
        end
      else
        _ << x
      end
      _ << ' '
    }
    @result = _
  end

  # ========================================================================= #
  # === report_input
  # ========================================================================= #
  def report_input
    e PADDING+@string
  end

  # ========================================================================= #
  # === report_result
  # ========================================================================= #
  def report_result
    e PADDING+@result
  end

  # ========================================================================= #
  # === run_everything
  # ========================================================================= #
  def run_everything
    set_keys
    sanitize_string
    report_input
    report_result
  end

  # ========================================================================= #
  # === run
  # ========================================================================= #
  def run
    run_everything
  end

end; end

if __FILE__ == $PROGRAM_NAME
  MultimediaParadise::FfmpegOptions.new(ARGV) 
end # ffmpeg1 ffmpeg -i jokes.flv -ab 56 -ar 22050 -b 500 -s 320×240 jokes.mpg