#!/usr/bin/ruby -w
# Encoding: UTF-8
# frozen_string_literal: true
# =========================================================================== #
# === MultimediaParadise::ImdbRating
#
# This class will display the imdb-rating for the given video file (the
# passed-in argument) on the commandline. Thus, it tells the user the
# rating of any given video (if this video is registered in IMDB).
#
# Usage example in ruby code:
#
#   MultimediaParadise::ImdbRating.new(ARGV)
#
# Usage example from the commandline:
#
#   imdbrating Naked Gun
#
# =========================================================================== #
# require 'multimedia_paradise/video/imdb_rating.rb'
# =========================================================================== #
require 'multimedia_paradise/base/base.rb'

module MultimediaParadise

class ImdbRating < Base # === MultimediaParadise::ImdbRating

  require 'open-uri'

  begin
    require 'save_file'
  rescue LoadError; end

  begin
    require 'cyberweb/requires/remove_html.rb'
  rescue LoadError; end

  # ========================================================================= #
  # === NAMESPACE
  # ========================================================================= #
  NAMESPACE = inspect

  # ========================================================================= #
  # === USE_THIS_REGEX
  #
  # The Rating-specific String may look like this:
  #
  #   Rating: 7,6/10 · ‎160,162 votes
  #
  # ========================================================================= #
  # See: https://rubular.com/r/wO46isDDw1R3NM
  # ========================================================================= #
  USE_THIS_REGEX = 
      /Rating\s+(\d{1,1}\.\d{1,1})\/\d{1,2}/

  # ========================================================================= #
  # === initialize
  # ========================================================================= #
  def initialize(
      commandline_arguments = nil,
      run_already           = true
    )
    reset
    set_commandline_arguments(
      commandline_arguments
    )
    run if run_already
  end

  # ========================================================================= #
  # === reset                                                     (reset tag)
  # ========================================================================= #
  def reset
    super()
    # ======================================================================= #
    # === @namespace
    # ======================================================================= #
    @namespace = NAMESPACE
  end

  # ========================================================================= #
  # === return_google_string_for_this_URL
  #
  # The task for this method is to return the String that can be used
  # for a google-search.
  # ========================================================================= #
  def return_google_string_for_this_URL(this_URL)
    this_URL = this_URL.join(' ') if this_URL.is_a? Array
    this_URL = this_URL.to_s.tr(' ','+')
    result = 'https://www.google.com/search?q='+this_URL.to_s+'&pws=0&gl=us&source=hp&ei=rBemYaygJd6HjLsPgqC4uAg&iflsig=ALs-wAMAAAAAYaYlvDplQlqKL-PSwzmrsOGF0IaDWwHi&ved=0ahUKEwispvb1icD0AhXeA2MBHQIQDocQ4dUDCAk&uact=5&oq=imdb+naked+gun&gs_lcp=Cgdnd3Mtd2l6EAMyBQgAEIAEOgsILhCABBDHARDRAzoLCC4QgAQQxwEQowI6BQguEIAEOgsILhCABBDHARCvAVCUBliFHmD5H2gBcAB4AYABsAKIAYgSkgEIMTYuMS4yLjGYAQCgAQGwAQA&sclient=gws-wiz'
    return result.dup
  end

  # ========================================================================= #
  # === obtain_google_search_result
  #
  # This method will obtain the google page.
  # ========================================================================= #
  def obtain_google_search_result
    # modified_input = '%22'+original_input.map {|entry|
    #   entry.capitalize
    # }.join('+')+'%22'
    simple_input = commandline_arguments?.map {|entry| entry.capitalize }
    _ = return_google_string_for_this_URL(simple_input)
    #'https://www.google.com/search?client=firefox-b-e&q='+
    #    'https://www.google.com/search?as_q='+
    #    modified_input+
    #    '&as_epq=&as_oq=&as_eq=&as_nlo=&as_nhi=&lr=&cr=&as_qdr=all&as_sitesearch=&as_occt=any&safe=active&as_filetype=&tbs='
    #_ = _.dup
    e
    e sfancy(_)
    e
    dataset = URI.open(_).read # Encoding is now ISO-8859-1.
    dataset = dataset.encode('UTF-8')
    # dataset = dataset.force_encoding('UTF-8')
    scanned_results = ::Cyberweb.remove_html(dataset).scan(USE_THIS_REGEX)
    e 'The rating for `'+steelblue(commandline_arguments?.join(' ').to_s)+
      '` is: '+
      steelblue(scanned_results.flatten.first.to_f.to_s)
  end

  # ========================================================================= #
  # === run                                                         (run tag)
  # ========================================================================= #
  def run
    obtain_google_search_result
  end

  # ========================================================================= #
  # === MultimediaParadise::ImdbRating[]
  # ========================================================================= #
  def self.[](i = '')
    new(i)
  end

end; end

if __FILE__ == $PROGRAM_NAME
  MultimediaParadise::ImdbRating.new(ARGV)
end # imdbrating
    # imdbrating Naked Gun