#!/usr/bin/ruby -w
# Encoding: UTF-8
# frozen_string_literal: true
# =========================================================================== #
# === MultimediaParadise::PlayRandomFile
#
# This class can be used to play a random file from the current working
# directory.
#
# Usage example:
#
#   MultimediaParadise::PlayRandomFile.new(ARGV)
#
# =========================================================================== #
# require 'multimedia_paradise/video/play_random_file.rb'
# =========================================================================== #
require 'multimedia_paradise/base/base.rb'

module MultimediaParadise

class PlayRandomFile < Base # === MultimediaParadise::PlayRandomFile

  require 'multimedia_paradise/video/random_video.rb'

  # ========================================================================= #
  # === initialize
  # ========================================================================= #
  def initialize(
      i           = nil,
      run_already = true
    )
    reset
    set_input(i)
    run if run_already
  end

  # ========================================================================= #
  # === reset                                                     (reset tag)
  # ========================================================================= #
  def reset
    super()
    # ======================================================================= #
    # === @from_this_directory
    # ======================================================================= #
    @from_this_directory = return_pwd
  end

  # ========================================================================= #
  # === set_input
  # ========================================================================= #
  def set_input(i = '')
    i = i.first if i.is_a? Array
    i = i.to_s.dup
    @input = i
  end

  # ========================================================================= #
  # === input?
  # ========================================================================= #
  def input?
    @input
  end

  # ========================================================================= #
  # === run                                                         (run tag)
  # ========================================================================= #
  def run
    MultimediaParadise::Video::RandomVideo.new(@from_this_directory)
  end

  # ========================================================================= #
  # === []
  # ========================================================================= #
  def self.[](i = '')
    new(i)
  end

end

# =========================================================================== #
# === MultimediaParadise.play_random_file
# =========================================================================== #
def self.play_random_file(i)
  MultimediaParadise::PlayRandomFile.new(i)
end

end

if __FILE__ == $PROGRAM_NAME
  MultimediaParadise::PlayRandomFile.new(ARGV)
end # playrandomfile