#!/usr/bin/ruby -w
# Encoding: UTF-8
# frozen_string_literal: true
# =========================================================================== #
# === MultimediaParadise::Video::PlayRandomRealvideo
#
# This small class will play a random video from the
# "realvideo/" subdirectory, specified by the constant
# REALVIDS.
#
# Usage example:
#   MultimediaParadise::Video::PlayRandomRealvideo.new
# =========================================================================== #
# require 'multimedia_paradise/video/play_random_realvideo.rb'
# =========================================================================== #
require 'multimedia_paradise/base/base.rb'
require 'multimedia_paradise/video/random_video.rb'

module MultimediaParadise

module Video

class PlayRandomRealvideo < ::MultimediaParadise::Base # === MultimediaParadise::Video::PlayRandomRealvideo.new

  # ========================================================================= #
  # === initialize
  # ========================================================================= #
  def initialize(
      i           = nil,
      run_already = true
    )
    reset
    set_input(i)
    run if run_already
  end

  # ========================================================================= #
  # === reset                                                     (reset tag)
  # ========================================================================= #
  def reset
    super()
  end

  # ========================================================================= #
  # === set_input
  # ========================================================================= #
  def set_input(i = '')
    i = i.first if i.is_a? Array
    i = i.to_s.dup
    @input = i
  end

  # ========================================================================= #
  # === input?
  # ========================================================================= #
  def input?
    @input
  end

  # ========================================================================= #
  # === run                                                         (run tag)
  # ========================================================================= #
  def run
    MultimediaParadise::Video::RandomVideo.new(REALVIDS)
  end

  # ========================================================================= #
  # === []
  # ========================================================================= #
  def self.[](i = '')
    new(i)
  end

end; end

# =========================================================================== #
# === MultimediaParadise.play_random_realvideo
# =========================================================================== #
def self.play_random_realvideo
  ::MultimediaParadise::Video::PlayRandomRealvideo.new
end

end

if __FILE__ == $PROGRAM_NAME
  MultimediaParadise::Video::PlayRandomRealvideo.new(ARGV)
end # play_random_realvideo