#!/usr/bin/ruby -w
# Encoding: UTF-8
# frozen_string_literal: true
# =========================================================================== #
# === MultimediaParadise::Video::RegisteredVideoFile
#
# This class will determine whether a video file is registered or whether
# it is not. Currently the main modus of operation is to input a number.
#
# Usage example:
#   MultimediaParadise::Video::RegisteredVideoFile.new
# =========================================================================== #
# require 'multimedia_paradise/video/registered_video_file.rb'
# =========================================================================== #
require 'multimedia_paradise/base/base.rb'

module MultimediaParadise

module Video

class RegisteredVideoFile < ::MultimediaParadise::Base # MultimediaParadise::Video::RegisteredVideoFile.new

  # ========================================================================= #
  # === NAMESPACE
  # ========================================================================= #
  NAMESPACE = inspect

  # ========================================================================= #
  # === initialize
  # ========================================================================= #
  def initialize(
      i           = nil,
      run_already = true
    )
    reset
    set_original_input(i)
    run if run_already
  end

  # ========================================================================= #
  # === reset                                                     (reset tag)
  # ========================================================================= #
  def reset
    super()
    # ======================================================================= #
    # === @namespace
    # ======================================================================= #
    @namespace = NAMESPACE
  end

  # ========================================================================= #
  # === set_original_input
  #
  # This must not be modified at ALL.
  # ========================================================================= #
  def set_original_input(i = nil)
    i = i.first if i.is_a? Array
    if i.nil?
      opnn; e 'Please provide a file name to this class.'
      exit
    end
    i = i.to_s.dup
    @original_input = i
  end

  # ========================================================================= #
  # === original_input?
  # ========================================================================= #
  def original_input?
    @original_input
  end

  # ========================================================================= #
  # === input_without_extension?
  # ========================================================================= #
  def input_without_extension?
    extname = File.extname(original_input?)
    _ = original_input?.gsub(/#{extname}$/,'')
    if _.include? '_'
      _ = _.split('_').first
    end
    return _
  end

  # ========================================================================= #
  # === check_for_the_position_first
  # ========================================================================= #
  def check_for_the_position_first
    this_position = input_without_extension?.to_i # Get the input position first.
    msg_to_the_user = 'Checking for position '+simp(this_position.to_s)+' first.'
    _ = return_video_collection_dataset
    this_result = _[this_position]
    if this_result
      title = this_result['title']
      msg_to_the_user << ' The file name should be '+sfancy(title)+'.'
    else
      msg_to_the_user << ' That position appears to be vacant.'+N
      msg_to_the_user << 'It could still exist on another position, '
                         'but it may also be obsolete.'
    end
    e msg_to_the_user
  end

  # ========================================================================= #
  # === return_video_collection_dataset
  # ========================================================================= #
  def return_video_collection_dataset
    YAML.load_file(
      MultimediaParadise.file_video_collection
    )
  end

  # ========================================================================= #
  # === run                                                         (run tag)
  # ========================================================================= #
  def run
    check_for_the_position_first
  end

  # ========================================================================= #
  # === MultimediaParadise::Video::RegisteredVideoFile[]
  # ========================================================================= #
  def self.[](i = '')
    self.new
  end

end; end; end

if __FILE__ == $PROGRAM_NAME
  MultimediaParadise::Video::RegisteredVideoFile.new(ARGV)
end # registered_video_file?