#!/usr/bin/ruby -w
# Encoding: UTF-8
# frozen_string_literal: true
# =========================================================================== #
# === MultimediaParadise::Video::RenameVideoFile
#
# This class will accept input, which shall be the name of an existing
# video file. It will then try to find out the number that it should
# have, and rename it accordingly.
# 
# Usage examples:
#
#   MultimediaParadise::Video::RenameVideoFile.new(ARGV)
#
# =========================================================================== #
require 'multimedia_paradise/base/base.rb'
require 'multimedia_paradise/video/find_video/find_video.rb'

module MultimediaParadise

module Video

class RenameVideoFile < ::MultimediaParadise::Base # === MultimediaParadise::Video::RenameVideoFile

  # ========================================================================= #
  # === initialize
  # ========================================================================= #
  def initialize(
      i, run_already = true
    )
    reset
    register_sigint
    set_file(i)
    run if run_already
  end

  # ========================================================================= #
  # === reset
  # ========================================================================= #
  def reset
    super()
    @search_term = nil
  end

  # ========================================================================= #
  # === set_file
  # ========================================================================= #
  def set_file(i)
    i = i.first if i.is_a? Array
    i = i.to_s
    @file = i
    determine_extension_name
  end

  # ========================================================================= #
  # === determine_extension_name
  # ========================================================================= #
  def determine_extension_name
    i = @file
    i = File.extname(i)
    i.prepend '.' unless i.start_with? '.'
    @extension_name = i
  end

  # ========================================================================= #
  # === extension_name?
  # ========================================================================= #
  def extension_name?
    @extension_name
  end

  # ========================================================================= #
  # === sanitize_search_term
  #
  # This method is ultimately not needed, but I wanted to just
  # chop down the input file to make it smaller.
  # ========================================================================= #
  def sanitize_search_term(i = @search_term)
    if i
      i.downcase!
      i.gsub!(/\.x264/,'') if i.include? '.x264'
      i.gsub!(/\.extended/,'') if i.include? '.extended'
      i.gsub!(/\.remastered/,'') if i.include? '.remastered'
      i.gsub!(/\.2000\.720p\.brrip\.bokutox\.yify/,'')
      # ===================================================================== #
      # Chop off file Extensions.
      # ===================================================================== #
      i.gsub!(/\.mp4$/,'') if i.end_with? '.mp4'
      @search_term = i
    end
  end

  # ========================================================================= #
  # === search_term?
  # ========================================================================= #
  def search_term?
    @search_term
  end

  # ========================================================================= #
  # === set_search_term
  # ========================================================================= #
  def set_search_term(i = @file)
    i = i.to_s.dup
    @search_term = i
    sanitize_search_term
  end

  # ========================================================================= #
  # === file?
  # ========================================================================= #
  def file?
    @file
  end; alias old_filename? file? # === old_filename?

  # ========================================================================= #
  # === try_to_find_corresponding_video_file
  # ========================================================================= #
  def try_to_find_corresponding_video_file
    results = MultimediaParadise::FindVideo[search_term?]
    unless results.empty?
      new_file_name = results.first.first
      position = results.first.last
      opn; e 'We found a result, thus we will rename.'
      new_file_name = position+'_'+new_file_name+extension_name?
      opn; e 'The new file name will be:'
      rename_file(old_filename?, new_file_name)
      _ = realvids_directory?
      if File.directory?(_) and !File.exist?(_+File.basename(new_file_name))
        target = _+File.basename(new_file_name)
        opn; e 'Also copying to '+sfile(target)+' next.'
        copy_file(new_file_name, target)
      end
    end
  end

  # ========================================================================= #
  # === run
  # ========================================================================= #
  def run
    set_search_term
    try_to_find_corresponding_video_file
  end

end; end; end

if __FILE__ == $PROGRAM_NAME
  MultimediaParadise::Video::RenameVideoFile.new(ARGV)
end # renamevideo