#!/usr/bin/ruby -w
# Encoding: UTF-8
# frozen_string_literal: true
# =========================================================================== #
# === MultimediaParadise::Video::ReportLocalVideos
#
# This class will report how many videos are under the Realvids
# subdirectory, if that directory exists.
#
# It will also report how many videos are registered in the
# video-yaml file.
#
# Usage example:
#
#   MultimediaParadise::Video::ReportLocalVideos.new
#
# =========================================================================== #
# require 'multimedia_paradise/video/report_local_videos.rb'
# =========================================================================== #
require 'multimedia_paradise/base/base.rb'

module MultimediaParadise

module Video

class ReportLocalVideos < ::MultimediaParadise::Base # === MultimediaParadise::Video::ReportLocalVideos

  # ========================================================================= #
  # === NAMESPACE
  # ========================================================================= #
  NAMESPACE = inspect

  # ========================================================================= #
  # === initialize
  # ========================================================================= #
  def initialize(
      commandline_arguments = nil,
      run_already           = true
    )
    reset
    set_commandline_arguments(
      commandline_arguments
    )
    run if run_already
  end

  # ========================================================================= #
  # === reset                                                     (reset tag)
  # ========================================================================= #
  def reset
    super()
    # ======================================================================= #
    # === @namespace
    # ======================================================================= #
    @namespace = NAMESPACE
  end

  # ========================================================================= #
  # === run                                                         (run tag)
  # ========================================================================= #
  def run
    report_how_many_video_files_are_under_the_realvids_subdirectory
    report_how_many_videos_are_registered_in_the_main_video_yaml_file
  end

  # ========================================================================= #
  # === report_how_many_video_files_are_under_the_realvids_subdirectory
  # ========================================================================= #
  def report_how_many_video_files_are_under_the_realvids_subdirectory
    _ = directory_to_realvids?
    if File.directory? _
      # ===================================================================== #
      # Select all video-files next:
      # ===================================================================== #
      all_video_files = Dir[_+'**'].select {|entry| is_a_video_file?(entry) }
      e rev+'There are a total of '+sfancy(all_video_files.size.to_s)+' videos '\
        'available in the directory `'+
        sdir(_)+'`.'
      e
    end
  end

  # ========================================================================= #
  # === report_how_many_videos_are_registered_in_the_main_video_yaml_file
  # ========================================================================= #
  def report_how_many_videos_are_registered_in_the_main_video_yaml_file
    _ = ::MultimediaParadise.file_video_collection?
    if File.exist? _
      dataset = YAML.load_file(_)
      e rev+'A total of '+royalblue(dataset.keys.size.to_s)+' videos are '\
        'registered in the file '+sfile(_)+'.'
    end
  end

  # ========================================================================= #
  # === MultimediaParadise::Video::ReportLocalVideos[]
  # ========================================================================= #
  def self.[](i = '')
    new(i)
  end

end; end; end

if __FILE__ == $PROGRAM_NAME
  MultimediaParadise::Video::ReportLocalVideos.new(ARGV)
end # reportlocalvideos