#!/usr/bin/ruby -w
# Encoding: UTF-8
# frozen_string_literal: true
# =========================================================================== #
# === MultimediaParadise::Resolution
#
# This class will tell us the resolution of a video file.
#
# Currently we use ffmpeg to obtain the resolution.
# =========================================================================== #
# require 'resolution'
# =========================================================================== #
require 'multimedia_paradise/base/base.rb'

module MultimediaParadise

module Video

class Resolution < ::MultimediaParadise::Base # === MultimediaParadise::Video::Resolution

  # ========================================================================= #
  # === initialize
  # ========================================================================= #
  def initialize(
      i           = nil,
      run_already = true
    )
    reset
    set_input(i)
    run if run_already
  end

  # ========================================================================= #
  # === reset                                                     (reset tag)
  # ========================================================================= #
  def reset
    super()
    @dataset = nil
  end

  # ========================================================================= #
  # === set_input
  # ========================================================================= #
  def set_input(i = '')
    i = i.first if i.is_a? Array
    i = i.to_s.dup if i
    @input = i
  end

  # ========================================================================= #
  # === input?
  # ========================================================================= #
  def input?
    @input
  end

  # ========================================================================= #
  # === obtain_dataset_via_ffmpeg
  # ========================================================================= #
  def obtain_dataset_via_ffmpeg(
      input = input?
    )
    if input.nil?
      e 'Please supply a valid argument, a video or audio file.'
    else
      _ = 'ffmpeg -i '+input+' 2>&1'
      @dataset = `#{_}`
    end
  end

  # ========================================================================= #
  # === dataset?
  # ========================================================================= #
  def dataset?
    @dataset
  end; alias data? dataset? # === data?

  # ========================================================================= #
  # === filter_dataset
  # ========================================================================= #
  def filter_dataset
    @dataset = @dataset.split(N)
    @dataset.select! {|entry| entry.include? 'Stream ' }
    @dataset.select! {|entry| entry.include?('x') and entry.include?('Video:') }
  end

  # ========================================================================= #
  # === grab_resolution_from_the_dataset
  # ========================================================================= #
  def grab_resolution_from_the_dataset
    _ = dataset?.first.strip
    use_this_regex = / (\d+x\d+) / # See: http://rubular.com/r/qvuyHxVDdg
    _ =~ use_this_regex
    set_resolution $1.to_s.dup
  end

  # ========================================================================= #
  # === set_resolution
  # ========================================================================= #
  def set_resolution(i = nil)
    @resolution = i
    guess_width_and_height if @resolution
  end

  # ========================================================================= #
  # === guess_width_and_height
  # ========================================================================= #
  def guess_width_and_height(i = @resolution)
    if i.include? 'x'
      splitted = i.split('x')
      @width  = splitted.first
      @height = splitted.last
    end
  end

  # ========================================================================= #
  # === width?
  # ========================================================================= #
  def width?
    @width
  end

  # ========================================================================= #
  # === height?
  # ========================================================================= #
  def height?
    @height
  end

  # ========================================================================= #
  # === resolution?
  # ========================================================================= #
  def resolution?
    @resolution
  end; alias res? resolution? # === res?

  # ========================================================================= #
  # === report_resolution
  # ========================================================================= #
  def report_resolution
    e 'The resolution of '+sfile(input?)+' is: '
    e '  '+simp(res?)
  end; alias report report_resolution # === report

  # ========================================================================= #
  # === run                                                         (run tag)
  # ========================================================================= #
  def run
    obtain_dataset_via_ffmpeg
    filter_dataset
    grab_resolution_from_the_dataset
  end

end; end; end

if __FILE__ == $PROGRAM_NAME
  _ = MultimediaParadise::Video::Resolution.new(ARGV)
  _.report_resolution
end # resolution? /Depot/Video/Columbo_Season_03_1973/Columbo_21_Double_Exposure_1973_TV.avi