#!/usr/bin/ruby -w
# Encoding: UTF-8
# frozen_string_literal: true
# =========================================================================== #
# === MultimediaParadise::Simpsons
#
# This class can be used to play a simpson movie.
#
# Usage example:
#
#   MultimediaParadise::Simpsons.new(ARGV)
#
# =========================================================================== #
# require 'multimedia_paradise/video/simpsons.rb'
# =========================================================================== #
require 'multimedia_paradise/base/base.rb'

module MultimediaParadise

class Simpsons < MultimediaParadise::Base # === MultimediaParadise::Simpsons.new

  # ========================================================================= #
  # === NAMESPACE
  # ========================================================================= #
  NAMESPACE = inspect

  # ========================================================================= #
  # === DEFAULT_PLAY_N_TIMES
  # ========================================================================= #
  DEFAULT_PLAY_N_TIMES = 3

  # ========================================================================= #
  # === MPLAYER_COMMAND_TO_USE
  # ========================================================================= #
  MPLAYER_COMMAND_TO_USE = 'mplayer -vo x11 '

  # ========================================================================= #
  # === HOME_DIRECTORY_OF_USER_X
  # ========================================================================= #
  HOME_DIRECTORY_OF_USER_X = '/home/x/'

  # ========================================================================= #
  # === BASE_DIR
  #
  # This constant purposely has no trailing '/' because we need to
  # match ALL entries that begin with the substring "Simpsons" -
  # including files AND directories.
  # ========================================================================= #
  BASE_DIR = HOME_DIRECTORY_OF_USER_X+'video/Cartoons/Simpsons'

  # ========================================================================= #
  # === AVAILABLE_SIMPSONS_VIDEOS
  # ========================================================================= #
  AVAILABLE_SIMPSONS_VIDEOS = Dir["#{BASE_DIR}**/**"]

  # ========================================================================= #
  # === initialize
  # ========================================================================= #
  def initialize(
      play_n_videos =  3,
      run_already   = true
    )
    reset
    set_play_n_videos(
      play_n_videos
    )
    run if run_already
  end

  # ========================================================================= #
  # === reset                                                     (reset tag)
  # ========================================================================= #
  def reset
    super()
    # ======================================================================= #
    # === @namespace
    # ======================================================================= #
    @namespace = NAMESPACE
  end

  # ========================================================================= #
  # === set_play_n_videos
  # ========================================================================= #
  def set_play_n_videos(
      i = DEFAULT_PLAY_N_TIMES
    )
    i = i.first if i.is_a? Array # Only use the first one.
    i = DEFAULT_PLAY_N_TIMES if i.nil?
    opnn; e "We will play #{sfancy(i.to_s)} simpson videos."
    i = i.to_i # Must always be a Number.
    @play_n_videos = i
  end

  # ========================================================================= #
  # === play_n_videos?
  # ========================================================================= #
  def play_n_videos?
    @play_n_videos
  end; alias run_n_times? play_n_videos? # === run_n_times?

  # ========================================================================= #
  # === run                                                         (run tag)
  # ========================================================================= #
  def run
    run_n_times?.times {
      use_this_video = AVAILABLE_SIMPSONS_VIDEOS.sample
      if use_this_video.nil?
        e 'No simpsons-video could be found.'
      else
        _ = '"'+use_this_video+'"'.dup
        _.prepend(MPLAYER_COMMAND_TO_USE)
        e _
        esystem(_)
      end
    }
  end

  # ========================================================================= #
  # === MultimediaParadise::Simpsons[]
  # ========================================================================= #
  def self.[](i = '')
    new(i)
  end

end; end

if __FILE__ == $PROGRAM_NAME
  MultimediaParadise::Simpsons.new(ARGV)
end # simpsons 5