#!/usr/bin/ruby -w
# Encoding: UTF-8
# frozen_string_literal: true
# =========================================================================== #
# === MultimediaParadise::Video::SplitThisVideo
#
# This class can be used to split a video file, such as an .avi or
# .mp4 file. The first argument must be the filename of the video;
# the second argument must be the split-position (as a timestamp).
#
# Commandline usage follows:
#
#   splitthisvideo foobar.avi  01:24
#
# Usage example:
#
#   MultimediaParadise::Video::SplitThisVideo.new(ARGV)
#
# =========================================================================== #
# require 'multimedia_paradise/video/split_this_video.rb'
# MultimediaParadise::Video::SplitThisVideo.new(ARGV)
# =========================================================================== #
require 'multimedia_paradise/base/base.rb'
  
module MultimediaParadise

module Video

class SplitThisVideo < ::MultimediaParadise::Base # === MultimediaParadise::Video::SplitThisVideo

  require 'multimedia_paradise/toplevel_methods/cut_from_to.rb'
  require 'multimedia_paradise/audio/file_duration/file_duration.rb'

  # ========================================================================= #
  # === initialize
  # ========================================================================= #
  def initialize(
      name_of_the_videofile = nil,
      split_position        = nil,
      run_already           = true
    )
    reset
    if name_of_the_videofile.is_a?(Array) and split_position.nil?
      split_position = name_of_the_videofile.pop
    end
    set_name_of_the_videofile(name_of_the_videofile)
    set_split_position(split_position)
    run if run_already
  end

  # ========================================================================= #
  # === reset                                                     (reset tag)
  # ========================================================================= #
  def reset
    super()
    infer_the_namespace
  end

  # ========================================================================= #
  # === set_name_of_the_videofile
  # ========================================================================= #
  def set_name_of_the_videofile(i)
    if i.is_a? Array
      i = i.first
    end
    @name_of_the_videofile = i
  end

  # ========================================================================= #
  # === set_split_position
  # ========================================================================= #
  def set_split_position(i)
    @split_position = i
  end

  # ========================================================================= #
  # === split_position?
  # ========================================================================= #
  def split_position?
    @split_position
  end; alias point? split_position? # === point?

  # ========================================================================= #
  # === name_of_the_videofile?
  # ========================================================================= #
  def name_of_the_videofile?
    @name_of_the_videofile
  end

  # ========================================================================= #
  # === duration?
  # ========================================================================= #
  def duration?(i)
    return MultimediaParadise.duration?(i)
  end

  # ========================================================================= #
  # === run                                                         (run tag)
  # ========================================================================= #
  def run
    _ = name_of_the_videofile?
    duration = point?.to_s.dup
    unless duration.include? '-'
      duration << '-'+
                  seconds_to_time_format( # Convert n seconds into the 24 hour time format.
                    duration?(_).to_s
                  )
    end
    _result = MultimediaParadise.cut_from_to(
      duration,
      this_file: _
    )
  end

  # ========================================================================= #
  # === MultimediaParadise::Video::SplitThisVideo[]
  # ========================================================================= #
  def self.[](i = ARGV)
    new(i)
  end

end; end; end

if __FILE__ == $PROGRAM_NAME
  MultimediaParadise::Video::SplitThisVideo.new(ARGV)
end # splitthisvideo