#!/usr/bin/ruby -w
# Encoding: UTF-8
# frozen_string_literal: true
# =========================================================================== #
# === MultimediaParadise::StoreAvailableVideoFiles
#
# This class will store which video-files are available on the local 
# filesystem.
#
# It will store this information into a .md file.
#
# This then allows us to do restorations or at the least notify if
# we are missing something.
#
# Usage example:
#
#   MultimediaParadise::StoreAvailableVideoFiles.new(ARGV)
#
# =========================================================================== #
# require 'multimedia_paradise/video/store_available_video_files.rb'
# MultimediaParadise::StoreAvailableVideoFiles
# =========================================================================== #
require 'multimedia_paradise/base/base.rb'

module MultimediaParadise

class StoreAvailableVideoFiles < Base # === MultimediaParadise::StoreAvailableVideoFiles

  # ========================================================================= #
  # === STORE_INTO_THIS_FILE
  # ========================================================================= #
  STORE_INTO_THIS_FILE = "#{MultimediaParadise.log_dir?}video_files.md"

  # ========================================================================= #
  # === initialize
  # ========================================================================= #
  def initialize(
      i           = nil,
      run_already = true
    )
    reset
    set_input(i)
    run if run_already
  end

  # ========================================================================= #
  # === reset                                                     (reset tag)
  # ========================================================================= #
  def reset
    super()
    infer_the_namespace
  end

  # ========================================================================= #
  # === set_input
  # ========================================================================= #
  def set_input(i = '')
    i = i.first if i.is_a? Array
    i = i.to_s.dup
    @input = i
  end

  # ========================================================================= #
  # === input?
  # ========================================================================= #
  def input?
    @input
  end

  # ========================================================================= #
  # === filter_select_all_videos
  # ========================================================================= #
  def filter_select_all_videos
    @all_available_videos.select! {|entry| is_video_file?(entry)}
  end

  # ========================================================================= #
  # === gather_all_available_videos
  #
  # This method will collect all available video-files.
  # ========================================================================= #
  def gather_all_available_videos(
      from_this_directory = DEPOT_VIDEO
    )
    @all_available_videos = Dir[from_this_directory+'**/**']
  end

  # ========================================================================= #
  # === store_results
  #
  # This method will store the results into a .md file.
  # ========================================================================= #
  def store_results
    ensure_that_the_output_directory_exists
    # ======================================================================= #
    # We aggregate the String that has to be saved into the variable
    # what. This will also include the file-size of the video file
    # at hand.
    # ======================================================================= #
    what = @all_available_videos.map {|entry|
      "#{entry} # file size: #{File.size(entry)}"
    }
    what = what.join(N)
    into = STORE_INTO_THIS_FILE
    opnn; e "Now storing into the file at `#{sfile(into)}`."
    write_what_into(what, into)
    # ======================================================================= #
    # Also store silently for my home system:
    # ======================================================================= #
    _ = '/home/x/data/personal/video/'
    if File.directory? _
      into = _+File.basename(into)
      opnn; e "Storing into the file at `#{sfile(into)}` as well."
      write_what_into(what, into)
    end
  end

  # ========================================================================= #
  # === run                                                         (run tag)
  # ========================================================================= #
  def run
    gather_all_available_videos
    filter_select_all_videos
    store_results
  end

  # ========================================================================= #
  # === MultimediaParadise::StoreAvailableVideoFiles[]
  # ========================================================================= #
  def self.[](i = '')
    new(i)
  end

end; end

if __FILE__ == $PROGRAM_NAME
  MultimediaParadise::StoreAvailableVideoFiles.new(ARGV)
end # storeavailablevideofiles