#!/usr/bin/ruby -w
# Encoding: UTF-8
# frozen_string_literal: true
# =========================================================================== #
# === MultimediaParadise::VideoMetadata
#
# We will dump the information that ffmpeg gives us into a new file,
# a file called "metadata.txt", into the directory from where we
# invoke this script.
#
# A constant defines the name for that metadata file by default.
# =========================================================================== #
require 'multimedia_paradise/base/base.rb'

module MultimediaParadise

class VideoMetadata < ::MultimediaParadise::Base # === MultimediaParadise::VideoMetadata

  # ========================================================================= #
  # === MultimediaParadise::VideoMetadata::NAMESPACE
  # ========================================================================= #
  NAMESPACE = inspect

  # ========================================================================= #
  # === STORE_INTO_THIS_FILE
  #
  # Where to store the information about a video file.
  # ========================================================================= #
  STORE_INTO_THIS_FILE = 'metadata.txt'

  # ========================================================================= #
  # === initialize
  # ========================================================================= #
  def initialize(
      this_file   = ARGV,
      run_already = true
    )
    reset
    set_main_file(this_file)
    run if run_already
  end

  # ========================================================================= #
  # === reset
  # ========================================================================= #
  def reset
    super()
    # ======================================================================= #
    # === @namespace
    # ======================================================================= #
    @namespace = NAMESPACE
  end

  # ========================================================================= #
  # === delete_old_file_if_it_exists
  # ========================================================================= #
  def delete_old_file_if_it_exists(i = store_where?)
    if File.exist? i # This must then be an old file.
      File.delete(i)
    end
  end

  # ========================================================================= #
  # === prepend_filename
  #
  # This method will prepend the filename into the file.
  #
  # It will use SaveFile for storing said content.
  # ========================================================================= #
  def prepend_filename
    dataset = read_from_main_file
    dataset.prepend 'Filename: '+@main_file+N
    write_what_into(dataset, store_where?)
  end

  # ========================================================================= #
  # === read_from_main_file
  # ========================================================================= #
  def read_from_main_file
    File.read(store_where?)
  end

  # ========================================================================= #
  # === show_content
  #
  # Display the content to the user.
  # ========================================================================= #
  def show_content
    opnn; e 'We have gathered this information about the file '+
             sfile(input_file?)+':'
    e simp(read_from_main_file.chomp)
  end

  # ========================================================================= #
  # === store_where?
  # ========================================================================= #
  def store_where?
    STORE_INTO_THIS_FILE
  end

  # ========================================================================= #
  # === set_main_file
  # ========================================================================= #
  def set_main_file(i)
    i = i.join(' ').strip if i.is_a? Array
    @main_file = i
  end

  # ========================================================================= #
  # === main_file?
  # ========================================================================= #
  def main_file?
    @main_file
  end; alias input_file? main_file? # === input_file?

  # ========================================================================= #
  # === run
  # ========================================================================= #
  def run
    delete_old_file_if_it_exists
    _ = 'ffmpeg -i '+@main_file+' -f ffmetadata '+store_where?
    esystem _
    prepend_filename
    show_content
    opnn; e 'Stored into file `'+sfile(store_where?)+'`.'
  end

end; end

if __FILE__ == $PROGRAM_NAME
  MultimediaParadise::VideoMetadata.new(ARGV)
end # metadata /Depot/Video/Realvids/255_RogerandMe.avi